@@ -11,10 +11,11 @@ public class FormatSource {
 		else
 			sourceUrlString=args[0];
 		if (sourceUrlString.indexOf(':')==-1) sourceUrlString="file:"+sourceUrlString;
+		MicrosoftTagTypes.register();
 		PHPTagTypes.register();
 		MasonTagTypes.register();
 		Source source=new Source(new URL(sourceUrlString));
 
-		source.getSourceFormatter().setIndentString("  ").setTidyTags(true).writeTo(new OutputStreamWriter(System.out));
+		new SourceFormatter(source).setIndentString("  ").setTidyTags(true).writeTo(new OutputStreamWriter(System.out));
 	}
 }
