@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.6
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -31,11 +31,14 @@ import java.io.*;
  * and ends at the end of the last attribute.
  * <p>
  * The attributes in this list are a representation of those found in the source document and are not modifiable.
- * The {@link AttributesOutputSegment} class provides the means to add, delete or modify attributes and
- * their values for inclusion in an {@link OutputDocument}.
+ * The {@link OutputDocument#replace(Attributes, Map)} and {@link OutputDocument#replace(Attributes, boolean convertNamesToLowerCase)} methods
+ * provide the means to add, delete or modify attributes and their values in an {@link OutputDocument}.
+ * <p>
+ * As of version 2.4, any {@linkplain TagType#isServerTag() server tags} encountered inside the attributes area of a non-server tag
+ * no longer interfere with the parsing of the attributes.
  * <p>
  * If too many syntax errors are encountered while parsing a start tag's attributes, the parser rejects the entire start tag
- * and generates a {@linkplain Source#setLogWriter(Writer) log} entry.
+ * and generates a {@linkplain Source#getLogger() log} entry.
  * The threshold for the number of errors allowed can be set using the {@link #setDefaultMaxErrorCount(int)} static method.
  * <p>
  * Obtained using the {@link StartTag#getAttributes()} method, or explicitly using the {@link Source#parseAttributes(int pos, int maxEnd)} method.
@@ -52,6 +55,8 @@ import java.io.*;
  */
 public final class Attributes extends SequentialListSegment {
 	private final LinkedList attributeList; // never null
+	
+	final boolean containsServerTagOutsideOfAttributeValue;
 
 	// parsing states:
 	private static final int AFTER_TAG_NAME=0;
@@ -64,9 +69,10 @@ public final class Attributes extends SequentialListSegment {
 
 	private static int defaultMaxErrorCount=2; // defines maximum number of minor errors that can be encountered in attributes before entire start tag is rejected.
 
-	private Attributes(final Source source, final int begin, final int end, final LinkedList attributeList) {
+	private Attributes(final Source source, final int begin, final int end, final LinkedList attributeList, final boolean containsServerTagOutsideOfAttributeValue) {
 		super(source,begin,end);
 		this.attributeList=attributeList;
+		this.containsServerTagOutsideOfAttributeValue=containsServerTagOutsideOfAttributeValue;
 	}
 
 	/** called from StartTagType.parseAttributes(Source, int startTagBegin, String tagName) */
@@ -87,11 +93,6 @@ public final class Attributes extends SequentialListSegment {
 	/**
 	 * Any &lt; character found within the start tag is treated as though it is part of the attribute
 	 * list, which is consistent with the way IE treats it.
-	 * In some cases an invalid character results in the entire start tag being rejected.
-	 * This may seem strict, but we have to be able to distinguish whether any
-	 * particular < found in the source is actually the start of a tag or not.
-	 * Being too lenient with attributes means more chance of false positives, which in turn
-	 * means surrounding tags may be ignored.
 	 * @param logBegin  the position of the beginning of the object being searched (for logging)
 	 * @param attributesBegin  the position of the beginning of the attribute list, or -1 if it should be calculated automatically from logBegin.
 	 * @param maxEnd  the position at which the attributes must end if a terminating character is not found, or -1 if no maximum.
@@ -108,6 +109,7 @@ public final class Attributes extends SequentialListSegment {
 		}
 		int attributesEnd=attributesBegin;
 		final LinkedList attributeList=new LinkedList();
+		boolean containsServerTagOutsideOfAttributeValue=false;
 		final ParseText parseText=source.getParseText();
 		int i=attributesBegin;
 		char quote=' ';
@@ -120,6 +122,22 @@ public final class Attributes extends SequentialListSegment {
 			while (!isTerminatingCharacter) {
 				if (i==maxEnd || startTagType.atEndOfAttributes(source,i,isClosingSlashIgnored)) isTerminatingCharacter=true;
 				final char ch=parseText.charAt(i);
+				// First check if there is a server tag in this position:
+				if (ch=='<') {
+					final Tag interlopingTag=Tag.getTagAt(source,i,true); // search for server tags only
+					if (interlopingTag!=null) {
+						// There is a server tag in this position. Skip over it:
+						if (state==START_VALUE) {
+							currentBegin=i;
+							quote=' ';
+							state=IN_VALUE;
+						}
+						i=attributesEnd=interlopingTag.end;
+						if (state!=IN_VALUE) containsServerTagOutsideOfAttributeValue=true;
+						continue;
+					}
+				}
+				// There is no server tag in this position. Now we can parse the attributes:
 				switch (state) {
 					case IN_VALUE:
 						if (isTerminatingCharacter || ch==quote || (quote==' ' && isWhiteSpace(ch))) {
@@ -130,7 +148,7 @@ public final class Attributes extends SequentialListSegment {
 							} else {
 								if (isTerminatingCharacter) {
 									if (i==maxEnd) {
-										if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"terminated in the middle of a quoted attribute value",i);
+										if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"terminated in the middle of a quoted attribute value",i);
 										if (reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 										valueSegment=new Segment(source,currentBegin,i);
 										valueSegmentIncludingQuotes=new Segment(source,currentBegin-1,i); // this is missing the end quote
@@ -148,7 +166,7 @@ public final class Attributes extends SequentialListSegment {
 							attributesEnd=valueSegmentIncludingQuotes.getEnd();
 							state=BETWEEN_ATTRIBUTES;
 						} else if (ch=='<' && quote==' ') {
-							if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character in unquoted attribute value",i);
+							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character in unquoted attribute value",i);
 							return null;
 						}
 						break;
@@ -164,13 +182,12 @@ public final class Attributes extends SequentialListSegment {
 							}
 						} else if (!Tag.isXMLNameChar(ch)) {
 							// invalid character detected in attribute name.
-							// only reject whole start tag if it is a < character or if the error count is exceeded.
 							if (ch=='<') {
-								if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character in attribute name",i);
+								if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character in attribute name",i);
 								return null;
 							}
 							if (isInvalidEmptyElementTag(startTagType,source,i,logType,tagName,logBegin)) break;
-							if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"contains attribute name with invalid character",i);
+							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"contains attribute name with invalid character",i);
 							if (reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 						}
 						break;
@@ -185,6 +202,9 @@ public final class Attributes extends SequentialListSegment {
 							i--; // want to reparse the same character again, so decrement i.  Note we could instead just fall into the next case statement without a break, but such code is always discouraged.
 						} else if (ch=='=') {
 							state=START_VALUE;
+						} else if (ch=='<') {
+							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character after attribute name",i);
+							return null;
 						}
 						break;
 					case BETWEEN_ATTRIBUTES:
@@ -194,23 +214,23 @@ public final class Attributes extends SequentialListSegment {
 								quote=' ';
 							} else {
 								if (quote!=' ') {
-									if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"has missing whitespace after quoted attribute value",i);
-									// log this as an error but don't count it
+									if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"has missing whitespace after quoted attribute value",i);
+									// only count this as an error if there have already been other errors, otherwise allow unlimited errors of this type.
+									if (errorCount>0 && reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 								}
 								if (!Tag.isXMLNameStartChar(ch)) {
 									// invalid character detected as first character of attribute name.
-									// only reject whole start tag if it is a < character or if the error count is exceeded.
 									if (ch=='<') {
-										if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character",i);
+										if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character",i);
 										return null;
 									}
 									if (isInvalidEmptyElementTag(startTagType,source,i,logType,tagName,logBegin)) break;
 									if (startTagType==StartTagType.NORMAL && startTagType.atEndOfAttributes(source,i,false)) {
 										// This checks whether we've found the characters "/>" but it wasn't recognised as the closing delimiter because isClosingSlashIgnored is true.
-										if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"contains a '/' character before the closing '>', which is ignored because tags of this name cannot be empty-element tags");
+										if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"contains a '/' character before the closing '>', which is ignored because tags of this name cannot be empty-element tags");
 										break;
 									}
-									if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"contains attribute name with invalid first character",i);
+									if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"contains attribute name with invalid first character",i);
 									if (reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 								}
 								state=IN_NAME;
@@ -221,21 +241,23 @@ public final class Attributes extends SequentialListSegment {
 					case START_VALUE:
 						currentBegin=i;
 						if (isTerminatingCharacter) {
-							if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"has missing attribute value after '=' sign",i);
-							// log this as an error but don't count it
+							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"has missing attribute value after '=' sign",i);
+							// only count this as an error if there have already been other errors, otherwise allow unlimited errors of this type.
+							if (errorCount>0 && reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 							final Segment valueSegment=new Segment(source,i,i);
 							attributeList.add(new Attribute(source,key,nameSegment,valueSegment,valueSegment));
 							attributesEnd=i;
 							state=BETWEEN_ATTRIBUTES;
 							break;
 						}
-						if (isWhiteSpace(ch)) break; // just ignore whitespace after the '=' sign as nearly all browsers do.
-						if (ch=='<') {
-							if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character at start of attribuite value",i);
-							return null;
-						} else if (ch=='\'' || ch=='"') {
+						if (ch=='\'' || ch=='"') {
 							quote=ch;
 							currentBegin++;
+						} else if (isWhiteSpace(ch)) {
+							break; // just ignore whitespace after the '=' sign as nearly all browsers do.
+						} else if (ch=='<') {
+							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character at the start of an attribute value",i);
+							return null;
 						} else {
 							quote=' ';
 						}
@@ -245,7 +267,7 @@ public final class Attributes extends SequentialListSegment {
 						if (!isTerminatingCharacter) {
 							if (!isWhiteSpace(ch)) {
 								if (isInvalidEmptyElementTag(startTagType,source,i,logType,tagName,logBegin)) break;
-								if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"rejected because name contains invalid character",i);
+								if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because the name contains an invalid character",i);
 								return null;
 							}
 							state=BETWEEN_ATTRIBUTES;
@@ -254,23 +276,23 @@ public final class Attributes extends SequentialListSegment {
 				}
 				i++;
 			}
-			return new Attributes(source,attributesBegin,attributesEnd,attributeList); // used to end at i-1
+			return new Attributes(source,attributesBegin,attributesEnd,attributeList,containsServerTagOutsideOfAttributeValue);
 		} catch (IndexOutOfBoundsException ex) {
-			if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"rejected because it has no closing '>' character");
+			if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because it has no closing '>' character");
 			return null;
 		}
 	}
 
 	private static boolean reachedMaxErrorCount(final int errorCount, final Source source, final String logType, final String tagName, final int logBegin, final int maxErrorCount) {
 		if (errorCount<=maxErrorCount) return false;
-		if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"rejected because it contains too many errors");
+		if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because it contains too many errors");
 		return true;
 	}
 
 	private static boolean isInvalidEmptyElementTag(final StartTagType startTagType, final Source source, final int i, final String logType, final String tagName, final int logBegin) {
 		// This checks whether we've found the characters "/>" but it wasn't recognised as the closing delimiter because isClosingSlashIgnored is true.
 		if (startTagType!=StartTagType.NORMAL || !startTagType.atEndOfAttributes(source,i,false)) return false;
-		if (source.isLoggingEnabled()) log(source,logType,tagName,logBegin,"contains a '/' character before the closing '>', which is ignored because tags of this name cannot be empty-element tags");
+		if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"contains a '/' character before the closing '>', which is ignored because tags of this name cannot be empty-element tags");
 		return true;
 	}
 
@@ -397,7 +419,7 @@ public final class Attributes extends SequentialListSegment {
 		if (isEmpty()) {
 			sb.append("EMPTY");
 		} else {
-			sb.append('\n');
+			sb.append(Config.NewLine);
 			for (final Iterator i=listIterator(0); i.hasNext();) {
 				Attribute attribute=(Attribute)i.next();
 				sb.append("  ").append(attribute.getDebugInfo());
@@ -422,8 +444,7 @@ public final class Attributes extends SequentialListSegment {
 	 * In this case the entire start tag is rejected.
 	 * <p>
 	 * On the other hand, it is quite common for attributes to contain minor syntactical errors,
-	 * such as an invalid character in an attribute name, or a couple of special characters in
-	 * {@linkplain TagType#isServerTag() server tags} that otherwise contain only attributes.
+	 * such as an invalid character in an attribute name.
 	 * For this reason the parser allows a certain number of minor errors to occur while parsing an
 	 * attribute list before the entire start tag or attribute list is rejected.
 	 * This property indicates the number of minor errors allowed.
@@ -431,14 +452,13 @@ public final class Attributes extends SequentialListSegment {
 	 * Major syntactical errors cause the start tag or attribute list to be rejected immediately, regardless
 	 * of the maximum error count setting.
 	 * <p>
-	 * Some errors are considered too minor to count at all (ignorable), such as missing whitespace between the end
+	 * Some errors are considered too minor to count at all (ignorable), such as missing white space between the end
 	 * of a quoted attribute value and the start of the next attribute name.
 	 * <p>
 	 * The classification of particular syntax errors in attribute lists into major, minor, and ignorable is
 	 * not part of the specification and may change in future versions.
 	 * <p>
-	 * To track errors as they occur, use the {@link Source#setLogWriter(Writer writer)} method to set the
-	 * destination of the error log.
+	 * Errors are {@linkplain Source#getLogger() logged} as they occur.
 	 * <p>
 	 * The value of this property is set using the {@link #setDefaultMaxErrorCount(int)} method.
 	 *
@@ -480,19 +500,6 @@ public final class Attributes extends SequentialListSegment {
 		return stringWriter.toString();
 	}
 
-	/**
-	 * Returns this instance.
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as the <code>Attributes</code> class now implements
- 	 * the <code>List</code> interface, so the instance itself can be used instead.
-	 *
-	 * @return this instance.
-	 * @deprecated  Use the {@link Attributes} object itself instead.
-	 */
-	public List getList() {
-		return this;
-	}
-
 	/**
 	 * Outputs the contents of the specified {@linkplain #populateMap(Map,boolean) attributes map} as HTML attribute name/value pairs to the specified <code>Writer</code>.
 	 * <p>
@@ -522,10 +529,10 @@ public final class Attributes extends SequentialListSegment {
 	}
 	
 	private static void log(final Source source, final String part1, final CharSequence part2, final int begin, final String part3, final int pos) {
-		source.log(source.getRowColumnVector(pos).appendTo(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append(part1).append(' ').append(part2).append(" at ")).append(' ').append(part3).append(" at position ")).toString());
+		source.logger.info(source.getRowColumnVector(pos).appendTo(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append(part1).append(' ').append(part2).append(" at ")).append(' ').append(part3).append(" at position ")).toString());
 	}
 
 	private static void log(final Source source, final String part1, final CharSequence part2, final int begin, final String part3) {
-		source.log(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append(part1).append(' ').append(part2).append(" at ")).append(' ').append(part3).toString());
+		source.logger.info(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append(part1).append(' ').append(part2).append(" at ")).append(' ').append(part3).toString());
 	}
 }
