@@ -1,26 +1,27 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.6
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
 import java.util.*;
+import java.io.*;
 
 /**
  * Represents an HTML <a target="_blank" href="http://www.w3.org/TR/REC-html40/charset.html#entities">Character Reference</a>,
@@ -56,6 +57,8 @@ public abstract class CharacterReference extends Segment {
 	 */
 	static final int MAX_CODE_POINT=0x10FFFF;
 
+	static int MAX_ENTITY_REFERENCE_LENGTH; // set in CharacterEntityReference static class initialisation
+
 	/** The number of spaces used to simulate a tab when {@linkplain #encodeWithWhiteSpaceFormatting encoding with white space formatting}. */
 	private static final int TAB_LENGTH=4;
 
@@ -131,6 +134,18 @@ public abstract class CharacterReference extends Segment {
 		return appendEncode(new StringBuffer(unencodedText.length()*2),unencodedText,false).toString();
 	}
 
+	/**
+	 * Encodes the specified character into a character reference if {@linkplain #requiresEncoding(char) required}.
+	 * <p>
+	 * The encoding of the character follows the same rules as for each character in the {@link #encode(CharSequence unencodedText)} method.
+	 *
+	 * @param ch  the character to encode.
+	 * @return a character reference if appropriate, otherwise a string containing the original character.
+	 */
+	public static String encode(final char ch) {
+		return appendEncode(new StringBuffer(MAX_ENTITY_REFERENCE_LENGTH),ch).toString();
+	}
+
 	/**
 	 * {@linkplain #encode(CharSequence) Encodes} the specified text, preserving line breaks, tabs and spaces for rendering by converting them to markup.
 	 * <p>
@@ -153,7 +168,7 @@ public abstract class CharacterReference extends Segment {
 	 * There is no method provided to reverse this encoding.
 	 *
 	 * @param unencodedText  the text to encode.
-	 * @return the encoded string with whitespace formatting converted to markup.
+	 * @return the encoded string with white space formatting converted to markup.
 	 * @see #encode(CharSequence)
 	 */
 	public static String encodeWithWhiteSpaceFormatting(final CharSequence unencodedText) {
@@ -172,6 +187,9 @@ public abstract class CharacterReference extends Segment {
 	 * <a href="#Unterminated">Unterminated</a> character references are dealt with according to the rules for
 	 * text outside of attribute values in the {@linkplain Config#CurrentCompatibilityMode current compatibility mode}.
 	 * <p>
+	 * If the static {@link Config#ConvertNonBreakingSpaces} property is set to <code>true</code> (the default),
+	 * then all non-breaking space ({@link CharacterEntityReference#_nbsp &amp;nbsp;}) character entity references are converted to normal spaces.
+	 * <p>
 	 * Although character entity reference names are case sensitive, and in some cases differ from other entity references only by their case,
 	 * some browsers also recognise them in a case-insensitive way.
 	 * For this reason, all decoding methods in this library recognise character entity reference names even if they are in the wrong case.
@@ -181,7 +199,7 @@ public abstract class CharacterReference extends Segment {
 	 * @see #encode(CharSequence)
 	 */
 	public static String decode(final CharSequence encodedText) {
-		return decode(encodedText,false);
+		return decode(encodedText,false,Config.ConvertNonBreakingSpaces);
 	}
 
 	/**
@@ -194,6 +212,9 @@ public abstract class CharacterReference extends Segment {
 	 * value of the <code>insideAttributeValue</code> parameter and the
 	 * {@linkplain Config#CurrentCompatibilityMode current compatibility mode}.
 	 * <p>
+	 * If the static {@link Config#ConvertNonBreakingSpaces} property is set to <code>true</code> (the default),
+	 * then all non-breaking space ({@link CharacterEntityReference#_nbsp &amp;nbsp;}) character entity references are converted to normal spaces.
+	 * <p>
 	 * Although character entity reference names are case sensitive, and in some cases differ from other entity references only by their case,
 	 * some browsers also recognise them in a case-insensitive way.
 	 * For this reason, all decoding methods in this library recognise character entity reference names even if they are in the wrong case.
@@ -205,11 +226,16 @@ public abstract class CharacterReference extends Segment {
 	 * @see #encode(CharSequence)
 	 */
 	public static String decode(final CharSequence encodedText, final boolean insideAttributeValue) {
+		return decode(encodedText,insideAttributeValue,Config.ConvertNonBreakingSpaces);
+	}
+
+	private static String decode(final CharSequence encodedText, final boolean insideAttributeValue, final boolean convertNonBreakingSpaces) {
 		if (encodedText==null) return null;
-		final String encodedString=encodedText.toString();
-		final int pos=encodedString.indexOf('&');
-		if (pos==-1) return encodedString;
-		return appendDecode(new StringBuffer(encodedString.length()),encodedString,pos,insideAttributeValue).toString();
+		for (int i=0; i<encodedText.length(); i++) {
+			if (encodedText.charAt(i)=='&')
+				return appendDecode(new StringBuffer(encodedText.length()),encodedText,i,insideAttributeValue,convertNonBreakingSpaces).toString();
+		}
+		return encodedText.toString();
 	}
 
 	/**
@@ -220,13 +246,25 @@ public abstract class CharacterReference extends Segment {
 	 * The result is how the text would normally be rendered by a
 	 * <a target="_blank" href="http://www.w3.org/TR/html401/conform.html#didx-user_agent">user agent</a>,
 	 * assuming it does not contain any tags.
+	 * <p>
+	 * If the static {@link Config#ConvertNonBreakingSpaces} property is set to <code>true</code> (the default),
+	 * then all non-breaking space ({@link CharacterEntityReference#_nbsp &amp;nbsp;}) character entity references are converted to normal spaces.
+	 * <p>
+	 * <a href="#Unterminated">Unterminated</a> character references are dealt with according to the rules for
+	 * text outside of attribute values in the {@linkplain Config#CurrentCompatibilityMode current compatibility mode}.
+	 * See the discussion of the <code>insideAttributeValue</code> parameter of the {@link #decode(CharSequence, boolean insideAttributeValue)}
+	 * method for a more detailed explanation of this topic.
 	 *
 	 * @param text  the source text
 	 * @return the decoded text with collapsed white space.
 	 * @see FormControl#getPredefinedValues()
 	 */
 	public static String decodeCollapseWhiteSpace(final CharSequence text) {
-		return decode(appendCollapseWhiteSpace(new StringBuffer(text.length()),text));
+		return decodeCollapseWhiteSpace(text,Config.ConvertNonBreakingSpaces);
+	}
+
+	static String decodeCollapseWhiteSpace(final CharSequence text, final boolean convertNonBreakingSpaces) {
+		return decode(appendCollapseWhiteSpace(new StringBuffer(text.length()),text),false,convertNonBreakingSpaces);
 	}
 
 	/**
@@ -425,7 +463,7 @@ public abstract class CharacterReference extends Segment {
 	 * @see #decode(CharSequence)
 	 */
 	public static CharacterReference parse(final CharSequence characterReferenceText) {
-		return construct(new Source(characterReferenceText.toString()),0,Config.UnterminatedCharacterReferenceSettings.ACCEPT_ALL);
+		return construct(new Source(characterReferenceText),0,Config.UnterminatedCharacterReferenceSettings.ACCEPT_ALL);
 	}
 
 	/**
@@ -468,6 +506,48 @@ public abstract class CharacterReference extends Segment {
 		return ch>127 || (CharacterEntityReference.getName(ch)!=null && (ch!='\'' || Config.IsApostropheEncoded));
 	}
 
+	/**
+	 * Returns a filter <code>Writer</code> that {@linkplain #encode(CharSequence) encodes} all text before passing it through to the specified <code>Writer</code>.
+	 *
+	 * @param writer  the destination for the encoded text
+	 * @return a filter <code>Writer</code> that {@linkplain #encode(CharSequence) encodes} all text before passing it through to the specified <code>Writer</code>.
+	 * @see #encode(CharSequence unencodedText)
+	 */
+	public static Writer getEncodingFilterWriter(final Writer writer) {
+		return new EncodingFilterWriter(writer);
+	}
+
+	private static final class EncodingFilterWriter extends FilterWriter {
+		StringBuffer sb=new StringBuffer(MAX_ENTITY_REFERENCE_LENGTH);
+		public EncodingFilterWriter(final Writer writer) {
+			super(writer);
+		}
+		public void write(final char ch) throws IOException {
+			sb.setLength(0);
+			appendEncode(sb,ch);
+			if (sb.length()==1)
+				out.write(sb.charAt(0));
+			else
+				Util.appendTo(out,sb);
+		}
+		public void write(final int chInt) throws IOException {
+			write((char)chInt);
+		}
+		public void write(final char[] cbuf, final int off, final int len) throws IOException {
+			final int end=off+len;
+			for (int i=off; i<end; i++) write(cbuf[i]);
+		}
+		public void write(final String str, final int off, final int len) throws IOException {
+			final int end=off+len;
+			for (int i=off; i<end; i++) write(str.charAt(i));
+		}
+	}
+
+	private static StringBuffer appendEncode(final StringBuffer sb, char ch) {
+		if (appendEncodeCheckForWhiteSpaceFormatting(sb,ch,false)) return sb;
+		return sb.append(ch);
+	}
+
 	static StringBuffer appendEncode(final StringBuffer sb, CharSequence unencodedText, final boolean whiteSpaceFormatting) {
 		if (unencodedText==null) return sb;
 		int beginPos=0;
@@ -483,74 +563,96 @@ public abstract class CharacterReference extends Segment {
 		final boolean isApostropheEncoded=Config.IsApostropheEncoded;
 		for (int i=beginPos; i<endPos; i++) {
 			char ch=unencodedText.charAt(i);
-			final String characterEntityReferenceName=CharacterEntityReference.getName(ch);
-			if (characterEntityReferenceName!=null) {
-				if (ch=='\'') {
-					if (isApostropheEncoded)
-						sb.append("&#39;");
-					else
-						sb.append(ch);
-				} else {
-					CharacterEntityReference.appendCharacterReferenceString(sb,characterEntityReferenceName);
-				}
-			} else if (ch>127) {
-				appendDecimalCharacterReferenceString(sb,ch);
-			} else if (!(whiteSpaceFormatting && isWhiteSpace(ch))) {
-				sb.append(ch);
-			} else {
-				// whiteSpaceFormatting tries to simulate the formatting characters by converting them to markup
-				int spaceCount;
-				int nexti=i+1;
-				if (ch!=' ') {
-					if (ch!='\t') {
-						// must be line feed, carriage return or form feed, since zero-width space should have been processed as a character reference string
-						if (ch=='\r' && nexti<endPos && unencodedText.charAt(nexti)=='\n') i++; // process cr/lf pair as one line break
-						sb.append("<br />"); // add line break
-						continue;
-					} else {
-						spaceCount=TAB_LENGTH;
-					}
-				} else {
-					spaceCount=1;
-				}
-				while (nexti<endPos) {
-					ch=unencodedText.charAt(nexti);
-					if (ch==' ')
-						spaceCount+=1;
-					else if (ch=='\t')
-						spaceCount+=TAB_LENGTH;
-					else
-						break;
-					nexti++;
-				}
-				if (spaceCount==1) {
-					// handle the very common case of a single character to improve efficiency slightly
-					sb.append(' ');
+			if (appendEncodeCheckForWhiteSpaceFormatting(sb,ch,whiteSpaceFormatting)) continue;
+			// need to process white space
+			// whiteSpaceFormatting tries to simulate the formatting characters by converting them to markup
+			int spaceCount;
+			int nexti=i+1;
+			if (ch!=' ') {
+				if (ch!='\t') {
+					// must be line feed, carriage return or form feed, since zero-width space should have been processed as a character reference string
+					if (ch=='\r' && nexti<endPos && unencodedText.charAt(nexti)=='\n') i++; // process cr/lf pair as one line break
+					sb.append("<br />"); // add line break
 					continue;
+				} else {
+					spaceCount=TAB_LENGTH;
 				}
-				if (spaceCount%2==1) sb.append(' '); // fist character is a space if we have an odd number of spaces
-				while (spaceCount>=2) {
-					sb.append("&nbsp; "); // use alternating &nbsp; and spaces to keep original number of spaces
-					spaceCount-=2;
-				}
-				// note that the last character is never a nbsp, so that word wrapping won't result in a nbsp before the first character in a line
-				i=nexti-1; // minus 1 because top level for loop will add it again
+			} else {
+				spaceCount=1;
 			}
+			while (nexti<endPos) {
+				ch=unencodedText.charAt(nexti);
+				if (ch==' ')
+					spaceCount+=1;
+				else if (ch=='\t')
+					spaceCount+=TAB_LENGTH;
+				else
+					break;
+				nexti++;
+			}
+			if (spaceCount==1) {
+				// handle the very common case of a single character to improve efficiency slightly
+				sb.append(' ');
+				continue;
+			}
+			if (spaceCount%2==1) sb.append(' '); // fist character is a space if we have an odd number of spaces
+			while (spaceCount>=2) {
+				sb.append("&nbsp; "); // use alternating &nbsp; and spaces to keep original number of spaces
+				spaceCount-=2;
+			}
+			// note that the last character is never a nbsp, so that word wrapping won't result in a nbsp before the first character in a line
+			i=nexti-1; // minus 1 because top level for loop will add it again
 		}
 		return sb;
 	}
 
-	static CharacterReference findPreviousOrNext(final Source source, final int pos, final boolean previous) {
-		return findPreviousOrNext(source,pos,Config.UnterminatedCharacterReferenceSettings.ACCEPT_ALL,previous);
+	private static final boolean appendEncodeCheckForWhiteSpaceFormatting(final StringBuffer sb, char ch, final boolean whiteSpaceFormatting) {
+		final String characterEntityReferenceName=CharacterEntityReference.getName(ch);
+		if (characterEntityReferenceName!=null) {
+			if (ch=='\'') {
+				if (Config.IsApostropheEncoded)
+					sb.append("&#39;");
+				else
+					sb.append(ch);
+			} else {
+				CharacterEntityReference.appendCharacterReferenceString(sb,characterEntityReferenceName);
+			}
+		} else if (ch>127) {
+			appendDecimalCharacterReferenceString(sb,ch);
+		} else if (!(whiteSpaceFormatting && isWhiteSpace(ch))) {
+			sb.append(ch);
+		} else {
+			return false;
+		}
+		return true;
 	}
 
-	private static CharacterReference findPreviousOrNext(final Source source, int pos, final Config.UnterminatedCharacterReferenceSettings unterminatedCharacterReferenceSettings, final boolean previous) {
+	static CharacterReference findPrevious(final Source source, final int pos) {
+		return findPrevious(source,pos,Config.UnterminatedCharacterReferenceSettings.ACCEPT_ALL);
+	}
+
+	static CharacterReference findNext(final Source source, final int pos) {
+		return findNext(source,pos,Config.UnterminatedCharacterReferenceSettings.ACCEPT_ALL);
+	}
+
+	private static CharacterReference findPrevious(final Source source, int pos, final Config.UnterminatedCharacterReferenceSettings unterminatedCharacterReferenceSettings) {
 		final ParseText parseText=source.getParseText();
-		pos=previous ? parseText.lastIndexOf('&',pos) : parseText.indexOf('&',pos);
+		pos=parseText.lastIndexOf('&',pos);
 		while (pos!=-1) {
 			final CharacterReference characterReference=construct(source,pos,unterminatedCharacterReferenceSettings);
 			if (characterReference!=null) return characterReference;
-			pos=previous ? parseText.lastIndexOf('&',pos-1) : parseText.indexOf('&',pos+1);
+			pos=parseText.lastIndexOf('&',pos-1);
+		}
+		return null;
+	}
+
+	private static CharacterReference findNext(final Source source, int pos, final Config.UnterminatedCharacterReferenceSettings unterminatedCharacterReferenceSettings) {
+		final ParseText parseText=source.getParseText();
+		pos=parseText.indexOf('&',pos);
+		while (pos!=-1) {
+			final CharacterReference characterReference=construct(source,pos,unterminatedCharacterReferenceSettings);
+			if (characterReference!=null) return characterReference;
+			pos=parseText.indexOf('&',pos+1);
 		}
 		return null;
 	}
@@ -574,18 +676,19 @@ public abstract class CharacterReference extends Segment {
 		}
 	}
 
-	private static StringBuffer appendDecode(final StringBuffer sb, final String encodedString, int pos, final boolean insideAttributeValue) {
+	private static StringBuffer appendDecode(final StringBuffer sb, final CharSequence encodedText, int pos, final boolean insideAttributeValue, final boolean convertNonBreakingSpaces) {
 		final Config.UnterminatedCharacterReferenceSettings unterminatedCharacterReferenceSettings=Config.CurrentCompatibilityMode.getUnterminatedCharacterReferenceSettings(insideAttributeValue);
 		int lastEnd=0;
-		final Source source=new Source(encodedString);
+		final Source source=new Source(encodedText);
 		while (true) {
-			final CharacterReference characterReference=findPreviousOrNext(source,pos,unterminatedCharacterReferenceSettings,false);
+			final CharacterReference characterReference=findNext(source,pos,unterminatedCharacterReferenceSettings);
 			if (characterReference==null) break;
-			if (lastEnd!=characterReference.getBegin()) Util.appendTo(sb,encodedString,lastEnd,characterReference.getBegin());
-			sb.append((char)characterReference.codePoint);
+			if (lastEnd!=characterReference.getBegin()) Util.appendTo(sb,encodedText,lastEnd,characterReference.getBegin());
+			final char ch=characterReference.getChar();
+			sb.append((ch==CharacterEntityReference._nbsp && convertNonBreakingSpaces) ? ' ' : ch);
 			pos=lastEnd=characterReference.getEnd();
 		}
-		if (lastEnd!=encodedString.length()) Util.appendTo(sb,encodedString,lastEnd,encodedString.length());
+		if (lastEnd!=encodedText.length()) Util.appendTo(sb,encodedText,lastEnd,encodedText.length());
 		return sb;
 	}
 }
