@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.6
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -66,6 +66,43 @@ public final class Config {
 	 */
 	public static String ColumnValueFalse=null;
 
+	/**
+	 * Determines whether the {@link CharacterReference#decode(CharSequence)} and similar methods convert non-breaking space ({@link CharacterEntityReference#_nbsp &amp;nbsp;}) character entity references to normal spaces.
+	 * <p>
+	 * The default value is <code>true</code>.
+	 * <p>
+	 * When this property is set to <code>false</code>, non-breaking space ({@link CharacterEntityReference#_nbsp &amp;nbsp;})
+	 * character entity references are decoded as non-breaking space characters (U+00A0) instead of being converted to normal spaces (U+0020).
+	 * <p>
+	 * Note that the introduction of this property in version 2.6 has changed the default behaviour of the methods listed below,
+	 * which prior to version 2.6 always decoded {@link CharacterEntityReference#_nbsp &amp;nbsp;} as non-breaking space characters (U+00A0).
+	 * <p>
+	 * The new default behaviour of the library reflects the fact that non-breaking space character entity references are almost always used in HTML documents
+	 * as a <a target="_blank" href="http://en.wikipedia.org/wiki/Non-breaking_space#Use_as_non-collapsing_whitespace">non-collapsing white space</a> character.
+	 * Converting them to the correct character code U+00A0, which is represented by a visible character in many older character sets, was confusing to most users
+	 * who expected to see only normal spaces.
+	 * The most common example of this is its visualisation as the character <b>&aacute;</b> in the MS-DOS <a target="_blank" href="http://en.wikipedia.org/wiki/Code_page_437">CP437</a> character set.
+	 * <p>
+	 * The functionality of the following methods is affected:
+	 * <ul>
+	 *  <li>{@link CharacterReference#decode(CharSequence)}
+	 *  <li>{@link CharacterReference#decode(CharSequence, boolean insideAttributeValue)}
+	 *  <li>{@link CharacterReference#decodeCollapseWhiteSpace(CharSequence)}
+	 *  <li>{@link CharacterReference#reencode(CharSequence)}
+ 	 *  <li>{@link Attribute#getValue()}
+ 	 *  <li>{@link Attributes#getValue(String name)}
+	 *  <li>{@link Attributes#populateMap(Map, boolean convertNamesToLowerCase)}
+	 *  <li>{@link StartTag#getAttributeValue(String attributeName)}
+	 *  <li>{@link Element#getAttributeValue(String attributeName)}
+	 *  <li>{@link FormControl#getPredefinedValues()}
+	 *  <li>{@link OutputDocument#replace(Attributes, boolean convertNamesToLowerCase)}
+	 *  <li>{@link Renderer#getConvertNonBreakingSpaces()}
+	 *  <li>{@link TextExtractor#getConvertNonBreakingSpaces()}
+	 * </ul>
+	 */
+	public static boolean ConvertNonBreakingSpaces=true;
+
+
 	/**
 	 * Determines the currently active {@linkplain Config.CompatibilityMode compatibility mode}.
 	 * <p>
@@ -91,9 +128,48 @@ public final class Config {
 	 */
 	public static boolean IsApostropheEncoded=false;
 
+	/**
+	 * Determines the {@link LoggerProvider} that is used to create the default {@link Logger} object for each new {@link Source} object.
+	 * <p>
+	 * The {@link LoggerProvider} interface contains several predefined <code>LoggerProvider</code> instances which this property can be set to,
+	 * mostly representing wrappers to common logging frameworks.
+	 * <p>
+	 * The default value is <code>null</code>, which results in the auto-detection of the most appropriate logging mechanism according to the following algorithm:
+	 * <p>
+	 * <ol>
+	 *  <li>If the class <code>org.slf4j.impl.StaticLoggerBinder</code> is detected:
+	 *   <ul>
+	 *    <li>If the class <code>org.slf4j.impl.JDK14LoggerFactory</code> is detected, use {@link LoggerProvider#JAVA}.
+	 *    <li>If the class <code>org.slf4j.impl.Log4jLoggerFactory</code> is detected, use {@link LoggerProvider#LOG4J}.
+	 *    <li>If the class <code>org.slf4j.impl.JCLLoggerFactory</code> is NOT detected, use {@link LoggerProvider#SLF4J}.
+	 *   </ul>
+	 *  <li>If the class <code>org.apache.commons.logging.Log</code> is detected:
+	 *   <blockquote>
+	 *    Create an instance of it using the commons-logging <code>LogFactory</code> class.
+	 *     <ul>
+	 *      <li>If the created <code>Log</code> is of type <code>org.apache.commons.logging.impl.Jdk14Logger</code>, use {@link LoggerProvider#JAVA}.
+	 *      <li>If the created <code>Log</code> is of type <code>org.apache.commons.logging.impl.Log4JLogger</code>, use {@link LoggerProvider#LOG4J}.
+	 *      <li>otherwise, use {@link LoggerProvider#JCL}.
+	 *     </ul>
+	 *   </blockquote>
+	 *  <li>If the class <code>org.apache.log4j.Logger</code> is detected, use {@link LoggerProvider#LOG4J}.
+	 *  <li>otherwise, use {@link LoggerProvider#JAVA}.
+	 * </ol>
+	 *
+	 * @see Source#setLogger(Logger)
+	 */
+	public static LoggerProvider LoggerProvider=null;
+
+	/**
+	 * Determines the string used to represent a <a target="_blank" href="http://en.wikipedia.org/wiki/Newline">newline</a> in text output throughout the library.
+	 * <p>
+	 * The default value is the standard new line character sequence of the host platform, determined by <code>System.getProperty("line.separator")</code>.
+	 */
+	public static String NewLine=System.getProperty("line.separator");
+
 	/**
 	 * Used in Element.getChildElements.
-	 * Will only make this public if someone makes a convincing argument why you would ever need to include server tags in an element hierarchy.
+	 * Server elements containing markup should be included in the hierarchy, so consider making this option public in future.
 	 */ 
 	static final boolean IncludeServerTagsInElementHierarchy=false;
 
@@ -133,9 +209,9 @@ public final class Config {
 		}
 
 		public String toString() {
-			return "\n    Character entity reference: "+getDescription(characterEntityReferenceMaxCodePoint)
-						+"\n    Decimal character reference: "+getDescription(decimalCharacterReferenceMaxCodePoint)
-						+"\n    Haxadecimal character reference: "+getDescription(hexadecimalCharacterReferenceMaxCodePoint);
+			return Config.NewLine+"    Character entity reference: "+getDescription(characterEntityReferenceMaxCodePoint)
+						+Config.NewLine+"    Decimal character reference: "+getDescription(decimalCharacterReferenceMaxCodePoint)
+						+Config.NewLine+"    Haxadecimal character reference: "+getDescription(hexadecimalCharacterReferenceMaxCodePoint);
 		}
 
 		private String getDescription(final int codePoint) {
@@ -157,8 +233,7 @@ public final class Config {
 	 * the predefined instances as well as newly constructed instances.
 	 * Take note however that modifying the properties of the predefined instances has a global affect.
 	 * <p>
-	 * The currently active compatibility mode is stored in the static 
-	 * {@link Config#CurrentCompatibilityMode} property.
+	 * The currently active compatibility mode is stored in the static {@link Config#CurrentCompatibilityMode} property.
 	 * <p>
 	 */
 	public static final class CompatibilityMode {
@@ -439,10 +514,10 @@ public final class Config {
 		 */
 		public String getDebugInfo() {
 			return "Form field name case insensitive: "+formFieldNameCaseInsensitive
-				+"\nMaximum codepoints in unterminated character references:"
-				+"\n  Inside attribute values:"
+				+Config.NewLine+"Maximum codepoints in unterminated character references:"
+				+Config.NewLine+"  Inside attribute values:"
 				+unterminatedCharacterReferenceSettingsInsideAttributeValue
-				+"\n  Outside attribute values:"
+				+Config.NewLine+"  Outside attribute values:"
 				+unterminatedCharacterReferenceSettingsOutsideAttributeValue;
 		}
 	
