@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -191,7 +191,7 @@ import java.util.*;
  *         <a href="#ImplicitlyTerminated">implicitly terminated element</a> is created.
  *       </ul>
  *     </ul>
- *     Note that the syntactical indication of an {@linkplain StartTag#isEmptyElementTag() empty-element tag} in the start tag
+ *     Note that the syntactical indication of an {@linkplain StartTag#isSyntacticalEmptyElementTag() empty-element tag} in the start tag
  *     is ignored when determining the end of <a href="HTMLElements.html#HTMLElement">HTML elements</a>.
  *     See the documentation of the {@link #isEmptyElementTag()} method for more information.
  *    <li>
@@ -199,7 +199,7 @@ import java.util.*;
  *     recognised {@linkplain HTMLElementName HTML element names} (indicating a <a href="HTMLElements.html#NonHTMLElement">non-HTML element</a>):
  *     <ul>
  *      <li>
- *       If the start tag is an {@linkplain StartTag#isEmptyElementTag() empty-element tag},
+ *       If the start tag is {@linkplain StartTag#isSyntacticalEmptyElementTag() syntactically an empty-element tag},
  *       the parser does not conduct any search for an end tag and a <a href="#SingleTag">single tag element</a> is created.
  *      <li>
  *       Otherwise, section <a target="_blank" href="http://www.w3.org/TR/REC-xml#CleanAttrVals">3.1</a>
@@ -243,6 +243,8 @@ public final class Element extends Segment implements HTMLElementName {
 	private int depth=-1;
 	
 	static final Element NOT_CACHED=new Element();
+	
+	private static final boolean INCLUDE_INCORRECTLY_NESTED_CHILDREN_IN_HIERARCHY=true;
 
 	Element(final Source source, final StartTag startTag, final EndTag endTag) {
 		super(source, startTag.begin, endTag==null ? startTag.end : endTag.end);
@@ -316,7 +318,13 @@ public final class Element extends Segment implements HTMLElementName {
 						continue;
 					}
 					final Element childElement=childStartTag.getElement();
-					if (childElement.end>end && source.logger.isInfoEnabled()) source.logger.info("Child "+childElement.getDebugInfo()+" extends beyond end of parent "+getDebugInfo());
+					if (childElement.end>end) {
+						if (source.logger.isInfoEnabled()) source.logger.info("Child "+childElement.getDebugInfo()+" extends beyond end of parent "+getDebugInfo());
+						if (!INCLUDE_INCORRECTLY_NESTED_CHILDREN_IN_HIERARCHY) {
+							pos=childElement.end; 
+							continue; 
+						}
+					}
 					childElement.getChildElements(childDepth);
 					if (childElement.parentElement==Element.NOT_CACHED) { // make sure element was not added as a child of a descendent element (can happen with overlapping elements)
 						childElement.parentElement=this;
@@ -424,25 +432,17 @@ public final class Element extends Segment implements HTMLElementName {
 	/**
 	 * Indicates whether this element is an <a target="_blank" href="http://www.w3.org/TR/REC-xml#dt-eetag">empty-element tag</a>.
 	 * <p>
-	 * It is signified by an {@linkplain #isEmpty() empty} element with the characters "<code>/&gt;</code>" at the end of the
-	 * {@linkplain #getStartTag() start tag}.
+	 * This is equivalent to {@link #getStartTag()}<code>.</code>{@link StartTag#isEmptyElementTag() isEmptyElementTag()}.
 	 * <p>
-	 * This is equivalent to {@link #isEmpty()}<code> && </code>{@link #getStartTag()}<code>.</code>{@link StartTag#isEmptyElementTag() isEmptyElementTag()}.
-	 * <p>
-	 * The {@link StartTag#isEmptyElementTag()} property only checks whether the start tag syntactically an
-	 * <a target="_blank" href="http://www.w3.org/TR/REC-xml#dt-eetag">empty-element tag</a>, whereas this property also makes sure
-	 * the element is in fact {@linkplain #isEmpty() empty}.
-	 * <p>
-	 * A syntactical empty-element tag that is not actually empty can occur if the end tag of an <a href="HTMLElements.html#HTMLElement">HTML element</a>
-	 * is either {@linkplain HTMLElements#getEndTagRequiredElementNames() required} or {@linkplain HTMLElements#getEndTagOptionalElementNames() optional},
-	 * but the start tag is erroneously terminated with the characters "<code>/&gt;</code>" in the source document.
-	 * All major browsers ignore the syntactical hint of an empty element in this case, even in an
-	 * <a target="_blank" href="http://www.w3.org/TR/xhtml1/">XHTML</a> document, so this parser does the same.
+	 * Prior to version 2.6, this method was equivalent to:<br />
+	 * {@link #isEmpty()}<code> && </code>{@link #getStartTag()}<code>.</code>{@link StartTag#isSyntacticalEmptyElementTag() isSyntacticalEmptyElementTag()}.<br />
+	 * It was changed to make it identical with the new behaviour of the {@link StartTag#isEmptyElementTag()}.
+	 * In practice there are very few circumstances where the result is different to the old implementation of this method.
 	 *
 	 * @return <code>true</code> if this element is an <a target="_blank" href="http://www.w3.org/TR/REC-xml#dt-eetag">empty-element tag</a>, otherwise <code>false</code>.
 	 */
 	public boolean isEmptyElementTag() {
-		return isEmpty() && startTag.isEmptyElementTag();
+		return startTag.isEmptyElementTag();
 	}
 
 	/**
