@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -130,7 +130,7 @@ public abstract class FormControl extends Segment {
 	 * <i>field name</i> are for the most part synonymous, with only a possible difference in case differentiating them.
 	 * <p>
 	 * In contrast to the {@link FormField#getName()} method, this method always returns the name using the original case
-	 * from the source document, regardless of the current setting of the
+	 * from the source document, regardless of the current setting of the static
 	 * {@link Config#CurrentCompatibilityMode}<code>.</code>{@link Config.CompatibilityMode#isFormFieldNameCaseInsensitive() FormFieldNameCaseInsensitive} property.
 	 *
 	 * @return the <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">name</a> of the control.
@@ -334,7 +334,7 @@ public abstract class FormControl extends Segment {
 	}
 
 	/**
-	 * Returns a collection of all {@linkplain #getPredefinedValue() predefined values} in this control.
+	 * Returns a collection of all {@linkplain #getPredefinedValue() predefined values} in this control in order of appearance.
 	 * <p>
 	 * All objects in the returned collection are of type <code>String</code>, with no <code>null</code> entries.
 	 * <p>
@@ -361,7 +361,7 @@ public abstract class FormControl extends Segment {
 	 * The predefined values of a control are not affected by changes to the
 	 * <a href="#SubmissionValue">submission values</a> of the control.
 	 *
-	 * @return a collection of all {@linkplain #getPredefinedValue() predefined values} in this control, guaranteed not <code>null</code>.
+	 * @return a collection of all {@linkplain #getPredefinedValue() predefined values} in this control in order of appearance, guaranteed not <code>null</code>.
 	 * @see FormField#getPredefinedValues()
 	 */
 	public Collection getPredefinedValues() {
@@ -369,9 +369,9 @@ public abstract class FormControl extends Segment {
 	}
 
 	/**
-	 * Returns a collection of the control's <a href="#SubmissionValue">submission values</a>.
+	 * Returns a list of the control's <a href="#SubmissionValue">submission values</a> in order of appearance.
 	 * <p>
-	 * All objects in the returned collection are of type <code>CharSequence</code>, with no <code>null</code> entries.
+	 * All objects in the returned list are of type <code>CharSequence</code>, with no <code>null</code> entries.
 	 * <p>
 	 * The term <i><a name="SubmissionValue">submission value</a></i> is used in this library to refer to the value the control 
 	 * would contribute to the
@@ -397,7 +397,7 @@ public abstract class FormControl extends Segment {
 	 * <code>OPTION</code> elements.
 	 * <p>
 	 * Only a {@link FormControlType#SELECT_MULTIPLE SELECT_MULTIPLE} control can have more than one submission value,
-	 * all other {@linkplain FormControlType control types} return a collection containing either one value or no values.
+	 * all other {@linkplain FormControlType control types} return a list containing either one value or no values.
 	 * A {@link FormControlType#SELECT_SINGLE SELECT_SINGLE} control only returns multiple submission values
 	 * if it illegally contains multiple selected options in the source document.
 	 * <p>
@@ -408,7 +408,7 @@ public abstract class FormControl extends Segment {
 	 * <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#submit-format">submit</a> the form.
 	 * Because the submission value is intended to be a static representation of a control's data without
 	 * interaction by the user, this library never associates submission values with
-	 * {@linkplain FormControlType#isSubmit() submit} buttons, so this method always returns an empty collection for these
+	 * {@linkplain FormControlType#isSubmit() submit} buttons, so this method always returns an empty list for these
 	 * control types.
 	 * <p>
 	 * The <a href="#SubmissionValue">submission value(s)</a> of a control can be modified for subsequent output in
@@ -429,11 +429,11 @@ public abstract class FormControl extends Segment {
 	 * The values returned by this method reflect any changes made using the submission value modification methods,
 	 * in contrast to methods found in the {@link Attributes} and {@link Attribute} classes, which always reflect the source document.
 	 *
-	 * @return a collection of the control's <i>submission values</i>, guaranteed not <code>null</code>.
+	 * @return a list of the control's <i>submission values</i> in order of appearance, guaranteed not <code>null</code>.
 	 * @see #getPredefinedValues()
 	 */
-	public Collection getValues() {
-		final HashSet values=new HashSet();
+	public List getValues() {
+		final List values=new ArrayList();
 		addValuesTo(values);
 		return values;
 	}
@@ -689,10 +689,10 @@ public abstract class FormControl extends Segment {
 			throw new UnsupportedOperationException("Use getPredefinedValues() method instead on SELECT controls");
 		}
 		public Collection getPredefinedValues() {
-			final ArrayList arrayList=new ArrayList(optionElementContainers.length);
+			final LinkedHashSet linkedHashSet=new LinkedHashSet(optionElementContainers.length*2,1.0F);
 			for (int i=0; i<optionElementContainers.length; i++)
-			arrayList.add(optionElementContainers[i].predefinedValue);
-			return arrayList;
+			linkedHashSet.add(optionElementContainers[i].predefinedValue);
+			return linkedHashSet;
 		}
 		public Iterator getOptionElementIterator() {
 			return new OptionElementIterator();
@@ -727,7 +727,7 @@ public abstract class FormControl extends Segment {
 				final StringBuffer sb=new StringBuffer(100);
 				for (int i=0; i<optionElementContainers.length; i++) {
 					if (optionElementContainers[i].getBooleanAttribute(Attribute.SELECTED)) {
-						appendCollapseWhiteSpace(sb,optionElementContainers[i].element.getContent());
+						sb.append(getOptionLabel(optionElementContainers[i].element));
 						sb.append(FormControlOutputStyle.ConfigDisplayValue.MultipleValueSeparator);
 					}
 				}
@@ -740,6 +740,11 @@ public abstract class FormControl extends Segment {
 				}
 			}
 		}
+		private static String getOptionLabel(final Element optionElement) {
+			final String labelAttributeValue=optionElement.getAttributeValue("label");
+			if (labelAttributeValue!=null) return labelAttributeValue;
+			return CharacterReference.decodeCollapseWhiteSpace(optionElement.getContent());
+		}
 		private final class OptionElementIterator implements Iterator {
 			private int i=0;
 			public boolean hasNext() {
@@ -900,4 +905,3 @@ public abstract class FormControl extends Segment {
 		}
 	}
 }
-
