@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -41,7 +41,7 @@ import java.util.*;
  *   while ensuring that the states of all the field's constituent form controls remain consistent with each other.
  *   <p>
  *   The methods available for this purpose are:<br />
- *   {@link #getValues() Collection getValues()}<br />
+ *   {@link #getValues() List getValues()}<br />
  *   {@link #clearValues() void clearValues()}<br />
  *   {@link #setValues(Collection) void setValues(Collection)}<br />
  *   {@link #setValue(CharSequence) boolean setValue(CharSequence)}<br />
@@ -92,7 +92,7 @@ import java.util.*;
  * <p>
  * <code>FormField</code> instances are created automatically with the creation of a {@link FormFields} collection.
  * <p>
- * The case sensitivity of form field names is determined by the
+ * The case sensitivity of form field names is determined by the static
  * {@link Config#CurrentCompatibilityMode}<code>.</code>{@link Config.CompatibilityMode#isFormFieldNameCaseInsensitive() FormFieldNameCaseInsensitive} property.
  *
  * @see FormFields
@@ -116,8 +116,8 @@ public final class FormField {
 	/**
 	 * Returns the <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">control name</a> shared by all of this field's constituent {@linkplain FormControl controls}.
 	 * <p>
-	 * If {@link Config#CurrentCompatibilityMode}<code>.</code>{@link Config.CompatibilityMode#isFormFieldNameCaseInsensitive() isFormFieldNameCaseInsensitive()}
-	 * is <code>true</code>, the grouping of the controls by name is case insensitive
+	 * If the static {@link Config#CurrentCompatibilityMode}<code>.</code>{@link Config.CompatibilityMode#isFormFieldNameCaseInsensitive() isFormFieldNameCaseInsensitive()}
+	 * property is set to <code>true</code>, the grouping of the controls by name is case insensitive
 	 * and this method always returns the name in lower case.
 	 * <p>
 	 * Since a form field is simply a group of controls with the same name, the terms <i>control name</i> and
@@ -241,17 +241,19 @@ public final class FormField {
 	}
 
 	/**
-	 * Returns a collection of the <a href="#FieldSubmissionValues">field submission values</a>.
+	 * Returns a list of the <a href="#FieldSubmissionValues">field submission values</a> in order of appearance.
 	 * <p>
 	 * The term <i><a name="FieldSubmissionValues">field submission values</a></i> is used in this library to refer to the aggregate of all the
 	 * <a href="FormControl.html#SubmissionValue">submission values</a> of a field's constituent {@linkplain #getFormControls() form controls}.
 	 * <p>
-	 * All objects in the returned collection are of type <code>CharSequence</code>, with no <code>null</code> entries.
+	 * All objects in the returned list are of type <code>CharSequence</code>, with no <code>null</code> entries.
+	 * <p>
+	 * The list may contain duplicates if the this field has multiple controls with the same value.
 	 *
-	 * @return a collection of the <a href="#FieldSubmissionValue">field submission values</a>, guaranteed not <code>null</code>.
+	 * @return a list of the <a href="#FieldSubmissionValue">field submission values</a> in order of appearance, guaranteed not <code>null</code>.
 	 */
-	public Collection getValues() {
-		final HashSet values=new HashSet();
+	public List getValues() {
+		final List values=new ArrayList();
 		for (final Iterator i=formControls.iterator(); i.hasNext();)
 			((FormControl)i.next()).addValuesTo(values);
 		return values;
