@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.6
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -36,7 +36,7 @@ import java.util.*;
  *   for subsequent output in an {@link OutputDocument}.
  *   <p>
  *   The methods available for this purpose are:<br />
- *   {@link #getValues(String) Collection getValues(String fieldName)}<br />
+ *   {@link #getValues(String) List getValues(String fieldName)}<br />
  *   {@link #getDataSet() Map getDataSet()}<br />
  *   {@link #clearValues() void clearValues()}<br />
  *   {@link #setDataSet(Map) void setDataSet(Map)}<br />
@@ -91,7 +91,7 @@ import java.util.*;
  * <code>FormFields</code> instances are obtained using the {@link #FormFields(Collection formControls)} constructor
  * or by calling the {@link Segment#findFormFields()} method.
  * <p>
- * The case sensitivity of form field names is determined by the
+ * The case sensitivity of form field names is determined by the static
  * {@link Config#CurrentCompatibilityMode}<code>.</code>{@link Config.CompatibilityMode#isFormFieldNameCaseInsensitive() FormFieldNameCaseInsensitive} property.
  * <p>
  * <b>Examples:</b>
@@ -133,7 +133,7 @@ import java.util.*;
  *    formFields.addValue("Name","Humphrey Bear");
  *    formFields.addValue("MailingList","A");
  *    formFields.addValue("MailingList","B");
- *    formFields.addValue("FavouriteFair","honey");
+ *    formFields.addValue("FavouriteFare","honey");
  *    OutputDocument outputDocument=new OutputDocument(source);
  *    outputDocument.replace(formFields);
  *    String newHtmlText=outputDocument.toString();</pre>
@@ -150,7 +150,7 @@ import java.util.*;
  *    mailingListFormField.getFormControl("D").setDisabled(true);
  *    // remove some controls:
  *    formFields.get("button1").getFormControl().setOutputStyle(FormControlOutputStyle.REMOVE);
- *    FormControl rhubarbFormControl=formFields.get("FavouriteFair").getFormControl("rhubarb");
+ *    FormControl rhubarbFormControl=formFields.get("FavouriteFare").getFormControl("rhubarb");
  *    rhubarbFormControl.setOutputStyle(FormControlOutputStyle.REMOVE);
  *    // set some controls to display value:
  *    formFields.setValue("Address","The Lodge\nDeakin  ACT  2600\nAustralia");
@@ -212,7 +212,7 @@ public final class FormFields extends AbstractCollection {
 	/**
 	 * Returns the <code>FormField</code> with the specified {@linkplain FormField#getName() name}.
 	 * <p>
-	 * The case sensitivity of the <code>fieldName</code> argument is determined by the
+	 * The case sensitivity of the <code>fieldName</code> argument is determined by the static
 	 * {@link Config#CurrentCompatibilityMode}<code>.</code>{@link Config.CompatibilityMode#isFormFieldNameCaseInsensitive() FormFieldNameCaseInsensitive} property.
 	 *
 	 * @param fieldName  the name of the <code>FormField</code> to get.
@@ -239,18 +239,18 @@ public final class FormFields extends AbstractCollection {
 	}
 
 	/**
-	 * Returns a collection of the <a href="FormField.html#FieldSubmissionValue">field submission values</a> of all the specified constituent {@linkplain FormField form field} with the specified {@linkplain FormField#getName() name}.
+	 * Returns a list of the <a href="FormField.html#FieldSubmissionValue">field submission values</a> of all the specified constituent {@linkplain FormField form fields} with the specified {@linkplain FormField#getName() name}.
 	 * <p>
-	 * All objects in the returned collection are of type <code>CharSequence</code>, with no <code>null</code> entries.
+	 * All objects in the returned list are of type <code>CharSequence</code>, with no <code>null</code> entries.
 	 * <p>
 	 * This is equivalent to {@link #get(String) get(fieldName)}<code>.</code>{@link FormField#getValues() getValues()},
 	 * assuming that a field with the specified name exists in this collection.
 	 *
 	 * @param fieldName  the {@linkplain FormField#getName() name} of the form field.
-	 * @return a collection of the <a href="FormField.html#FieldSubmissionValue">field submission values</a> of all the specified constituent {@linkplain FormField form field} with the specified {@linkplain FormField#getName() name}, or <code>null</code> if no form field with this name exists.
+	 * @return a list of the <a href="FormField.html#FieldSubmissionValue">field submission values</a> of all the specified constituent {@linkplain FormField form field} with the specified {@linkplain FormField#getName() name}, or <code>null</code> if no form field with this name exists.
 	 * @see FormField#getValues()
 	 */
-	public Collection getValues(final String fieldName) {
+	public List getValues(final String fieldName) {
 		final FormField formField=get(fieldName);
 		return formField==null ? null : formField.getValues();
 	}
@@ -264,15 +264,17 @@ public final class FormFields extends AbstractCollection {
 	 * <p>
 	 * Only the {@linkplain FormField#getName() names} of form fields with at least one {@linkplain FormField#getValues() value}
 	 * are included in the map, meaning every <code>String[]</code> is guaranteed to have at least one entry.
+	 * <p>
+	 * Iterating over the map keys returns them in the order of appearance in the source document.
 	 *
 	 * @return the entire <a href="#FieldDataSet">field data set</a> represented by the {@linkplain FormField#getValues() values} of the constituent form fields.
 	 * @see #setDataSet(Map)
 	 */
 	public Map getDataSet() {
-		final HashMap map=new HashMap((int)(getCount()/0.7));
+		final LinkedHashMap map=new LinkedHashMap((int)(getCount()/0.7));
 		for (final Iterator i=iterator(); i.hasNext();) {
 			final FormField formField=(FormField)i.next();
-			final Collection values=formField.getValues();
+			final List values=formField.getValues();
 			if (values.isEmpty()) continue;
 			final String[] valuesArray=new String[values.size()];
 			final Iterator valuesIterator=values.iterator();
@@ -417,7 +419,7 @@ public final class FormFields extends AbstractCollection {
 	 *        <tr><td>Value:<td>the single value mapped to this field in the specified <a href="#FieldDataSet">field data set</a>.
 	 *       </table>
 	 *       In the unlikely event that this field contains more than one value, all values are included in this one column and
-	 *       separated by the text defined in the {@link Config#ColumnMultipleValueSeparator} property.
+	 *       separated by the text defined in the static {@link Config#ColumnMultipleValueSeparator} property.
 	 *     </ul>
 	 *    <li>
 	 *     Otherwise, if the form field does have {@linkplain FormField#getPredefinedValues() predefined values},
@@ -478,7 +480,7 @@ public final class FormFields extends AbstractCollection {
 	 *          <tr><td>{@linkplain #getColumnLabels() Label}:<td>the {@linkplain FormField#getName() name} of the form field in original case
 	 *          <tr><td>Value:<td>all values mapped to this field in the specified <a href="#FieldDataSet">field data set</a>
 	 *          that do not match any of the {@linkplain FormField#getPredefinedValues() predefined values},
-	 *          separated by the text defined in the {@link Config#ColumnMultipleValueSeparator} property.
+	 *          separated by the text defined in the static {@link Config#ColumnMultipleValueSeparator} property.
 	 *         </table>
 	 *       </ul>
 	 *     </ul>
