@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.6
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -43,20 +43,13 @@ public final class ParseText implements CharSequence {
 	public static final int NO_BREAK=-1;
 
 	/**
-	 * Constructs a new <code>ParseText</code> object based on the specified <code>CharSequence</code>.
-	 * @param charSequence  the character sequence upon which the parse text is based.
+	 * Constructs a new <code>ParseText</code> object based on the specified string.
+	 * @param string  the string upon which the parse text is based.
 	 */
-	ParseText(final CharSequence charSequence) {
-		text=new char[charSequence.length()];
-		for (int i=0; i<text.length; i++) text[i]=Character.toLowerCase(charSequence.charAt(i));
-	}
-
-	/**
-	 * Constructs a new <code>ParseText</code> object based on the specified {@link OutputDocument}.
-	 * @param outputDocument  the {@link OutputDocument} upon which the parse text is based.
-	 */
-	ParseText(final OutputDocument outputDocument) {
-		this(outputDocument.toString());
+	ParseText(final String string) {
+		text=new char[string.length()];
+		string.getChars(0,string.length(),text,0);
+		for (int i=0; i<text.length; i++) text[i]=Character.toLowerCase(text[i]);
 	}
 
 	/**
