@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -41,8 +41,8 @@ public class Segment implements Comparable, CharSequence {
 	/**
 	 * Constructs a new <code>Segment</code> within the specified {@linkplain Source source} document with the specified begin and end character positions.
 	 * @param source  the {@link Source} document, must not be <code>null</code>.
-	 * @param begin  the character position in the source where this segment begins.
-	 * @param end  the character position in the source where this segment ends.
+	 * @param begin  the character position in the source where this segment {@linkplain #getBegin() begins}, inclusive.
+	 * @param end  the character position in the source where this segment {@linkplain #getEnd() ends}, exclusive.
 	 */
 	public Segment(final Source source, final int begin, final int end) {
 		if (begin==-1 || end==-1 || begin>end) throw new IllegalArgumentException();
@@ -59,7 +59,7 @@ public class Segment implements Comparable, CharSequence {
 		source=(Source)this;
 	}
 
-	// Only used for creating dummy flag instances of this type (see Element.NOT_CACHED)
+	// Only used for creating dummy flag instances of this type (see Tag.NOT_CACHED and Element.NOT_CACHED)
 	Segment() {
 		begin=0;
 		end=0;
@@ -67,8 +67,16 @@ public class Segment implements Comparable, CharSequence {
 	}
 
 	/**
-	 * Returns the character position in the {@link Source} document at which this segment begins.
-	 * @return the character position in the {@link Source} document at which this segment begins.
+	 * Returns the {@link Source} document containing this segment.
+	 * @return the {@link Source} document containing this segment.
+	 */
+	public final Source getSource() {
+		return source;
+	}
+
+	/**
+	 * Returns the character position in the {@link Source} document at which this segment begins, inclusive.
+	 * @return the character position in the {@link Source} document at which this segment begins, inclusive.
 	 */
 	public final int getBegin() {
 		return begin;
@@ -186,6 +194,47 @@ public class Segment implements Comparable, CharSequence {
 		return new TextExtractor(this);
 	}
 
+	/**
+	 * Returns an iterator over every tag and text segment contained within this segment.
+	 * <p>
+	 * Every tag found in the {@link #findAllTags()} list is included in this iterator, including all {@linkplain TagType#isServerTag() server tags}.
+	 * <p>
+	 * Segments of the document between the tags are also included, resulting in a sequential walk-through of every "node" in this segment, where a node is either
+	 * a tag or a segment of text.
+	 * The {@linkplain #getEnd() end} position of each segment should correspond with the {@linkplain #getBegin() begin} position of the subsequent segment,
+	 * unless any of the tags are enclosed by other tags, which is common when {@linkplain TagType#isServerTag() server tags} are present.
+	 * <p>
+	 * The {@link CharacterReference#decodeCollapseWhiteSpace(CharSequence)} method can be used to retrieve the text from each text segment.
+	 * <p>
+	 * <dl>
+	 *  <dt>Example:</dt>
+	 *  <dd>
+	 *   <p>
+	 *   The following code demonstrates the typical usage of this method.
+	 *   <p>
+	 * <pre>
+	 * for (Iterator nodeIterator=segment.getNoteIterator(); nodeIterator.hasNext();) {
+	 *   Segment nodeSegment=(Segment)nodeIterator.next();
+	 *   if (nodeSegment instanceof Tag) {
+	 *     Tag tag=(Tag)nodeSegment;
+	 *     if (tag.getTagType().isServerTag()) continue; // ignore server tags
+	 *     // Process the tag (just output it in this example):
+	 *     System.out.println(tag.tidy());
+	 *   } else {
+	 *     // Segment is a text segment.
+	 *     // Process the text segment (just output its text in this example):
+	 *     String text=CharacterReference.decodeCollapseWhiteSpace(nodeSegment);
+	 *     System.out.println(text);
+	 *   }
+	 * }</pre>
+	 *  </dd>
+	 * </dl>
+	 * @return an iterator over every tag and text segment contained within this segment.
+	 */
+	public Iterator getNodeIterator() {
+		return new NodeIterator(this);
+	}
+
 	/**
 	 * Returns a list of all {@link Tag} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * <p>
@@ -212,12 +261,12 @@ public class Segment implements Comparable, CharSequence {
 	 * @return a list of all {@link Tag} objects of the specified {@linkplain TagType type} that are {@linkplain #encloses(Segment) enclosed} by this segment.
 	 */
 	public List findAllTags(final TagType tagType) {
-		Tag tag=checkEnclosure(Tag.findPreviousOrNextTag(source,begin,tagType,false));
+		Tag tag=checkEnclosure(Tag.findNextTag(source,begin,tagType));
 		if (tag==null) return Collections.EMPTY_LIST;
 		final ArrayList list=new ArrayList();
 		do {
 			list.add(tag);
-			tag=checkEnclosure(Tag.findPreviousOrNextTag(source,tag.begin+1,tagType,false));
+			tag=checkEnclosure(tagType==null ? tag.findNextTag() : Tag.findNextTag(source,tag.begin+1,tagType)); // tag.findNext() may be more efficient than Tag.findNextTag(source,tag.begin+1,null) due to caching
 		} while (tag!=null);
 		return list;
 	}
@@ -238,7 +287,7 @@ public class Segment implements Comparable, CharSequence {
 	}
 
 	/**
-	 * Returns a list of all {@link StartTag} objects with the specified name that are {@linkplain #encloses(Segment) enclosed} by this segment.
+	 * Returns a list of all {@linkplain StartTagType#NORMAL normal} {@link StartTag} objects with the specified {@linkplain StartTag#getName() name} that are {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * <p>
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 * <p>
@@ -247,17 +296,17 @@ public class Segment implements Comparable, CharSequence {
 	 * This method also returns {@linkplain Tag#isUnregistered() unregistered} tags if the specified name is not a valid {@linkplain Tag#isXMLName(CharSequence) XML tag name}.
 	 *
 	 * @param name  the {@linkplain StartTag#getName() name} of the start tags to find.
-	 * @return a list of all {@link StartTag} objects with the specified name that are {@linkplain #encloses(Segment) enclosed} by this segment.
+	 * @return a list of all {@linkplain StartTagType#NORMAL normal} {@link StartTag} objects with the specified {@linkplain StartTag#getName() name} that are {@linkplain #encloses(Segment) enclosed} by this segment.
 	 */
 	public List findAllStartTags(String name) {
 		if (name!=null) name=name.toLowerCase();
 		final boolean isXMLTagName=Tag.isXMLName(name);
-		StartTag startTag=(StartTag)checkEnclosure(StartTag.findPreviousOrNext(source,begin,name,isXMLTagName,false));
+		StartTag startTag=(StartTag)checkEnclosure(StartTag.findNext(source,begin,name,StartTagType.NORMAL,isXMLTagName));
 		if (startTag==null) return Collections.EMPTY_LIST;
 		final ArrayList list=new ArrayList();
 		do {
 			list.add(startTag);
-			startTag=(StartTag)checkEnclosure(StartTag.findPreviousOrNext(source,startTag.begin+1,name,isXMLTagName,false));
+			startTag=(StartTag)checkEnclosure(StartTag.findNext(source,startTag.begin+1,name,StartTagType.NORMAL,isXMLTagName));
 		} while (startTag!=null);
 		return list;
 	}
@@ -470,16 +519,20 @@ public class Segment implements Comparable, CharSequence {
 	 * Ignored segments are treated as blank spaces by the parsing mechanism, but are included as normal text in all other functions.
 	 * <p>
 	 * This method was originally the only means of preventing {@linkplain TagType#isServerTag() server tags} located inside
-	 * {@linkplain StartTagType#NORMAL normal} tags from interfering with the parsing of the tags.
-	 * The most common scenario is where the {@linkplain Attributes attributes} of a normal tag uses server tags to dynamically set the values of the attributes.
+	 * {@linkplain StartTagType#NORMAL normal} tags from interfering with the parsing of the tags
+	 * (such as where an {@linkplain Attribute attribute} of a normal tag uses a server tag to dynamically set its value),
+	 * as well as preventing non-server tags from being recognised inside server tags.
 	 * <p>
-	 * As of version 2.4 it is no longer necessary to use this method to ignore {@linkplain StartTagType#SERVER_COMMON common server tags} inside normal tags,
-	 * as the attributes parser now automatically ignores common server tags.
+	 * As of version 2.4 it is no longer necessary to use this method to ignore {@linkplain TagType#isServerTag() server tags} located inside normal tags,
+	 * as the attributes parser now automatically ignores any server tags.
 	 * <p>
-	 * As of version 2.5 it is also unnecessary to use this method to ignore the contents of {@link HTMLElementName#SCRIPT SCRIPT} elements,
-	 * as the parser automatically ignores this content when performing a {@linkplain Source#fullSequentialParse() full sequential parse}.
+	 * As of version 2.5 it is no longer necessary to use this method to ignore the contents of {@link HTMLElementName#SCRIPT SCRIPT} elements,
+	 * as the parser now does this automatically when performing a {@linkplain Source#fullSequentialParse() full sequential parse}.
 	 * <p>
-	 * This leaves only a few scenarios where calling this method still provides a significant benefit.
+	 * As of version 2.6 is is no longer necessary to use this method to ignore non-server tags inside server tags,
+	 * as the parser now does this automatically when performing a {@linkplain Source#fullSequentialParse() full sequential parse}.
+	 * <p>
+	 * This leaves only very few scenarios where calling this method still provides a significant benefit.
 	 * <p>
 	 * One such case is where XML-style server tags are used inside {@linkplain StartTagType#NORMAL normal} tags.
 	 * Here is an example using an XML-style JSP tag:
@@ -569,7 +622,7 @@ public class Segment implements Comparable, CharSequence {
 	 * </ul>
 	 * <p>
 	 * Despite the explicit inclusion of the zero-width space in the HTML specification, Microsoft IE6 does not
-	 * recognise them as whitespace and renders them as an unprintable character (empty square).
+	 * recognise them as white space and renders them as an unprintable character (empty square).
 	 * Even zero-width spaces included using the numeric character reference <code>&amp;#x200B;</code> are rendered this way.
 	 *
 	 * @param ch  the character to test.
