@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -28,7 +28,7 @@ import java.net.*;
  * Represents a source HTML document.
  * <p>
  * The first step in parsing an HTML document is always to construct a <code>Source</code> object from the source data, which can be a
- * <code>String</code>, <code>Reader</code>, <code>InputStream</code> or <code>URL</code>.
+ * <code>String</code>, <code>Reader</code>, <code>InputStream</code>, <code>URLConnection</code> or <code>URL</code>.
  * Each constructor uses all the evidence available to determine the original {@linkplain #getEncoding() character encoding} of the data.
  * <p>
  * Once the <code>Source</code> object has been created, you can immediately start searching for {@linkplain Tag tags} or {@linkplain Element elements} within the document
@@ -67,6 +67,7 @@ public class Source extends Segment {
 	final Cache cache=new Cache(this);
 	boolean useAllTypesCache=true;
 	boolean useSpecialTypesCache=true;
+	int[] fullSequentialParseData=null; // non-null iff a fullSequentialParse is underway. In version 2.5 this was passed around as a parameter during full sequential parse, but this approach was found to be error-prone and abandoned in 2.6
 	// cached result lists:
 	Tag[] allTagsArray=null; // non-null iff fullSequentialParse was called
 	List allTags=null; // non-null iff fullSequentialParse was called
@@ -139,10 +140,23 @@ public class Source extends Segment {
 	/**
 	 * Constructs a new <code>Source</code> object by loading the content from the specified URL.
 	 * <p>
+	 * This is equivalent to {@link #Source(URLConnection) Source(url.openConnection())}.
+	 *
+	 * @param url  the URL from which to load the source text.
+	 * @throws java.io.IOException if an I/O error occurs.
+	 * @see #getEncoding()
+	 */
+	public Source(final URL url) throws IOException {
+		this(new EncodingDetector(url.openConnection()));
+	}
+
+	/**
+	 * Constructs a new <code>Source</code> object by loading the content from the specified <code>URLConnection</code>.
+	 * <p>
 	 * The algorithm for detecting the character {@linkplain #getEncoding() encoding} of the source document is as follows:
 	 * <br />(process termination is marked by &diams;)
 	 * <ol class="HalfSeparated">
-	 *  <li>If the HTTP headers received when opening a connection to the URL include a 
+	 *  <li>If the HTTP headers received from the URL connection include a 
 	 *   <a target="_blank" href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.17">Content-Type</a> header
 	 *   specifying a <code>charset</code> parameter, then use the encoding specified in the value of the <code>charset</code> parameter. &diams;
 	 *  <li>Read the first four bytes of the input stream.
@@ -195,12 +209,12 @@ public class Source extends Segment {
 	 *   This is consistent with the preliminary encoding detected in this scenario.
 	 * </ol>
 	 *
-	 * @param url  the URL from which to load the source text.
+	 * @param urlConnection  the URL connection from which to load the source text.
 	 * @throws java.io.IOException if an I/O error occurs.
 	 * @see #getEncoding()
 	 */
-	public Source(final URL url) throws IOException {
-		this(new EncodingDetector(url));
+	public Source(final URLConnection urlConnection) throws IOException {
+		this(new EncodingDetector(urlConnection));
 	}
 
 	private String setEncoding(final String encoding, final String encodingSpecificationInfo) {
@@ -711,7 +725,7 @@ public class Source extends Segment {
 	 * @return the {@link Tag} at the specified position in the source document, or <code>null</code> if no tag exists at the specified position or it is out of bounds.
 	 */
 	public final Tag getTagAt(final int pos) {
-		return Tag.getTagAt(this,pos);
+		return Tag.getTagAt(this,pos,false);
 	}
 
 	/**
@@ -723,7 +737,7 @@ public class Source extends Segment {
 	 * @return the {@link Tag} beginning at or immediately preceding the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public Tag findPreviousTag(final int pos) {
-		return Tag.findPreviousOrNextTag(this,pos,true);
+		return Tag.findPreviousTag(this,pos);
 	}
 
 	/**
@@ -736,7 +750,7 @@ public class Source extends Segment {
 	 * @return the {@link Tag} with the specified {@linkplain TagType type} beginning at or immediately preceding the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public Tag findPreviousTag(final int pos, final TagType tagType) {
-		return Tag.findPreviousOrNextTag(this,pos,tagType,true);
+		return Tag.findPreviousTag(this,pos,tagType);
 	}
 	
 	/**
@@ -750,7 +764,25 @@ public class Source extends Segment {
 	 * @return the {@link Tag} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public Tag findNextTag(final int pos) {
-		return Tag.findPreviousOrNextTag(this,pos,false);
+		return Tag.findNextTag(this,pos);
+	}
+
+	Tag findNextNonServerTag(int pos) {
+		while (true) {
+			final Tag tag=findNextTag(pos);
+			if (tag==null) return null;
+			if (!tag.getTagType().isServerTag()) return tag;
+			pos=tag.end;
+		}
+	}
+
+	Tag findPreviousNonServerTag(int pos) {
+		while (true) {
+			final Tag tag=findPreviousTag(pos-1);
+			if (tag==null) return null;
+			if (!tag.getTagType().isServerTag()) return tag;
+			pos=tag.begin-1;
+		}
 	}
 
 	/**
@@ -763,7 +795,7 @@ public class Source extends Segment {
 	 * @return the {@link Tag} with the specified {@linkplain TagType type} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public Tag findNextTag(final int pos, final TagType tagType) {
-		return Tag.findPreviousOrNextTag(this,pos,tagType,false);
+		return Tag.findNextTag(this,pos,tagType);
 	}
 
 	/**
@@ -808,7 +840,7 @@ public class Source extends Segment {
 	}
 
 	/**
-	 * Returns the {@link Element} with the specified {@linkplain Element#getName() name} beginning at or immediately following the specified position in the source document.
+	 * Returns the {@linkplain StartTagType#NORMAL normal} {@link Element} with the specified {@linkplain Element#getName() name} beginning at or immediately following the specified position in the source document.
 	 * <p>
 	 * This is equivalent to {@link #findNextStartTag(int,String) findNextStartTag(pos,name)}<code>.</code>{@link StartTag#getElement() getElement()},
 	 * assuming the result is not <code>null</code>.
@@ -823,7 +855,7 @@ public class Source extends Segment {
 	 *
 	 * @param pos  the position in the source document from which to start the search, may be out of bounds.
 	 * @param name  the {@linkplain Element#getName() name} of the element to search for.
-	 * @return the {@link Element} with the specified {@linkplain Element#getName() name} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
+	 * @return the {@linkplain StartTagType#NORMAL normal} {@link Element} with the specified {@linkplain Element#getName() name} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public Element findNextElement(final int pos, String name) {
 		final StartTag startTag=findNextStartTag(pos,name);
@@ -856,11 +888,11 @@ public class Source extends Segment {
 	 * @return the {@link StartTag} at or immediately preceding the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public StartTag findPreviousStartTag(final int pos) {
-		return StartTag.findPreviousOrNext(this,pos,true);
+		return StartTag.findPrevious(this,pos);
 	}
 
 	/**
-	 * Returns the {@link StartTag} with the specified {@linkplain StartTag#getName() name} at or immediately preceding (or {@linkplain Segment#encloses(int) enclosing}) the specified position in the source document.
+	 * Returns the {@linkplain StartTagType#NORMAL normal} {@link StartTag} with the specified {@linkplain StartTag#getName() name} at or immediately preceding (or {@linkplain Segment#encloses(int) enclosing}) the specified position in the source document.
 	 * <p>
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 * <p>
@@ -871,12 +903,28 @@ public class Source extends Segment {
 	 *
 	 * @param pos  the position in the source document from which to start the search, may be out of bounds.
 	 * @param name  the {@linkplain StartTag#getName() name} of the start tag to search for.
-	 * @return the {@link StartTag} with the specified {@linkplain StartTag#getName() name} at or immediately preceding the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
+	 * @return the {@linkplain StartTagType#NORMAL normal} {@link StartTag} with the specified {@linkplain StartTag#getName() name} at or immediately preceding the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
-	public StartTag findPreviousStartTag(final int pos, String name) {
+	public StartTag findPreviousStartTag(final int pos, final String name) {
+		return findPreviousStartTag(pos,name,StartTagType.NORMAL);
+	}
+
+	/**
+	 * Returns the {@link StartTag} with the specified {@linkplain StartTag#getName() name} and {@linkplain StartTagType type} at or immediately preceding (or {@linkplain Segment#encloses(int) enclosing}) the specified position in the source document.
+	 * <p>
+	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
+	 * <p>
+	 * Specifying {@link StartTagType#NORMAL} as the argument to the <code>startTagType</code> parameter is equivalent to
+	 * {@link #findPreviousStartTag(int,String) findPreviousStartTag(pos,name)}.
+	 *
+	 * @param pos  the position in the source document from which to start the search, may be out of bounds.
+	 * @param name  the {@linkplain StartTag#getName() name} of the start tag to search for, may be <code>null</code>.
+	 * @param startTagType  the {@linkplain StartTagType type} of the start tag to search for, must not be <code>null</code>.
+	 * @return the {@link StartTag} with the specified {@linkplain StartTag#getName() name} and {@linkplain StartTagType type} at or immediately preceding (or {@linkplain Segment#encloses(int) enclosing}) the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
+	 */
+	public StartTag findPreviousStartTag(final int pos, String name, final StartTagType startTagType) {
 		if (name!=null) name=name.toLowerCase();
-		final boolean isXMLTagName=Tag.isXMLName(name);
-		return StartTag.findPreviousOrNext(this,pos,name,isXMLTagName,true);
+		return StartTag.findPrevious(this,pos,name,startTagType);
 	}
 
 	/**
@@ -888,11 +936,11 @@ public class Source extends Segment {
 	 * @return the {@link StartTag} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public StartTag findNextStartTag(final int pos) {
-		return StartTag.findPreviousOrNext(this,pos,false);
+		return StartTag.findNext(this,pos);
 	}
 
 	/**
-	 * Returns the {@link StartTag} with the specified {@linkplain StartTag#getName() name} beginning at or immediately following the specified position in the source document.
+	 * Returns the {@linkplain StartTagType#NORMAL normal} {@link StartTag} with the specified {@linkplain StartTag#getName() name} beginning at or immediately following the specified position in the source document.
 	 * <p>
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 * <p>
@@ -905,13 +953,29 @@ public class Source extends Segment {
 	 * This method also returns {@linkplain Tag#isUnregistered() unregistered} tags if the specified name is not a valid {@linkplain Tag#isXMLName(CharSequence) XML tag name}.
 	 *
 	 * @param pos  the position in the source document from which to start the search, may be out of bounds.
-	 * @param name  the {@linkplain StartTag#getName() name} of the start tag to search for.
-	 * @return the {@link StartTag} with the specified {@linkplain StartTag#getName() name} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
+	 * @param name  the {@linkplain StartTag#getName() name} of the start tag to search for, may be <code>null</code>.
+	 * @return the {@linkplain StartTagType#NORMAL normal} {@link StartTag} with the specified {@linkplain StartTag#getName() name} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
-	public StartTag findNextStartTag(final int pos, String name) {
+	public StartTag findNextStartTag(final int pos, final String name) {
+		return findNextStartTag(pos,name,StartTagType.NORMAL);
+	}
+
+	/**
+	 * Returns the {@link StartTag} with the specified {@linkplain StartTag#getName() name} and {@linkplain StartTagType type} beginning at or immediately following the specified position in the source document.
+	 * <p>
+	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
+	 * <p>
+	 * Specifying {@link StartTagType#NORMAL} as the argument to the <code>startTagType</code> parameter is equivalent to
+	 * {@link #findNextStartTag(int,String) findNextStartTag(pos,name)}.
+	 *
+	 * @param pos  the position in the source document from which to start the search, may be out of bounds.
+	 * @param name  the {@linkplain StartTag#getName() name} of the start tag to search for, may be <code>null</code>.
+	 * @param startTagType  the {@linkplain StartTagType type} of the start tag to search for, must not be <code>null</code>.
+	 * @return the {@link StartTag} with the specified {@linkplain StartTag#getName() name} and {@linkplain StartTagType type} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
+	 */
+	public StartTag findNextStartTag(final int pos, String name, final StartTagType startTagType) {
 		if (name!=null) name=name.toLowerCase();
-		final boolean isXMLTagName=Tag.isXMLName(name);
-		return StartTag.findPreviousOrNext(this,pos,name,isXMLTagName,false);
+		return StartTag.findNext(this,pos,name,startTagType);
 	}
 
 	/**
@@ -938,7 +1002,7 @@ public class Source extends Segment {
 	 * @return the {@link EndTag} beginning at or immediately preceding the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public EndTag findPreviousEndTag(final int pos) {
-		return EndTag.findPreviousOrNext(this,pos,true);
+		return EndTag.findPrevious(this,pos);
 	}
 
 	/**
@@ -952,7 +1016,7 @@ public class Source extends Segment {
 	 */
 	public EndTag findPreviousEndTag(final int pos, final String name) {
 		if (name==null) throw new IllegalArgumentException("name argument must not be null");
-		return EndTag.findPreviousOrNext(this,pos,name.toLowerCase(),EndTagType.NORMAL,true);
+		return EndTag.findPrevious(this,pos,name.toLowerCase(),EndTagType.NORMAL);
 	}
 
 	/**
@@ -964,7 +1028,7 @@ public class Source extends Segment {
 	 * @return the {@link EndTag} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public EndTag findNextEndTag(final int pos) {
-		return EndTag.findPreviousOrNext(this,pos,false);
+		return EndTag.findNext(this,pos);
 	}
 
 	/**
@@ -973,7 +1037,7 @@ public class Source extends Segment {
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 *
 	 * @param pos  the position in the source document from which to start the search, may be out of bounds.
-	 * @param name  the {@linkplain StartTag#getName() name} of the end tag to search for, must not be <code>null</code>.
+	 * @param name  the {@linkplain EndTag#getName() name} of the end tag to search for, must not be <code>null</code>.
 	 * @return the {@linkplain EndTagType#NORMAL normal} {@link EndTag} with the specified {@linkplain EndTag#getName() name} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public EndTag findNextEndTag(final int pos, final String name) {
@@ -992,11 +1056,11 @@ public class Source extends Segment {
 	 */
 	public EndTag findNextEndTag(final int pos, final String name, final EndTagType endTagType) {
 		if (name==null) throw new IllegalArgumentException("name argument must not be null");
-		return EndTag.findPreviousOrNext(this,pos,name.toLowerCase(),endTagType,false);
+		return EndTag.findNext(this,pos,name.toLowerCase(),endTagType);
 	}
 
 	/**
-	 * Returns the most nested {@link Element} that {@linkplain Segment#encloses(int) encloses} the specified position in the source document.
+	 * Returns the most nested {@linkplain StartTagType#NORMAL normal} {@link Element} that {@linkplain Segment#encloses(int) encloses} the specified position in the source document.
 	 * <p>
 	 * The specified position can be anywhere inside the {@linkplain Element#getStartTag() start tag}, {@linkplain Element#getEndTag() end tag},
 	 * or {@linkplain Element#getContent() content} of the element.  There is no requirement that the returned element has an end tag, and it
@@ -1005,14 +1069,14 @@ public class Source extends Segment {
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 *
 	 * @param pos  the position in the source document, may be out of bounds.
-	 * @return the most nested {@link Element} that {@linkplain Segment#encloses(int) encloses} the specified position in the source document, or <code>null</code> if the position is not within an element or is out of bounds.
+	 * @return the most nested {@linkplain StartTagType#NORMAL normal} {@link Element} that {@linkplain Segment#encloses(int) encloses} the specified position in the source document, or <code>null</code> if the position is not within an element or is out of bounds.
 	 */
 	public Element findEnclosingElement(final int pos) {
 		return findEnclosingElement(pos,null);
 	}
 
 	/**
-	 * Returns the most nested {@link Element} with the specified {@linkplain Element#getName() name} that {@linkplain Segment#encloses(int) encloses} the specified position in the source document.
+	 * Returns the most nested {@linkplain StartTagType#NORMAL normal} {@link Element} with the specified {@linkplain Element#getName() name} that {@linkplain Segment#encloses(int) encloses} the specified position in the source document.
 	 * <p>
 	 * The specified position can be anywhere inside the {@linkplain Element#getStartTag() start tag}, {@linkplain Element#getEndTag() end tag},
 	 * or {@linkplain Element#getContent() content} of the element.  There is no requirement that the returned element has an end tag, and it
@@ -1024,14 +1088,14 @@ public class Source extends Segment {
 	 *
 	 * @param pos  the position in the source document, may be out of bounds.
 	 * @param name  the {@linkplain Element#getName() name} of the element to search for.
-	 * @return the most nested {@link Element} with the specified {@linkplain Element#getName() name} that {@linkplain Segment#encloses(int) encloses} the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
+	 * @return the most nested {@linkplain StartTagType#NORMAL normal} {@link Element} with the specified {@linkplain Element#getName() name} that {@linkplain Segment#encloses(int) encloses} the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public Element findEnclosingElement(final int pos, String name) {
 		int startBefore=pos;
 		if (name!=null) name=name.toLowerCase();
 		final boolean isXMLTagName=Tag.isXMLName(name);
 		while (true) {
-			StartTag startTag=StartTag.findPreviousOrNext(this,startBefore,name,isXMLTagName,true);
+			StartTag startTag=StartTag.findPrevious(this,startBefore,name,StartTagType.NORMAL,isXMLTagName);
 			if (startTag==null) return null;
 			Element element=startTag.getElement();
 			if (pos < element.end) return element;
@@ -1048,7 +1112,7 @@ public class Source extends Segment {
 	 * @return the {@link CharacterReference} beginning at or immediately preceding the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public CharacterReference findPreviousCharacterReference(final int pos) {
-		return CharacterReference.findPreviousOrNext(this,pos,true);
+		return CharacterReference.findPrevious(this,pos);
 	}
 
 	/**
@@ -1060,7 +1124,7 @@ public class Source extends Segment {
 	 * @return the {@link CharacterReference} beginning at or immediately following the specified position in the source document, or <code>null</code> if none exists or the specified position is out of bounds.
 	 */
 	public CharacterReference findNextCharacterReference(final int pos) {
-		return CharacterReference.findPreviousOrNext(this,pos,false);
+		return CharacterReference.findNext(this,pos);
 	}
 
 	/**
@@ -1243,10 +1307,10 @@ public class Source extends Segment {
 	public final ParseText getParseText() {
 		if (parseText==null) {
 			if (parseTextOutputDocument!=null) {
-				parseText=new ParseText(parseTextOutputDocument);
+				parseText=new ParseText(parseTextOutputDocument.toString());
 				parseTextOutputDocument=null;
 			} else {
-				parseText=new ParseText(this);
+				parseText=new ParseText(string);
 			}
 		}
 		return parseText;
