@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -83,6 +83,7 @@ public final class SourceFormatter implements CharStreamSource {
 	private String indentString="\t";
 	private boolean tidyTags=false;
 	private boolean collapseWhiteSpace=false;
+	private boolean removeLineBreaks=false;
 	private boolean indentAllElements=false;
 	private String newLine=null;
 
@@ -97,7 +98,7 @@ public final class SourceFormatter implements CharStreamSource {
 
 	// Documentation inherited from CharStreamSource
 	public void writeTo(final Writer writer) throws IOException {
-		new Processor(segment,getIndentString(),getTidyTags(),getCollapseWhiteSpace(),getIndentAllElements(),getIndentAllElements(),getNewLine()).writeTo(writer);
+		new Processor(segment,getIndentString(),getTidyTags(),getCollapseWhiteSpace(),getRemoveLineBreaks(),getIndentAllElements(),getIndentAllElements(),getNewLine()).writeTo(writer);
 	}
 
 	// Documentation inherited from CharStreamSource
@@ -195,6 +196,37 @@ public final class SourceFormatter implements CharStreamSource {
 		return collapseWhiteSpace;
 	}
 
+	/**
+	 * Sets whether all non-essential line breaks are removed.
+	 * <p>
+	 * The default value is <code>false</code>.
+	 * <p>
+	 * If this property is set to <code>true</code>, only essential line breaks are retained in the output.
+	 * <p>
+	 * Setting this property automatically engages the {@link #setCollapseWhiteSpace(boolean) CollapseWhiteSpace} option, regardless of its property setting.
+	 * <p>
+	 * It is recommended to set the {@link #setTidyTags(boolean) TidyTags} property when this option is used so that non-essential line breaks are also removed from tags.
+	 *
+	 * @param removeLineBreaks  specifies whether all non-essential line breaks are removed.
+	 * @return this <code>SourceFormatter</code> instance, allowing multiple property setting methods to be chained in a single statement. 
+	 * @see #getRemoveLineBreaks()
+	 */
+	SourceFormatter setRemoveLineBreaks(final boolean removeLineBreaks) {
+		this.removeLineBreaks=removeLineBreaks;
+		return this;
+	}
+	
+	/**
+	 * Indicates whether all non-essential line breaks are removed.
+	 * <p>
+	 * See the {@link #setRemoveLineBreaks(boolean removeLineBreaks)} method for a full description of this property.
+	 * 
+	 * @return <code>true</code> if all non-essential line breaks are removed, otherwise <code>false</code>.
+	 */
+	boolean getRemoveLineBreaks() {
+		return removeLineBreaks;
+	}
+
 	/**
 	 * Sets whether all elements are to be indented, including {@linkplain HTMLElements#getInlineLevelElementNames() inline-level elements} and those with preformatted contents.
 	 * <p>
@@ -231,7 +263,7 @@ public final class SourceFormatter implements CharStreamSource {
 	 * <p>
 	 * The default is to use the same new line string as is used in the source document, which is determined via the {@link Source#getNewLine()} method.
 	 * If the source document does not contain any new lines, a "best guess" is made by either taking the new line string of a previously parsed document,
-	 * or using the value from {@link Config#NewLine}.
+	 * or using the value from the static {@link Config#NewLine} property.
 	 * <p>
 	 * Specifying a <code>null</code> argument resets the property to its default value, which is to use the same new line string as is used in the source document.
 	 * 
@@ -263,6 +295,7 @@ public final class SourceFormatter implements CharStreamSource {
 		private final String indentString;
 		private final boolean tidyTags;
 		private final boolean collapseWhiteSpace;
+		private final boolean removeLineBreaks; // Indicates whether all non-essential line breaks are removed. Must be used with collapseWhiteSpace=true.
 		private final boolean indentAllElements;
 		private final boolean indentScriptElements; // at present this parameter is tied to indentAllElements.  SCRIPT elements need to be inline to keep functional equivalency of output
 		private final String newLine;
@@ -271,19 +304,21 @@ public final class SourceFormatter implements CharStreamSource {
 		private Tag nextTag;
 		private int index;
 	
-		public Processor(final Segment segment, final String indentString, final boolean tidyTags, final boolean collapseWhiteSpace, final boolean indentAllElements, final boolean indentAllScripts, final String newLine) {
+		public Processor(final Segment segment, final String indentString, final boolean tidyTags, final boolean collapseWhiteSpace, final boolean removeLineBreaks, final boolean indentAllElements, final boolean indentScriptElements, final String newLine) {
 			this.segment=segment;
 			sourceText=segment.source.toString();
 			this.indentString=indentString;
 			this.tidyTags=tidyTags;
-			this.collapseWhiteSpace=collapseWhiteSpace;
+			this.collapseWhiteSpace=collapseWhiteSpace || removeLineBreaks;
+			this.removeLineBreaks=removeLineBreaks;
 			this.indentAllElements=indentAllElements;
-			this.indentScriptElements=indentAllScripts;
+			this.indentScriptElements=indentScriptElements;
 			this.newLine=newLine;
 		}
 	
 		public void writeTo(final Writer writer) throws IOException {
 			this.writer=writer;
+			if (segment instanceof Source) ((Source)segment).fullSequentialParse();
 			nextTag=segment.source.findNextTag(segment.begin);
 			index=segment.begin;
 			writeContent(segment.end,segment.getChildElements(),0);
@@ -297,41 +332,50 @@ public final class SourceFormatter implements CharStreamSource {
 				final int elementBegin=element.begin;
 				if (elementBegin>=end) break;
 				if (indentAllElements) {
-					writeText(elementBegin,depth,false,false,false,collapseWhiteSpace);
+					writeText(elementBegin,depth);
 					writeElement(element,depth,end,false,false);
 				} else {
+					if (inlinable(element)) continue; // skip over elements that can be inlined.
+					writeText(elementBegin,depth);
 					final String elementName=element.getName();
-					if (!indent(element)) continue;
-					writeText(elementBegin,depth,false,false,false,collapseWhiteSpace);
 					if (elementName==HTMLElementName.PRE || elementName==HTMLElementName.TEXTAREA) {
 						writeElement(element,depth,end,true,true);
 					} else if (elementName==HTMLElementName.SCRIPT) {
 						writeElement(element,depth,end,true,false);
 					} else {
-						writeElement(element,depth,end,false,!containsNonInlineLevelChildElements(element));
+						writeElement(element,depth,end,false,!removeLineBreaks && containsOnlyInlineLevelChildElements(element));
 					}
 				}
 			}
-			writeText(end,depth,false,false,false,collapseWhiteSpace);
+			writeText(end,depth);
 		}
 	
-		private boolean indent(final Element element) {
+		private boolean inlinable(final Element element) {
+			// returns true if the specified element should be inlined
 			final StartTagType startTagType=element.getStartTag().getStartTagType();
-			if (startTagType==StartTagType.DOCTYPE_DECLARATION) return true;
-			if (startTagType!=StartTagType.NORMAL) return false;
+			if (startTagType==StartTagType.DOCTYPE_DECLARATION) return false;
+			if (startTagType!=StartTagType.NORMAL) return true;
+			// element is a normal type
 			final String elementName=element.getName();
-			if (elementName==HTMLElementName.SCRIPT) return indentScriptElements;
-			if (!HTMLElements.getInlineLevelElementNames().contains(elementName)) return true;
-			return containsNonInlineLevelChildElements(element);
+			if (elementName==HTMLElementName.SCRIPT) return !indentScriptElements;
+			if (removeLineBreaks && !HTMLElements.getElementNames().contains(elementName)) return true; // inline non-HTML elements if removing line breaks
+			if (!HTMLElements.getInlineLevelElementNames().contains(elementName)) return false;
+			// element is inline type
+			if (removeLineBreaks) return true;
+			return containsOnlyInlineLevelChildElements(element); // only inline if it doesn't illegally contain non-inline elements
 		}
 	
-		private void writeText(final int end, int depth, final boolean beginInline, final boolean endInline, final boolean increaseIndentAfterFirstLineBreak, final boolean collapseWhiteSpace) throws IOException {
+		private void writeText(final int end, int depth) throws IOException {
 			// sets index to end
 			if (index==end) return;
 			while (Segment.isWhiteSpace(sourceText.charAt(index))) if (++index==end) return; // trim whitespace.
-			if (!beginInline) writeIndent(depth);
-			writeTextInline(end,depth,increaseIndentAfterFirstLineBreak,collapseWhiteSpace);
-			if (!endInline) writer.write(newLine);
+			writeIndent(depth);
+			if (collapseWhiteSpace) {
+				writeTextCollapseWhiteSpace(end,depth);
+			} else {
+				writeTextInline(end,depth,false);
+			}
+			writeFormattingNewLine();
 		}
 	
 		private void writeElement(final Element element, final int depth, final int end, final boolean preformatted, boolean renderContentInline) throws IOException {
@@ -343,10 +387,10 @@ public final class SourceFormatter implements CharStreamSource {
 			writeIndent(depth);
 			writeTag(startTag,depth,end);
 			if (index==end) {
-				writer.write(newLine);
+				writeFormattingNewLine();
 				return;
 			}
-			if (!renderContentInline) writer.write(newLine);
+			if (!renderContentInline) writeFormattingNewLine();
 			int contentEnd=element.getContentEnd();
 			if (end<contentEnd) contentEnd=end;
 			if (index<contentEnd) {
@@ -360,17 +404,16 @@ public final class SourceFormatter implements CharStreamSource {
 					}
 				} else {
 					if (renderContentInline) {
-						// Inline-level element
+						// Element contains only inline-level elements, so don't bother putting start and end tags on separate lines
 						if (collapseWhiteSpace) {
 							writeTextCollapseWhiteSpace(contentEnd,depth);
 						} else {
-							if (!writeTextInline(contentEnd,depth,true,false)) {
-								writer.write(newLine);
+							if (!writeTextInline(contentEnd,depth,true)) {
+								writeFormattingNewLine();
 								renderContentInline=false;
 							}
 						}
 					} else {
-						// Block-level element
 						writeContent(contentEnd,element.getChildElements(),depth+1);
 					}
 				}
@@ -379,9 +422,9 @@ public final class SourceFormatter implements CharStreamSource {
 				if (!renderContentInline) writeIndent(depth);
 				// assert index=endTag.begin
 				writeTag(endTag,depth,end);
-				writer.write(newLine);
+				writeFormattingNewLine();
 			} else if (renderContentInline) {
-				writer.write(newLine);
+				writeFormattingNewLine();
 			}
 		}
 	
@@ -396,31 +439,36 @@ public final class SourceFormatter implements CharStreamSource {
 		private void writeIndentedScriptContent(final int end, final int depth) throws IOException {
 			// sets index to end
 			// assert index < end
+			if (removeLineBreaks) {
+				writeTextRemoveIndentation(end);
+				return;
+			}
 			int startOfLinePos=getStartOfLinePos(end,false);
 			if (index==end) return;
 			if (startOfLinePos==-1) {
 				// Script started on same line as start tag.  Use the start of the next line to determine the original indent.
 				writeIndent(depth);
 				writeLineKeepWhiteSpace(end,depth);
-				writer.write(newLine);
+				writeEssentialNewLine();
 				if (index==end) return;
 				startOfLinePos=getStartOfLinePos(end,true);
 				if (index==end) return;
 			}
 			writeTextPreserveIndentation(end,depth,index-startOfLinePos);
-			writer.write(newLine);
+			writeEssentialNewLine();
 		}
 	
 		private boolean writeTextPreserveIndentation(final int end, final int depth) throws IOException {
 			// sets index to end
 			// returns true if all text was on one line, otherwise false
 			// assert index < end
+			if (removeLineBreaks) return writeTextRemoveIndentation(end);
 			// Use the start of the next line to determine the original indent.
 			writeLineKeepWhiteSpace(end,depth);
 			if (index==end) return true;
 			int startOfLinePos=getStartOfLinePos(end,true);
 			if (index==end) return true;
-			writer.write(newLine);
+			writeEssentialNewLine();
 			writeTextPreserveIndentation(end,depth+1,index-startOfLinePos);
 			return false;
 		}
@@ -437,13 +485,32 @@ public final class SourceFormatter implements CharStreamSource {
 					if (!(ch==' ' || ch=='\t')) break;
 					if (++index==end) return;
 				}
-				writer.write(newLine);
+				writeEssentialNewLine();
 				// Insert our indent:
 				writeIndent(depth);
 				// Write the rest of the line including any indent greater than the first line's indent:
 				writeLineKeepWhiteSpace(end,depth);
 			}
 		}
+
+		private boolean writeTextRemoveIndentation(final int end) throws IOException {
+			// assert index < end
+			// sets index to end
+			writeLineKeepWhiteSpace(end,0);
+			if (index==end) return true;
+			while (index!=end) {
+				// Skip over the original indent:
+				while (true) {
+					final char ch=sourceText.charAt(index);
+					if (!(ch==' ' || ch=='\t')) break;
+					if (++index==end) return false;
+				}
+				writeEssentialNewLine();
+				// Write the rest of the line including any indent greater than the first line's indent:
+				writeLineKeepWhiteSpace(end,0);
+			}
+			return false;
+		}
 	
 		private int getStartOfLinePos(final int end, final boolean atStartOfLine) {
 			// returns the starting position of the next complete line containing text, or -1 if texts starts on the current line (hence not a complete line).
@@ -466,7 +533,7 @@ public final class SourceFormatter implements CharStreamSource {
 				final int subsequentLineDepth=depth+1;
 				do {
 					while (Segment.isWhiteSpace(text.charAt(i))) if (++i>=textLength) return; // trim whitespace.
-					writer.write(newLine);
+					writeEssentialNewLine();
 					writeIndent(subsequentLineDepth);
 					i=writeSpecifiedLine(text,i);
 				} while (i<textLength);
@@ -490,32 +557,22 @@ public final class SourceFormatter implements CharStreamSource {
 			}
 		}
 	
-		private boolean writeTextInline(final int end, int depth, final boolean increaseIndentAfterFirstLineBreak, final boolean collapseWhiteSpace) throws IOException {
+		private boolean writeTextInline(final int end, int depth, final boolean increaseIndentAfterFirstLineBreak) throws IOException {
 			// returns true if all text was on one line, otherwise false
 			// sets index to end
 			// assert index < end
-			writeLine(end,depth,collapseWhiteSpace);
+			writeLineKeepWhiteSpace(end,depth);
 			if (index==end) return true;
 			final int subsequentLineDepth=increaseIndentAfterFirstLineBreak ? depth+1 : depth;
 			do {
 				while (Segment.isWhiteSpace(sourceText.charAt(index))) if (++index==end) return false; // trim whitespace.
-				writer.write(newLine);
+				writeEssentialNewLine(); // essential because we might be inside a tag attribute value.  If new lines in normal text aren't required this method wouldn't have been called.
 				writeIndent(subsequentLineDepth);
-				writeLine(end,subsequentLineDepth,collapseWhiteSpace);
+				writeLineKeepWhiteSpace(end,subsequentLineDepth);
 			} while (index<end);
 			return false;
 		}
 	
-		private void writeLine(final int end, final int depth, final boolean collapseWhiteSpace) throws IOException {
-			// sets index to the position following the first line break character(s), or to end if collapseWhiteSpace or the text contains no line breaks.
-			// assert index < end
-			if (collapseWhiteSpace) {
-				writeTextCollapseWhiteSpace(end,depth);
-			} else {
-				writeLineKeepWhiteSpace(end,depth);
-			}
-		}
-	
 		private void writeLineKeepWhiteSpace(final int end, final int depth) throws IOException {
 			// Writes the first line from the source text starting from index, ending at the specified end position.
 			// The line break characters are not written.
@@ -591,9 +648,9 @@ public final class SourceFormatter implements CharStreamSource {
 			// assert index==tag.begin
 			// assert index < end
 			nextTag=tag.findNextTag();
-			final int tagEnd=(end>tag.end) ? tag.end : end;
+			final int tagEnd=(tag.end<end) ? tag.end : end;
 			// assert index < tagEnd
-			if (tag.getTagType()==StartTagType.COMMENT || tag.getTagType()==StartTagType.CDATA_SECTION) {
+			if (tag.getTagType()==StartTagType.COMMENT || tag.getTagType()==StartTagType.CDATA_SECTION || tag.getTagType().isServerTag()) {
 				writeTextPreserveIndentation(tagEnd,depth);
 			} else if (tidyTags) {
 				final String tidyTag=tag.tidy();
@@ -603,20 +660,30 @@ public final class SourceFormatter implements CharStreamSource {
 					writeSpecifiedTextInline(tidyTag,depth);
 				index=tagEnd;
 			} else {
-				writeTextInline(tagEnd,depth,true,false);
+				writeTextInline(tagEnd,depth,true); // Write tag keeping linefeeds. This will add an indent to any attribute values containing linefeeds, but the normal situation where line breaks are between attributes will look nice.
 			}
 			if (end<=tag.end || !(tag instanceof StartTag)) return;
 			if ((tag.name==HTMLElementName.SCRIPT && !indentScriptElements) || tag.getTagType().isServerTag()) {
-				// this is a server start tag, we may need to write the whole server element:
+				// NOTE SERVER ELEMENTS CONTAINING NON-INLINE TAGS WILL NOT FORMAT PROPERLY. NEED TO INVESTIGATE INCLUDING SUCH SERVER ELEMENTS IN DOCUMENT HIERARCHY.
+				// this is a script or server start tag, we may need to write the whole element:
 				final Element element=tag.getElement();
 				final EndTag endTag=element.getEndTag();
 				if (endTag==null) return;
 				final int contentEnd=(end<endTag.begin) ? end : endTag.begin;
-				final boolean singleLineContent=(index==contentEnd) ? true : writeTextPreserveIndentation(contentEnd,depth);
-				//final boolean singleLineContent=(index==contentEnd) ? true : writeTextInline(contentEnd,depth+1,false,false); // use this line instead of previous if indentation shouldn't be preserved in server elements.
+				boolean singleLineContent=true;
+				if (index!=contentEnd) {
+					// elementContainsMarkup should be made into a TagType property one day.
+					// for the time being assume all server element content is code, although this is not true for some Mason elements.
+					final boolean elementContainsMarkup=false;
+					if (elementContainsMarkup) {
+						singleLineContent=writeTextInline(contentEnd,depth+1,false);
+					} else {
+						singleLineContent=writeTextPreserveIndentation(contentEnd,depth);
+					}
+				}
 				if (endTag.begin>=end) return;
 				if (!singleLineContent) {
-					writer.write(newLine);
+					writeEssentialNewLine(); // some server or client side scripting languages might need the final new line
 					writeIndent(depth);
 				}
 				// assert index==endTag.begin
@@ -625,20 +692,28 @@ public final class SourceFormatter implements CharStreamSource {
 		}
 		
 	  private void writeIndent(final int depth) throws IOException {
-			for (int x=0; x<depth; x++) writer.write(indentString);
+			if (!removeLineBreaks) for (int x=0; x<depth; x++) writer.write(indentString);
 	  }
 	
-		private boolean containsNonInlineLevelChildElements(final Element element) {
-			// returns true if the element contains any non-inline-level elements or SCRIPT elements.
+		private void writeFormattingNewLine() throws IOException {
+			if (!removeLineBreaks) writer.write(newLine);
+		}
+
+		private void writeEssentialNewLine() throws IOException {
+			writer.write(newLine);
+		}
+	
+		private boolean containsOnlyInlineLevelChildElements(final Element element) {
+			// returns true if the element contains only inline-level elements except for SCRIPT elements.
 			final Collection childElements=element.getChildElements();
-			if (childElements==Collections.EMPTY_LIST) return false;
+			if (childElements.isEmpty()) return true;
 			for (final Iterator i=childElements.iterator(); i.hasNext();) {
 				final Element childElement=(Element)i.next();
 				final String elementName=childElement.getName();
-				if (elementName==HTMLElementName.SCRIPT || !HTMLElements.getInlineLevelElementNames().contains(elementName)) return true;
-				if (containsNonInlineLevelChildElements(childElement)) return true;
+				if (elementName==HTMLElementName.SCRIPT || !HTMLElements.getInlineLevelElementNames().contains(elementName)) return false;
+				if (!containsOnlyInlineLevelChildElements(childElement)) return false;
 			}
-			return false;
+			return true;
 		}
 	}
 }
