@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -99,47 +99,61 @@ public final class StartTag extends Tag {
 		this.startTagType=startTagType;
 	}
 
+	// only used to create Tag.NOT_CACHED
+	StartTag() {
+		attributes=null;
+		startTagType=null;
+	}
+
 	/**
 	 * Returns the {@linkplain Element element} that is started by this start tag.
 	 * Guaranteed not <code>null</code>.
-	 * <h4>Example 1: Elements for which the {@linkplain HTMLElements#getEndTagRequiredElementNames() end tag is required}</h4>
-	 * <pre>
-	 * 1. &lt;div&gt;
-	 * 2.   &lt;div&gt;
-	 * 3.     &lt;div&gt;
-	 * 4.       &lt;div&gt;This is line 4&lt;/div&gt;
-	 * 5.     &lt;/div&gt;
-	 * 6.     &lt;div&gt;This is line 6&lt;/div&gt;
-	 * 7.   &lt;/div&gt;</pre>
-	 * <ul>
-	 *  <li>The start tag on line 1 returns an empty element spanning only the start tag.
-	 *   This is because the end tag of a <code>&lt;div&gt;</code> element is required,
-	 *   making the sample code invalid as all the end tags are matched with other start tags.
-	 *  <li>The start tag on line 2 returns an element spanning to the end of line 7.
-	 *  <li>The start tag on line 3 returns an element spanning to the end of line 5.
-	 *  <li>The start tag on line 4 returns an element spanning to the end of line 4.
-	 *  <li>The start tag on line 6 returns an element spanning to the end of line 6.
-	 * </ul>
-	 * <h4>Example 2: Elements for which the {@linkplain HTMLElements#getEndTagOptionalElementNames() end tag is optional}</h4>
-	 * <pre>
-	 * 1. &lt;ul&gt;
-	 * 2.   &lt;li&gt;item 1
-	 * 3.   &lt;li&gt;item 2
-	 * 4.     &lt;ul&gt;
-	 * 5.       &lt;li&gt;subitem 1&lt;/li&gt;
-	 * 6.       &lt;li&gt;subitem 2
-	 * 7.     &lt;/ul&gt;
-	 * 8.   &lt;li&gt;item 3&lt;/li&gt;
-	 * 9. &lt;/ul&gt;</pre>
-	 * <ul>
-	 *  <li>The start tag on line 1 returns an element spanning to the end of line 9.
-	 *  <li>The start tag on line 2 returns an element spanning to the start of the <code>&lt;li&gt;</code> start tag on line 3.
-	 *  <li>The start tag on line 3 returns an element spanning to the start of the <code>&lt;li&gt;</code> start tag on line 8.
-	 *  <li>The start tag on line 4 returns an element spanning to the end of line 7.
-	 *  <li>The start tag on line 5 returns an element spanning to the end of line 5.
-	 *  <li>The start tag on line 6 returns an element spanning to the start of the <code>&lt;/ul&gt;</code> end tag on line 7.
-	 *  <li>The start tag on line 8 returns an element spanning to the end of line 8.
-	 * </ul>
+ 	 * <p>
+	 * <dl>
+	 *  <dt>Example 1: Elements for which the {@linkplain HTMLElements#getEndTagRequiredElementNames() end tag is required}</dt>
+	 *  <dd>
+	 *   <pre>
+	 *    1. &lt;div&gt;
+	 *    2.   &lt;div&gt;
+	 *    3.     &lt;div&gt;
+	 *    4.       &lt;div&gt;This is line 4&lt;/div&gt;
+	 *    5.     &lt;/div&gt;
+	 *    6.     &lt;div&gt;This is line 6&lt;/div&gt;
+	 *    7.   &lt;/div&gt;</pre>
+	 *   <ul>
+	 *    <li>The start tag on line 1 returns an empty element spanning only the start tag.
+	 *     This is because the end tag of a <code>&lt;div&gt;</code> element is required,
+	 *     making the sample code invalid as all the end tags are matched with other start tags.
+	 *    <li>The start tag on line 2 returns an element spanning to the end of line 7.
+	 *    <li>The start tag on line 3 returns an element spanning to the end of line 5.
+	 *    <li>The start tag on line 4 returns an element spanning to the end of line 4.
+	 *    <li>The start tag on line 6 returns an element spanning to the end of line 6.
+	 *   </ul>
+	 *   <p>
+	 *  </dd>
+	 *  <dt>Example 2: Elements for which the {@linkplain HTMLElements#getEndTagOptionalElementNames() end tag is optional}</dt>
+	 *  <dd>
+	 *   <pre>
+	 *    1. &lt;ul&gt;
+	 *    2.   &lt;li&gt;item 1
+	 *    3.   &lt;li&gt;item 2
+	 *    4.     &lt;ul&gt;
+	 *    5.       &lt;li&gt;subitem 1&lt;/li&gt;
+	 *    6.       &lt;li&gt;subitem 2
+	 *    7.     &lt;/ul&gt;
+	 *    8.   &lt;li&gt;item 3&lt;/li&gt;
+	 *    9. &lt;/ul&gt;</pre>
+	 *   <ul>
+	 *    <li>The start tag on line 1 returns an element spanning to the end of line 9.
+	 *    <li>The start tag on line 2 returns an element spanning to the start of the <code>&lt;li&gt;</code> start tag on line 3.
+	 *    <li>The start tag on line 3 returns an element spanning to the start of the <code>&lt;li&gt;</code> start tag on line 8.
+	 *    <li>The start tag on line 4 returns an element spanning to the end of line 7.
+	 *    <li>The start tag on line 5 returns an element spanning to the end of line 5.
+	 *    <li>The start tag on line 6 returns an element spanning to the start of the <code>&lt;/ul&gt;</code> end tag on line 7.
+	 *    <li>The start tag on line 8 returns an element spanning to the end of line 8.
+	 *   </ul>
+	 *  </dd>
+	 * </dl>
 	 *
 	 * @return the {@linkplain Element element} that is started by this start tag.
 	 */
@@ -148,33 +162,54 @@ public final class StartTag extends Tag {
 			final EndTag endTag=findEndTagInternal();
 			element=new Element(source,this,endTag);
 			if (endTag!=null) {
-				if (endTag.element!=Element.NOT_CACHED)
-					if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(endTag.begin).appendTo(new StringBuffer(200).append("End tag ").append(endTag).append(" at ")).append(" terminates more than one element").toString()); // presumably impossible, but log it just in case
+				if (endTag.element!=Element.NOT_CACHED) {
+					// This is presumably impossible, except in certain circumstances where the cache was cleared, such as if the parser decides to do a full sequential parse after some tags have already been found.
+					// If the existing element and the current element are not the same, log it.
+					if (source.logger.isInfoEnabled() && !element.equals(endTag.element)) source.logger.info(source.getRowColumnVector(endTag.begin).appendTo(new StringBuffer(200).append("End tag ").append(endTag).append(" at ")).append(" terminates more than one element").toString()); 
+				}
 				endTag.element=element;
 			}
 		}
 		return element;
 	}
 
+	/**
+	 * Indicates whether this start tag is an <a target="_blank" href="http://www.w3.org/TR/REC-xml#dt-eetag">empty-element tag</a>.
+	 * <p>
+	 * This property checks that the the tag is {@linkplain #isSyntacticalEmptyElementTag() syntactically an empty-element tag},
+	 * but in addition checks that the {@linkplain #getName() name} of the tag is not one that is defined in the HTML specification to have a
+	 * {@linkplain HTMLElements#getEndTagRequiredElementNames() required} or {@linkplain HTMLElements#getEndTagOptionalElementNames() optional} end tag,
+	 * which the major browsers do not recognise as empty-element tags, even in an <a target="_blank" href="http://www.w3.org/TR/xhtml1/">XHTML</a> document.
+	 * <p>
+	 * This is equivalent to:<br />
+	 * {@link #isSyntacticalEmptyElementTag()}<code> && !(</code>{@link HTMLElements#getEndTagOptionalElementNames()}<code>.contains(</code>{@link #getName() getName()}<code>) || </code>{@link HTMLElements#getEndTagRequiredElementNames()}<code>.contains(</code>{@link #getName() getName()}<code>))</code>.
+	 * <p>
+	 * Prior to version 2.6, the implementation of this method was equivalent to {@link #isSyntacticalEmptyElementTag()}.
+	 *
+	 * @return <code>true</code> if this start tag is an <a target="_blank" href="http://www.w3.org/TR/REC-xml#dt-eetag">empty-element tag</a>, otherwise <code>false</code>.
+	 */
+	public boolean isEmptyElementTag() {
+		return isSyntacticalEmptyElementTag() && !HTMLElements.isClosingSlashIgnored(name);
+	}
+
 	/**
 	 * Indicates whether this start tag is syntactically an <a target="_blank" href="http://www.w3.org/TR/REC-xml#dt-eetag">empty-element tag</a>.
 	 * <p>
 	 * This is signified by the characters "/&gt;" at the end of the start tag.
 	 * <p>
-	 * Only a {@linkplain StartTagType#NORMAL normal} start tag can be an empty-element tag.
+	 * Only a {@linkplain StartTagType#NORMAL normal} start tag can be syntactically an empty-element tag.
 	 * <p>
 	 * This property simply reports whether the syntax of the start tag is consistent with that of an empty-element tag,
 	 * it does not guarantee that this start tag's {@linkplain #getElement() element} is actually {@linkplain Element#isEmpty() empty}.
 	 * <p>
 	 * This possible discrepancy reflects the way major browsers interpret illegal empty element tags used in
 	 * <a href="HTMLElements.html#HTMLElement">HTML elements</a>, and is explained further in the documentation of the
-	 * {@link Element#isEmptyElementTag()} property.
-	 * <p>
-	 * Compare this property with the {@link Element#isEmptyElementTag()} property, which does check that the element is actually empty.
+	 * {@link #isEmptyElementTag()} property.
 	 *
 	 * @return <code>true</code> if this start tag is syntactically an <a target="_blank" href="http://www.w3.org/TR/REC-xml#dt-eetag">empty-element tag</a>, otherwise <code>false</code>.
+	 * @see #isEmptyElementTag()
 	 */
-	public boolean isEmptyElementTag() {
+	public boolean isSyntacticalEmptyElementTag() {
 		return startTagType==StartTagType.NORMAL && source.charAt(end-2)=='/';
 	}
 
@@ -301,7 +336,7 @@ public final class StartTag extends Tag {
 	 *   {@linkplain StartTagType#getCorrespondingEndTagType() corresponding end tag type}.
 	 *  <li>The {@linkplain #getName() name} of this start tag indicates it is the start of an
 	 *   <a href="Element.html#HTML">HTML element</a> whose {@linkplain HTMLElements#getEndTagForbiddenElementNames() end tag is forbidden}.
-	 *  <li>This start tag is syntactically an {@linkplain #isEmptyElementTag() empty-element tag} and its
+	 *  <li>This start tag is {@linkplain #isSyntacticalEmptyElementTag() syntactically an empty-element tag} and its
 	 *   {@linkplain #getName() name} indicates it is the start of a <a href="HTMLElements.html#NonHTMLElement">non-HTML element</a>.
 	 * </ul>
 	 * <p>
@@ -315,7 +350,7 @@ public final class StartTag extends Tag {
 			return getStartTagType().getCorrespondingEndTagType()==null;
 		if (HTMLElements.getEndTagForbiddenElementNames().contains(name)) return true;
 		if (HTMLElements.getElementNames().contains(name)) return false;
-		return isEmptyElementTag();
+		return isSyntacticalEmptyElementTag();
 	}
 
 	/**
@@ -327,7 +362,7 @@ public final class StartTag extends Tag {
 	 *   {@linkplain StartTagType#getCorrespondingEndTagType() corresponding end tag type}.
 	 *  <li>The {@linkplain #getName() name} of this start tag indicates it is the start of an
 	 *   <a href="Element.html#HTML">HTML element</a> whose {@linkplain HTMLElements#getEndTagRequiredElementNames() end tag is required}.
-	 *  <li>This start tag is syntactically NOT an {@linkplain #isEmptyElementTag() empty-element tag} and its
+	 *  <li>This start tag is NOT {@linkplain #isSyntacticalEmptyElementTag() syntactically an empty-element tag} and its
 	 *   {@linkplain #getName() name} indicates it is the start of a <a href="HTMLElements.html#NonHTMLElement">non-HTML element</a>.
 	 * </ul>
 	 *
@@ -338,7 +373,7 @@ public final class StartTag extends Tag {
 			return getStartTagType().getCorrespondingEndTagType()!=null;
 		if (HTMLElements.getEndTagRequiredElementNames().contains(name)) return true;
 		if (HTMLElements.getElementNames().contains(name)) return false;
-		return !isEmptyElementTag();
+		return !isSyntacticalEmptyElementTag();
 	}
 
 	// Documentation inherited from Tag
@@ -365,7 +400,9 @@ public final class StartTag extends Tag {
 	 * The tidying of the tag is carried out as follows:
 	 * <ul>
 	 *  <li>if this start tag is of a {@linkplain #getStartTagType() type} that does not {@linkplain StartTagType#hasAttributes() have attributes},
-	 *   then the original {@linkplain Segment#toString() source text} is returned.
+	 *   then the original {@linkplain Segment#toString() source text} of the enture tag is returned.
+	 *  <li>if this start tag contain any {@linkplain TagType#isServerTag() server tags} outside of an attribute value,
+	 *   then the original {@linkplain Segment#toString() source text} of the entire tag is returned.
 	 *  <li>name converted to lower case if the <code>toXHTML</code> argument is <code>true</code> and this is a {@linkplain StartTagType#NORMAL normal} start tag
 	 *  <li>attributes separated by a single space
 	 *  <li>attribute names in original case
@@ -378,20 +415,27 @@ public final class StartTag extends Tag {
 	 * <p>
 	 * The <code>toXHTML</code> parameter determines only whether the name is converted to lower case for {@linkplain StartTagType#NORMAL normal} tags.
 	 * In all other respects the generated tag is already valid XHTML.
-	 * <h4>Example:</h4>
-	 * The following source text:
 	 * <p>
-	 * <code>&lt;INPUT name=Company value='G&amp;uuml;nter O&amp#39;Reilly &amp;amp Associ&eacute;s'&gt;</code>
-	 * <p>
-	 * produces the following regenerated HTML:
-	 * <p>
-	 * <code>&lt;input name="Company" value="G&amp;uuml;nter O'Reilly &amp;amp; Associ&amp;eacute;s" /&gt;</code>
+	 * <dl>
+	 *  <dt>Example:</dt>
+	 *  <dd>
+	 *   <p>
+	 *   The following source text:
+	 *   <blockquote class="code">
+	 *    <code>&lt;INPUT name=Company value='G&amp;uuml;nter O&amp#39;Reilly &amp;amp Associ&eacute;s'&gt;</code>
+	 *   </blockquote>
+	 *   produces the following regenerated HTML:
+	 *   <blockquote class="code">
+	 *    <code>&lt;input name="Company" value="G&amp;uuml;nter O'Reilly &amp;amp; Associ&amp;eacute;s" /&gt;</code>
+	 *   </blockquote>
+	 *  </dd>
+	 * </dl>
 	 *
 	 * @param toXHTML  specifies whether the output is XHTML.
 	 * @return an XML or XHTML representation of this start tag, or the {@linkplain Segment#toString() source text} if it is of a {@linkplain #getStartTagType() type} that does not {@linkplain StartTagType#hasAttributes() have attributes}.
 	 */
 	public String tidy(boolean toXHTML) {
-		if (attributes==null) return toString();
+		if (attributes==null || attributes.containsServerTagOutsideOfAttributeValue) return toString();
 		final StringBuffer sb=new StringBuffer();
 		sb.append('<');
 		if (toXHTML && startTagType==StartTagType.NORMAL) {
@@ -419,16 +463,25 @@ public final class StartTag extends Tag {
 	 * <a target="_blank" href="http://www.w3.org/TR/REC-xml#dt-eetag">empty-element tag</a>,
 	 * in which case a slash is inserted before the closing angle bracket, separated from the name
 	 * or last attribute by a single space.
-	 * <h4>Example:</h4>
-	 * The following code:
+	 * <p>
+	 * <dl>
+	 *  <dt>Example:</dt>
+	 *  <dd>
+	 *   <p>
+	 *   The following code:
+	 *   <blockquote class="code">
 	 * <pre>
 	 * LinkedHashMap attributesMap=new LinkedHashMap();
 	 * attributesMap.put("name","Company");
 	 * attributesMap.put("value","G\n00fcnter O'Reilly & Associ&eacute;s");
 	 * System.out.println(StartTag.generateHTML("INPUT",attributesMap,true));</pre>
-	 * generates the following output:
-	 * <p>
-	 * <code>&lt;INPUT name="Company" value="G&amp;uuml;nter O'Reilly &amp;amp; Associ&amp;eacute;s" /&gt;</code>
+	 *   </blockquote>
+	 *   generates the following output:
+	 *   <blockquote class="code">
+	 *    <code>&lt;INPUT name="Company" value="G&amp;uuml;nter O'Reilly &amp;amp; Associ&amp;eacute;s" /&gt;</code>
+	 *   </blockquote>
+	 *  </dd>
+	 * </dl>
 	 *
 	 * @param tagName  the name of the start tag.
 	 * @param attributesMap  a map containing attribute name/value pairs.
@@ -462,7 +515,7 @@ public final class StartTag extends Tag {
 			sb.append(this);
 		} else {
 			sb.append('<').append(getNameSegment()).append(' ');
-			if (isEmptyElementTag()) sb.append('/');
+			if (isSyntacticalEmptyElementTag()) sb.append('/');
 			sb.append(startTagType.getClosingDelimiter());
 		}
 		return sb;
@@ -476,6 +529,7 @@ public final class StartTag extends Tag {
 	private EndTag findEndTagInternal() {
 		boolean checkForEmptyElementTag=true;
 		// A missing optional end tag returns a zero length EndTag instead of null
+		final EndTagType endTagType=startTagType.getCorrespondingEndTagType();
 		if (startTagType==StartTagType.NORMAL) {
 			final HTMLElementTerminatingTagNameSets terminatingTagNameSets=HTMLElements.getTerminatingTagNameSets(name);
 			if (terminatingTagNameSets!=null) // end tag is optional
@@ -483,17 +537,17 @@ public final class StartTag extends Tag {
 			if (HTMLElements.getEndTagForbiddenElementNames().contains(name)) // end tag is forbidden
 				return null;
 			checkForEmptyElementTag=!HTMLElements.getEndTagRequiredElementNames().contains(name); // check for empty-element tags if tag is not an HTML element
-			if (checkForEmptyElementTag && isEmptyElementTag()) // non-html empty-element tag
+			if (checkForEmptyElementTag && isSyntacticalEmptyElementTag()) // non-html empty-element tag
 				return null; 
-		} else if (startTagType.getCorrespondingEndTagType()==null) {
+		} else if (endTagType==null) {
 			return null;
 		}
 		// This is either a start tag type other than NORMAL that requires an end tag, or an HTML element tag that requires an end tag,
 		// or a non-HTML element tag that is not an empty-element tag.
 		// In all of these cases the end tag is required.
-		final EndTag nextEndTag=source.findNextEndTag(end,name,startTagType.getCorrespondingEndTagType());
+		final EndTag nextEndTag=source.findNextEndTag(end,endTagType.getEndTagName(name),endTagType);
 		if (nextEndTag!=null) {
-			if (HTMLElements.END_TAG_REQUIRED_NESTING_FORBIDDEN_SET.contains(name)) {
+			if (startTagType==StartTagType.NORMAL && HTMLElements.END_TAG_REQUIRED_NESTING_FORBIDDEN_SET.contains(name)) {
 				final StartTag nextStartTag=source.findNextStartTag(end,name);
 				if (nextStartTag==null || nextStartTag.begin>nextEndTag.begin) return nextEndTag;
 				if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append("StartTag at ")).append(" missing required end tag - invalid nested start tag encountered before end tag").toString());
@@ -501,7 +555,7 @@ public final class StartTag extends Tag {
 				// This is how IE and Mozilla treat illegally nested A elements, but other elements may vary.
 				return new EndTag(source,nextStartTag.begin,nextStartTag.begin,EndTagType.NORMAL,name);
 			}
-			final Segment[] findResult=findEndTag(nextEndTag,checkForEmptyElementTag);
+			final Segment[] findResult=findEndTag(nextEndTag,checkForEmptyElementTag,Tag.isXMLName(name));
 			if (findResult!=null) return (EndTag)findResult[0];
 		}
 		if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append("StartTag at ")).append(" missing required end tag").toString());
@@ -511,7 +565,7 @@ public final class StartTag extends Tag {
 	private EndTag findOptionalEndTag(final HTMLElementTerminatingTagNameSets terminatingTagNameSets) {
 		int pos=end;
 		while (pos<source.end) {
-			final Tag tag=Tag.findPreviousOrNextTag(source,pos,false);
+			final Tag tag=Tag.findNextTag(source,pos);
 			if (tag==null) break;
 			Set terminatingTagNameSet;
 			if (tag instanceof EndTag) {
@@ -533,23 +587,36 @@ public final class StartTag extends Tag {
 		return new EndTag(source,source.end,source.end,EndTagType.NORMAL,name);
 	}
 
-	static StartTag findPreviousOrNext(final Source source, final int pos, final String searchName, final boolean isXMLTagName, final boolean previous) {
-		// searchName is already in lower case
-		if (searchName==null) return findPreviousOrNext(source,pos,previous);
+	static char[] getStartDelimiterCharArray(final String searchName) {
 		if (searchName.length()==0) throw new IllegalArgumentException("searchName argument must not be zero length");
-		final char[] startDelimiterCharArray=new char[searchName.length()+1];
-		startDelimiterCharArray[0]='<';
-		for (int i=1; i<startDelimiterCharArray.length; i++) startDelimiterCharArray[i]=searchName.charAt(i-1);
-		if (startDelimiterCharArray[1]=='/') throw new IllegalArgumentException("searchName argument \""+searchName+"\" must not start with '/'");
+		final char[] startDelimiterCharArray=Util.getConcatenatedCharArray(StartTagType.START_DELIMITER_PREFIX,searchName);
+		if (startDelimiterCharArray[StartTagType.START_DELIMITER_PREFIX.length()]=='/') throw new IllegalArgumentException("searchName argument \""+searchName+"\" must not start with '/'");
+		return startDelimiterCharArray;
+	}
+
+	static StartTag findPrevious(final Source source, final int pos, final String searchName, final StartTagType searchStartTagType) {
+		return findPrevious(source,pos,searchName,searchStartTagType,searchStartTagType==StartTagType.NORMAL ? Tag.isXMLName(searchName) : true);
+	}
+
+	static StartTag findPrevious(final Source source, final int pos, final String searchName, final StartTagType searchStartTagType, final boolean isXMLTagName) {
+		// searchName is already in lower case
+		if (searchName==null) return (StartTag)source.findPreviousTag(pos,searchStartTagType);
+		final char[] startDelimiterCharArray=getStartDelimiterCharArray(searchName);
 		try {
 			final ParseText parseText=source.getParseText();
 			int begin=pos;
 			do {
-				begin=previous?parseText.lastIndexOf(startDelimiterCharArray,begin):parseText.indexOf(startDelimiterCharArray,begin);
+				begin=parseText.lastIndexOf(startDelimiterCharArray,begin);
 				if (begin==-1) return null;
-				final StartTag startTag=(StartTag)Tag.getTagAt(source,begin);
-				if (startTag==null || (isXMLTagName && startTag.isUnregistered())) continue;
-				if (startTag.startTagType.isNameAfterPrefixRequired() && startTag.name.length()>searchName.length()) {
+				final StartTag startTag=(StartTag)Tag.getTagAt(source,begin,false);
+				if (startTag==null) continue; // keep looking if it wasn't a start tag
+				if (searchStartTagType!=startTag.getStartTagType()) {
+					// The start tag is of the wrong type.  The only case in which we want to return it is if
+					// we are looking for a normal start tag, the found start tag is unregistered, and the search name is NOT a valid XML name.
+					// This allows users to search for some types of unregistered tags by name rather than having to register custom tag types.
+					if (searchStartTagType!=StartTagType.NORMAL || isXMLTagName || !startTag.isUnregistered()) continue;
+				}
+				if (startTag.getStartTagType().isNameAfterPrefixRequired() && startTag.getName().length()>searchName.length()) {
 					// The name of the start tag is longer than the search name, and the type of tag indicates 
 					// that we are probably looking for an exact match.
 					// (eg searchName="a", startTag.name="applet" -> reject)
@@ -563,7 +630,53 @@ public final class StartTag extends Tag {
 					if (lastSearchNameChar!=':' && isXMLNameChar(lastSearchNameChar)) continue;
 				}
 				return startTag;
-			} while (previous ? (begin-=2)>=0 : (begin+=1)<source.end);
+			} while ((begin-=2)>=0);
+		} catch (IndexOutOfBoundsException ex) {
+			// this should never happen during a find previous operation so rethrow it:
+			throw ex;
+		}
+		return null;
+	}
+
+	static StartTag findNext(final Source source, final int pos, final String searchName, final StartTagType searchStartTagType) {
+		return findNext(source,pos,searchName,searchStartTagType,searchStartTagType==StartTagType.NORMAL ? Tag.isXMLName(searchName) : true);
+	}
+
+	static StartTag findNext(final Source source, final int pos, final String searchName, final StartTagType searchStartTagType, final boolean isXMLTagName) {
+		// searchName is already in lower case, but may be null
+		// searchStartTagType must not be null
+		// isXMLTagName is only used if searchStartTagType==StartTagType.NORMAL
+		if (searchName==null) return (StartTag)source.findNextTag(pos,searchStartTagType);
+		final char[] startDelimiterCharArray=getStartDelimiterCharArray(searchName);
+		try {
+			final ParseText parseText=source.getParseText();
+			int begin=pos;
+			do {
+				begin=parseText.indexOf(startDelimiterCharArray,begin);
+				if (begin==-1) return null;
+				final StartTag startTag=(StartTag)Tag.getTagAt(source,begin,false);
+				if (startTag==null) continue; // keep looking if it wasn't a start tag
+				if (searchStartTagType!=startTag.getStartTagType()) {
+					// The start tag is of the wrong type.  The only case in which we want to return it is if
+					// we are looking for a normal start tag, the found start tag is unregistered, and the search name is NOT a valid XML name.
+					// This allows users to search for some types of unregistered tags by name rather than having to register custom tag types.
+					if (searchStartTagType!=StartTagType.NORMAL || isXMLTagName || !startTag.isUnregistered()) continue;
+				}
+				if (startTag.getStartTagType().isNameAfterPrefixRequired() && startTag.getName().length()>searchName.length()) {
+					// The name of the start tag is longer than the search name, and the type of tag indicates 
+					// that we are probably looking for an exact match.
+					// (eg searchName="a", startTag.name="applet" -> reject)
+					// We only require an exact match if the last character of the search name is part of the name, as the
+					// search name might be just the prefix of a server tag.
+					// (eg searchName="?", startTag.name="?abc" -> accept, but searchName="?a", startTag.name="?abc" -> reject)
+					// The only exception to this is if the last character of the search name is a colon (which also forms part of
+					// the name), but signifies that we want to search on the entire namespace.
+					// (eg searchName="o:", startTag.name="o:p" -> accept)
+					char lastSearchNameChar=searchName.charAt(searchName.length()-1);
+					if (lastSearchNameChar!=':' && isXMLNameChar(lastSearchNameChar)) continue;
+				}
+				return startTag;
+			} while ((begin+=1)<source.end);
 		} catch (IndexOutOfBoundsException ex) {
 			// this should only happen when the end of file is reached in the middle of a tag.
 			// we don't have to do anything to handle it as there are no more tags anyway.
@@ -571,12 +684,21 @@ public final class StartTag extends Tag {
 		return null;
 	}
 
-	static StartTag findPreviousOrNext(final Source source, int pos, final boolean previous) {
+	static StartTag findPrevious(final Source source, int pos) {
 		while (true) {
-			final Tag tag=Tag.findPreviousOrNextTag(source,pos,previous);
+			final Tag tag=Tag.findPreviousTag(source,pos);
 			if (tag==null) return null;
 			if (tag instanceof StartTag) return (StartTag)tag;
-			pos+=previous?-1:1;
+			pos-=1;
+		}
+	}
+
+	static StartTag findNext(final Source source, int pos) {
+		while (true) {
+			final Tag tag=Tag.findNextTag(source,pos);
+			if (tag==null) return null;
+			if (tag instanceof StartTag) return (StartTag)tag;
+			pos+=1;
 		}
 	}
 	
@@ -616,28 +738,28 @@ public final class StartTag extends Tag {
 		return null;
 	}
 
-	private Segment[] findEndTag(final EndTag nextEndTag, final boolean checkForEmptyElementTag) {
-		StartTag nextStartTag=source.findNextStartTag(end,name);
+	private Segment[] findEndTag(final EndTag nextEndTag, final boolean checkForEmptyElementTag, final boolean isXMLTagName) {
+		// assert nextEndTag!=null
+		StartTag nextStartTag=findNext(source,end,name,startTagType,isXMLTagName);
 		if (checkForEmptyElementTag) {
-			while (nextStartTag!=null && nextStartTag.isEmptyElementTag())
-				nextStartTag=source.findNextStartTag(nextStartTag.end,name);
+			while (nextStartTag!=null && nextStartTag.isSyntacticalEmptyElementTag())
+				nextStartTag=findNext(source,nextStartTag.end,name,startTagType,isXMLTagName);
 		}
-		return findEndTag(end,nextStartTag,nextEndTag,checkForEmptyElementTag);
+		return findEndTag(end,nextStartTag,nextEndTag,checkForEmptyElementTag,isXMLTagName);
 	}
 
-	private Segment[] findEndTag(final int afterPos, StartTag nextStartTag, EndTag nextEndTag, final boolean checkForEmptyElementTag) {
+	private Segment[] findEndTag(final int afterPos, final StartTag nextStartTag, final EndTag nextEndTag, final boolean checkForEmptyElementTag, final boolean isXMLTagName) {
 		// returns null if no end tag exists in the rest of the file, otherwise the following two segments:
 		// first is the matching end tag to this start tag.  Must be present if array is returned.
 		// second is the next occurrence after the returned end tag of a start tag of the same name. (null if none exists)
 		if (nextEndTag==null) return null;  // no end tag in the rest of the file
-		final Segment[] returnArray={nextEndTag, nextStartTag};
+		final Segment[] returnArray={nextEndTag,nextStartTag};
 		if (nextStartTag==null || nextStartTag.begin>nextEndTag.begin) return returnArray;  // no more start tags of the same name in rest of file, or they occur after the end tag that we found.  This means we have found the matching end tag.
-		final Segment[] findResult=nextStartTag.findEndTag(nextEndTag,checkForEmptyElementTag);  // find the matching end tag to the interloping start tag
+		final Segment[] findResult=nextStartTag.findEndTag(nextEndTag,checkForEmptyElementTag,isXMLTagName);  // find the matching end tag to the interloping start tag
 		if (findResult==null) return null;  // no end tag in the rest of the file
 		final EndTag nextStartTagsEndTag=(EndTag)findResult[0];
-		nextStartTag=(StartTag)findResult[1];
-		nextEndTag=source.findNextEndTag(nextStartTagsEndTag.end, name);  // find end tag after the interloping start tag's end tag
-		return findEndTag(nextStartTagsEndTag.end,nextStartTag,nextEndTag,checkForEmptyElementTag);  // recurse to see if this is the matching end tag
+		final EndTag nextNextEndTag=EndTag.findNext(source,nextStartTagsEndTag.end,nextEndTag.getName(),nextEndTag.getEndTagType()); // find end tag after the interloping start tag's end tag
+		return findEndTag(nextStartTagsEndTag.end,(StartTag)findResult[1],nextNextEndTag,checkForEmptyElementTag,isXMLTagName);  // recurse to see if this is the matching end tag
 	}
 }
 
