@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -32,8 +32,11 @@ import java.util.*;
  * and would not intuitively be categorised as a "start tag".  For example, an HTML {@linkplain #COMMENT comment} in a document
  * is represented as a single start tag that spans the whole comment, and does not have an end tag at all.
  * <p>
- * Instances of all the <a href="TagType.html#Standard">standard</a> start tag types are available in this class as static
+ * The singleton instances of all the <a href="TagType.html#Standard">standard</a> start tag types are available in this class as static
  * <a href="#field_summary">fields</a>.
+ * <p>
+ * Because all <code>StartTagType</code> instaces must be singletons, the '<code>==</code>' operator can be used to test for a particular tag type
+ * instead of the <code>equals(Object)</code> method.
  *
  * @see EndTagType
  */
@@ -45,7 +48,8 @@ public abstract class StartTagType extends TagType {
 	static final String START_DELIMITER_PREFIX="<";
 
 	/**
-	 * The tag type given to an {@linkplain Tag#isUnregistered() unregistered} {@linkplain StartTag start tag} (<code>&lt; </code>&#46;&#46;&#46;<code> &gt;</code>).
+	 * The tag type given to an {@linkplain Tag#isUnregistered() unregistered} {@linkplain StartTag start tag}
+	 * (<code>&lt;<var> &#46;&#46;&#46; </var>&gt;</code>).
 	 * <p>
 	 * See the documentation of the {@link Tag#isUnregistered()} method for details.
 	 * <p>
@@ -71,7 +75,8 @@ public abstract class StartTagType extends TagType {
 	public static final StartTagType UNREGISTERED=StartTagTypeUnregistered.INSTANCE;
 
 	/**
-	 * The tag type given to a normal HTML or XML {@linkplain StartTag start tag} (<code>&lt;</code><i>name</i><code> </code>&#46;&#46;&#46;<code> &gt;</code>).
+	 * The tag type given to a normal HTML or XML {@linkplain StartTag start tag}
+	 * (<code>&lt;<var>name</var><var> &#46;&#46;&#46; </var>&gt;</code>).
 	 * <p>
 	 * <dl>
 	 *  <dt>Properties:</dt>
@@ -94,7 +99,8 @@ public abstract class StartTagType extends TagType {
 	public static final StartTagType NORMAL=StartTagTypeNormal.INSTANCE;
 
 	/**
-	 * The tag type given to an HTML <a target="_blank" href="http://www.w3.org/TR/html401/intro/sgmltut.html#h-3.2.4">comment</a> (<code>&lt;&#33;-- </code>&#46;&#46;&#46;<code> --&gt;</code>).
+	 * The tag type given to an HTML <a target="_blank" href="http://www.w3.org/TR/html401/intro/sgmltut.html#h-3.2.4">comment</a>
+	 * (<code>&lt;&#33;--<var> &#46;&#46;&#46; </var>--&gt;</code>).
 	 * <p>
 	 * An HTML comment is an area of the source document enclosed by the delimiters
 	 * <code>&lt;!--</code> on the left and <code>--&gt;</code> on the right.
@@ -128,7 +134,8 @@ public abstract class StartTagType extends TagType {
 	public static final StartTagType COMMENT=StartTagTypeComment.INSTANCE;
 
 	/**
-	 * The tag type given to an <a target="_blank" href="http://www.w3.org/TR/REC-xml/#sec-prolog-dtd">XML declaration</a> (<code>&lt;&#63;xml </code>&#46;&#46;&#46;<code> &#63;&gt;</code>).
+	 * The tag type given to an <a target="_blank" href="http://www.w3.org/TR/REC-xml/#sec-prolog-dtd">XML declaration</a>
+	 * (<code>&lt;&#63;xml<var> &#46;&#46;&#46; </var>&#63;&gt;</code>).
 	 * <p>
 	 * An XML declaration is often referred to in texts as a special type of processing instruction with the reserved
 	 * <a target="_blank" href="http://www.w3.org/TR/REC-xml/#NT-PITarget">PITarget</a> name of "<code>xml</code>".
@@ -161,7 +168,8 @@ public abstract class StartTagType extends TagType {
 	public static final StartTagType XML_DECLARATION=StartTagTypeXMLDeclaration.INSTANCE;
 
 	/**
-	 * The tag type given to an <a target="_blank" href="http://www.w3.org/TR/REC-xml#sec-pi">XML processing instruction</a> (<code>&lt;&#63;</code><i>PITarget</i> &#46;&#46;&#46; <code>&#63;&gt;</code>).
+	 * The tag type given to an <a target="_blank" href="http://www.w3.org/TR/REC-xml#sec-pi">XML processing instruction</a>
+	 * (<code>&lt;&#63;<var>PITarget</var><var> &#46;&#46;&#46; </var>&#63;&gt;</code>).
 	 * <p>
 	 * An XML processing instruction is a specific form of
 	 * <a target="_blank" href="http://www.w3.org/TR/html401/appendix/notes.html#h-B.3.6">SGML processing instruction</a> with the following
@@ -206,7 +214,8 @@ public abstract class StartTagType extends TagType {
 	public static final StartTagType XML_PROCESSING_INSTRUCTION=StartTagTypeXMLProcessingInstruction.INSTANCE;
 
 	/**
-	 * The tag type given to a <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#h-7.2">document type declaration</a> (<code>&lt;&#33;DOCTYPE </code>&#46;&#46;&#46;<code> &gt;</code>).
+	 * The tag type given to a <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#h-7.2">document type declaration</a>
+	 * (<code>&lt;&#33;DOCTYPE<var> &#46;&#46;&#46; </var>&gt;</code>).
 	 * <p>
 	 * Information about the document type declaration can be found in the
 	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#h-7.2">HTML 4.01 specification section 7.2</a>, and the
@@ -236,7 +245,8 @@ public abstract class StartTagType extends TagType {
 	public static final StartTagType DOCTYPE_DECLARATION=StartTagTypeDoctypeDeclaration.INSTANCE;
 
 	/**
-	 * The tag type given to a <a target="_blank" href="http://www.w3.org/TR/REC-xml/#dt-markupdecl">markup declaration</a> (<code>&lt;&#33; </code>&#46;&#46;&#46;<code> &gt;</code>).
+	 * The tag type given to a <a target="_blank" href="http://www.w3.org/TR/REC-xml/#dt-markupdecl">markup declaration</a>
+	 * (<code>&lt;&#33;ELEMENT<var> &#46;&#46;&#46; </var>&gt;</code> | <code>&lt;&#33;ATTLIST<var> &#46;&#46;&#46; </var>&gt;</code> | <code>&lt;&#33;ENTITY<var> &#46;&#46;&#46; </var>&gt;</code> | <code>&lt;&#33;NOTATION<var> &#46;&#46;&#46; </var>&gt;</code>).
 	 * <p>
 	 * The {@linkplain Tag#getName() name} of a markup declaration tag is must be one of
 	 * "<code>!element</code>", "<code>!attlist</code>", "<code>!entity</code>" or "<code>!notation</code>".
@@ -269,13 +279,13 @@ public abstract class StartTagType extends TagType {
 	public static final StartTagType MARKUP_DECLARATION=StartTagTypeMarkupDeclaration.INSTANCE;
 
 	/**
-	 * The tag type given to a <a target="_blank" href="http://www.w3.org/TR/html401/appendix/notes.html#h-B.3.5">CDATA section</a> (<code>&lt;&#33;[CDATA[ </code>&#46;&#46;&#46;<code> ]]&gt;</code>).
+	 * The tag type given to a <a target="_blank" href="http://www.w3.org/TR/html401/appendix/notes.html#h-B.3.5">CDATA section</a>
+	 * (<code>&lt;&#33;[CDATA[<var> &#46;&#46;&#46; </var>]]&gt;</code>).
 	 * <p>
 	 * A CDATA section is a specific form of a
 	 * <a target="_blank" href="http://www.w3.org/TR/html401/appendix/notes.html#h-B.3.5">marked section</a>.
-	 * <p>
 	 * This library does not include a <a href="TagType.html#Predefined">predefined</a> generic tag type for marked sections,
-	 * as the only form in which they are found in HTML documents are CDATA sections.
+	 * as the only type of marked sections found in HTML documents are CDATA sections.
 	 * <p>
 	 * The <a target="_blank" href="http://www.w3.org/TR/html401/appendix/notes.html#h-B.3.5">HTML 4.01 specification section B.3.5</a>
 	 * and the <a target="_blank" href="http://www.w3.org/TR/REC-xml/#sec-cdata-sect">XML 1.0 specification section 2.7</a>
@@ -310,13 +320,16 @@ public abstract class StartTagType extends TagType {
 	 *    </table>
 	 *  <dt>Example:</dt>
 	 *   <dd>This example shows the recommended practice of enclosing scripts inside a CDATA section:
-	 *    <div style="margin-top: 0.5em"><code>&lt;script&gt;&lt;![CDATA[ function min(a,b) {return a&lt;b ? a : b;} ]]&gt;&lt;/script&gt;</code></div>
+	 *    <div style="margin-top: 0.5em">
+	 *     <pre>&lt;script type="text/javascript"&gt;<br />  //&lt;![CDATA[<br />    function min(a,b) {return a&lt;b ? a : b;}<br />  //]]&gt<br />&lt;/script&gt;</pre>
+	 *    </div>
 	 * </dl>
 	 */
 	public static final StartTagType CDATA_SECTION=StartTagTypeCDATASection.INSTANCE;
-	
+
 	/**
-	 * The tag type given to a common server tag (<code>&lt;% </code>&#46;&#46;&#46;<code> %&gt;</code>).
+	 * The tag type given to a common server tag
+	 * (<code>&lt;%<var> &#46;&#46;&#46; </var>%&gt;</code>).
 	 * <p>
 	 * Common server tags include
 	 * <a target="_blank" href="http://msdn.microsoft.com/asp/">ASP</a>,
@@ -326,8 +339,9 @@ public abstract class StartTagType extends TagType {
 	 * <a target="_blank" href="http://www.rubycentral.com/book/web.html#S2">eRuby</a>, and
 	 * <a target="_blank" href="http://www.masonbook.com/book/chapter-2.mhtml#CHP-2-SECT-3.1">Mason substitution</a> tags.
 	 * <p>
-	 * This is the only <a href="TagType.html#Standard">standard</a> tag type that defines a {@linkplain #isServerTag() server tag}.
-	 * It is included as a standard tag type because of its widespread use in many platforms, including those listed above.
+	 * This tag and the {@linkplain #SERVER_COMMON_ESCAPED escaped common server tag} are the only <a href="TagType.html#Standard">standard</a> tag types
+	 * that define {@linkplain #isServerTag() server tags}.
+	 * They are included as standard tag types because of the common server tag's widespread use in many platforms, including those listed above.
 	 * <p>
 	 * <dl>
 	 *  <dt>Properties:</dt>
@@ -349,6 +363,34 @@ public abstract class StartTagType extends TagType {
 	 */
 	public static final StartTagType SERVER_COMMON=StartTagTypeServerCommon.INSTANCE;
 
+	/**
+	 * The tag type given to an escaped common server tag
+	 * (<code>&lt;\%<var> &#46;&#46;&#46; </var>%&gt;</code>).
+	 * <p>
+	 * Some of the platforms that support the {@linkplain #SERVER_COMMON common server tag} also support a mechanism to escape that tag by adding a
+	 * backslash (<code>\</code>) before the percent (<code>%</code>) character.
+	 * Although rarely used, this tag type allows the parser to recognise these escaped tags in addition to the common server tag itself.
+	 * <p>
+	 * <dl>
+	 *  <dt>Properties:</dt>
+	 *   <dd>
+	 *    <table class="bordered" style="margin: 15px" cellspacing="0">
+	 *     <tr><th>Property<th>Value
+	 *     <tr><td>{@link #getDescription() Description}<td>escaped common server tag
+	 *     <tr><td>{@link #getStartDelimiter() StartDelimiter}<td><code>&lt;\%</code>
+	 *     <tr><td>{@link #getClosingDelimiter() ClosingDelimiter}<td><code>%&gt;</code>
+	 *     <tr><td>{@link #isServerTag() IsServerTag}<td><code>true</code>
+	 *     <tr><td>{@link #getNamePrefix() NamePrefix}<td><code>\%</code>
+	 *     <tr><td>{@link #getCorrespondingEndTagType() CorrespondingEndTagType}<td><code>null</code>
+	 *     <tr><td>{@link #hasAttributes() HasAttributes}<td><code>false</code>
+	 *     <tr><td>{@link #isNameAfterPrefixRequired() IsNameAfterPrefixRequired}<td><code>false</code>
+	 *    </table>
+	 *  <dt>Example:</dt>
+	 *   <dd><code>&lt;\%@ include file="header.html" %&gt;</code></dd>
+	 * </dl>
+	 */
+	public static final StartTagType SERVER_COMMON_ESCAPED=StartTagTypeServerCommonEscaped.INSTANCE;
+
 	/**
 	 * Constructs a new <code>StartTagType</code> object with the specified properties.
 	 * <br />(<a href="TagType.html#ImplementationAssistance">implementation assistance</a> method)
@@ -395,6 +437,7 @@ public abstract class StartTagType extends TagType {
 	 *     <tr><td>{@link StartTagType#MARKUP_DECLARATION}<td><code>null</code>
 	 *     <tr><td>{@link StartTagType#CDATA_SECTION}<td><code>null</code>
 	 *     <tr><td>{@link StartTagType#SERVER_COMMON}<td><code>null</code>
+	 *     <tr><td>{@link StartTagType#SERVER_COMMON_ESCAPED}<td><code>null</code>
 	 *    </table>
 	 * </dl>
 	 * <dl>
@@ -402,6 +445,7 @@ public abstract class StartTagType extends TagType {
 	 *   <dd>
 	 *    <table class="bordered" style="margin: 15px" cellspacing="0">
 	 *     <tr><th>Start Tag Type<th>Corresponding End Tag Type
+	 *     <tr><td>{@link MicrosoftTagTypes#DOWNLEVEL_REVEALED_CONDITIONAL_COMMENT}<td><code>null</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SCRIPT}<td>{@link EndTagType#NORMAL}
 	 *     <tr><td>{@link PHPTagTypes#PHP_SHORT}<td><code>null</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_STANDARD}<td><code>null</code>
@@ -443,6 +487,7 @@ public abstract class StartTagType extends TagType {
 	 *     <tr><td>{@link StartTagType#MARKUP_DECLARATION}<td><code>false</code>
 	 *     <tr><td>{@link StartTagType#CDATA_SECTION}<td><code>false</code>
 	 *     <tr><td>{@link StartTagType#SERVER_COMMON}<td><code>false</code>
+	 *     <tr><td>{@link StartTagType#SERVER_COMMON_ESCAPED}<td><code>false</code>
 	 *    </table>
 	 * </dl>
 	 * <dl>
@@ -450,6 +495,7 @@ public abstract class StartTagType extends TagType {
 	 *   <dd>
 	 *    <table class="bordered" style="margin: 15px" cellspacing="0">
 	 *     <tr><th>Start Tag Type<th>Has Attributes
+	 *     <tr><td>{@link MicrosoftTagTypes#DOWNLEVEL_REVEALED_CONDITIONAL_COMMENT}<td><code>false</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SCRIPT}<td><code>true</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SHORT}<td><code>false</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_STANDARD}<td><code>false</code>
@@ -489,6 +535,7 @@ public abstract class StartTagType extends TagType {
 	 *     <tr><td>{@link StartTagType#MARKUP_DECLARATION}<td><code>true</code>
 	 *     <tr><td>{@link StartTagType#CDATA_SECTION}<td><code>false</code>
 	 *     <tr><td>{@link StartTagType#SERVER_COMMON}<td><code>false</code>
+	 *     <tr><td>{@link StartTagType#SERVER_COMMON_ESCAPED}<td><code>false</code>
 	 *    </table>
 	 * </dl>
 	 * <dl>
@@ -496,6 +543,7 @@ public abstract class StartTagType extends TagType {
 	 *   <dd>
 	 *    <table class="bordered" style="margin: 15px" cellspacing="0">
 	 *     <tr><th>Start Tag Type<th>Name After Prefix Required
+	 *     <tr><td>{@link MicrosoftTagTypes#DOWNLEVEL_REVEALED_CONDITIONAL_COMMENT}<td><code>true</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SCRIPT}<td><code>false</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SHORT}<td><code>false</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_STANDARD}<td><code>false</code>
@@ -530,7 +578,7 @@ public abstract class StartTagType extends TagType {
 	 *
 	 * @param source  the {@link Source} document.
 	 * @param pos  the character position in the source document.
-	 * @param isClosingSlashIgnored  indicates whether the {@linkplain StartTag#getName() name} of the {@linkplain StartTag start tag} being tested is incompatible with an {@linkplain Element#isEmptyElementTag() empty-element tag}.
+	 * @param isClosingSlashIgnored  indicates whether the {@linkplain StartTag#getName() name} of the {@linkplain StartTag start tag} being tested is incompatible with an {@linkplain StartTag#isEmptyElementTag() empty-element tag}.
 	 * @return <code>true</code> if the specified source document position is at the end of a tag's {@linkplain Attributes attributes}, otherwise <code>false</code>.
 	 */
 	public boolean atEndOfAttributes(final Source source, final int pos, final boolean isClosingSlashIgnored) {
