@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -79,15 +79,18 @@ final class SubCache {
 		array[eof.index]=eof;
 	}
 
-	public Tag getTagAt(final int pos) {
+	public Tag getTagAt(final int pos, final boolean serverTagOnly) {
 		// This must only be called on allTagTypesSubCache (ie tagType==null)
 		if (cache.getSourceLength()==0) return null;
 		if (pos<0 || pos>=cache.getSourceLength()) return null;
 		final int index=getIndexOfPos(pos);
 		final CacheEntry cacheEntry=array[index];
-		if (cacheEntry.pos==pos) return cacheEntry.tag;
+		if (cacheEntry.pos==pos) {
+			if (serverTagOnly && !cacheEntry.tag.getTagType().isServerTag()) return null;
+			return cacheEntry.tag;
+		}
 		if (cacheEntry.previousCached) return null;
-		return cache.addTagAt(pos);
+		return cache.addTagAt(pos,serverTagOnly);
 	}
 
 	public void addTagAt(final int pos, final Tag tag) {
@@ -97,26 +100,33 @@ final class SubCache {
 		add(previousCacheEntry,new CacheEntry(index,pos,tag,pos==previousCacheEntry.pos+1,pos==nextCacheEntry.pos-1),nextCacheEntry);
 	}
 
-	public Tag findPreviousOrNextTag(final int pos, final boolean previous) {
+	public Tag findPreviousTag(final int pos) {
 		// Note that this method never returns tags for which tag.includInSearch() is false, so separate caching of unregistered tags won't work.
 		if (cache.getSourceLength()==0) return null;
 		if (pos<0 || pos>=cache.getSourceLength()) return null;
 		int index=getIndexOfPos(pos);
 		final CacheEntry cacheEntry=array[index];
 		final Tag tag;
-		if (previous) {
-			if (cacheEntry.pos==pos && cacheEntry.tag!=null && cacheEntry.tag.includeInSearch()) return cacheEntry.tag;
-			tag=findPreviousTag(getPrevious(cacheEntry),pos,cacheEntry);
-			addPreviousTag(pos,tag);
+		if (cacheEntry.pos==pos && cacheEntry.tag!=null && cacheEntry.tag.includeInSearch()) return cacheEntry.tag;
+		tag=findPreviousTag(getPrevious(cacheEntry),pos,cacheEntry);
+		addPreviousTag(pos,tag);
+		return tag;
+	}
+
+	public Tag findNextTag(final int pos) {
+		// Note that this method never returns tags for which tag.includInSearch() is false, so separate caching of unregistered tags won't work.
+		if (cache.getSourceLength()==0) return null;
+		if (pos<0 || pos>=cache.getSourceLength()) return null;
+		int index=getIndexOfPos(pos);
+		final CacheEntry cacheEntry=array[index];
+		final Tag tag;
+		if (cacheEntry.pos==pos) {
+			if (cacheEntry.tag!=null && cacheEntry.tag.includeInSearch()) return cacheEntry.tag;
+			tag=findNextTag(cacheEntry,pos,getNext(cacheEntry));
 		} else {
-			if (cacheEntry.pos==pos) {
-				if (cacheEntry.tag!=null && cacheEntry.tag.includeInSearch()) return cacheEntry.tag;
-				tag=findNextTag(cacheEntry,pos,getNext(cacheEntry));
-			} else {
-				tag=findNextTag(getPrevious(cacheEntry),pos,cacheEntry);
-			}
-			addNextTag(pos,tag);
+			tag=findNextTag(getPrevious(cacheEntry),pos,cacheEntry);
 		}
+		addNextTag(pos,tag);
 		return tag;
 	}
 
@@ -138,7 +148,7 @@ final class SubCache {
 		// previousCacheEntry.pos < pos <= nextCacheEntry.pos
 		while (true) {
 			if (!nextCacheEntry.previousCached) {
-				final Tag tag=Tag.findPreviousOrNextTagUncached(cache.source,pos,tagType,true,previousCacheEntry.pos); // if useAllTypesCache is true, automatically adds tag to all caches if one is found, and maybe some unregistered tags along the way.
+				final Tag tag=Tag.findPreviousTagUncached(cache.source,pos,tagType,previousCacheEntry.pos); // if useAllTypesCache is true, automatically adds tag to all caches if one is found, and maybe some unregistered tags along the way.
 				if (tag!=null) {
 					if (!cache.source.useAllTypesCache) addTagAt(tag.begin,tag); // have to add tag manually if useAllTypesCache is false
 					return tag;
@@ -155,7 +165,7 @@ final class SubCache {
 		// previousCacheEntry.pos <= pos < nextCacheEntry.pos
 		while (true) {
 			if (!previousCacheEntry.nextCached) {
-				final Tag tag=Tag.findPreviousOrNextTagUncached(cache.source,pos,tagType,false,nextCacheEntry.pos); // if useAllTypesCache is true, automatically adds tag to caches if one is found, and maybe some unregistered tags along the way.
+				final Tag tag=Tag.findNextTagUncached(cache.source,pos,tagType,nextCacheEntry.pos); // if useAllTypesCache is true, automatically adds tag to caches if one is found, and maybe some unregistered tags along the way.
 				if (tag!=null) {
 					if (!cache.source.useAllTypesCache) addTagAt(tag.begin,tag); // have to add tag manually if useAllTypesCache is false
 					return tag;
@@ -184,7 +194,7 @@ final class SubCache {
 		} else if (!stepCacheEntry.previousCached) {
 			// we have to add a new cacheEntry at pos:
 			if (tagType==null)
-				cache.addTagAt(pos); // this pos has never been checked before, so add it to all relevant SubCaches (a null or unregistered tag entry is always added to this SubCache)
+				cache.addTagAt(pos,false); // this pos has never been checked before, so add it to all relevant SubCaches (a null or unregistered tag entry is always added to this SubCache)
 			else
 				addTagAt(pos,null); // all we know is that the pos doesn't contain a tag of this SubCache's type, so add a null entry to this SubCache only.
 			// now we have to reload the index and stepCacheEntry as they may have changed:
@@ -229,7 +239,7 @@ final class SubCache {
 		} else if (!getPrevious(stepCacheEntry).nextCached) {
 			// we have to add a new cacheEntry at pos:
 			if (tagType==null)
-				cache.addTagAt(pos); // this pos has never been checked before, so add it to all relevant SubCaches (a null or unregistered tag entry is always added to this SubCache)
+				cache.addTagAt(pos,false); // this pos has never been checked before, so add it to all relevant SubCaches (a null or unregistered tag entry is always added to this SubCache)
 			else
 				addTagAt(pos,null); // all we know is that the pos doesn't contain a tag of this SubCache's type, so add a null entry to this SubCache only.
 			// now we have to reload the index and stepCacheEntry as they may have changed:
