@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.6
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -24,6 +24,14 @@ import java.util.*;
 
 /**
  * Represents either a {@link StartTag} or {@link EndTag} in a specific {@linkplain Source source} document.
+ * <p>
+ * Take the following HTML segment as an example:
+ * <p>
+ * <code>&lt;p&gt;This is a sample paragraph.&lt;/p&gt;</code>
+ * <p>
+ * The "<code>&lt;p&gt;</code>" is represented by a {@link StartTag} object, and the "<code>&lt;/p&gt;</code>" is represented by an {@link EndTag} object,
+ * both of which are subclasses of the <code>Tag</code> class.
+ * The whole segment, including the start tag, its corresponding end tag and all of the content in between, is represented by an {@link Element} object.
  *
  * <h3><a name="ParsingProcess">Tag Parsing Process</a></h3>
  * The following process describes how each tag is identified by the parser:
@@ -40,13 +48,17 @@ import java.util.*;
  *   The analysis performed in relation to each candidate tag type is a two-stage process:
  *   <ol>
  *    <li>
- *     The position of the tag is checked to determine whether it is {@linkplain TagType#isValidPosition(Source,int) valid}.
- *     In theory, a {@linkplain TagType#isServerTag() server tag} is valid in any position, but a non-server tag is not valid inside another non-server tag.
+ *     The position of the tag is checked to determine whether it is {@linkplain TagType#isValidPosition(Source,int,int[]) valid}.
+ *     In theory, a {@linkplain TagType#isServerTag() server tag} is valid in any position, but a non-server tag is not valid inside any other tag,
+ *     nor inside elements with CDATA content such as {@link HTMLElementName#SCRIPT SCRIPT} and {@link HTMLElementName#STYLE STYLE} elements.
+ *     Theory dictates therefore that {@linkplain StartTagType#COMMENT comments} and explicit {@linkplain StartTagType#CDATA_SECTION CDATA sections}
+ *     inside script elements should not be recognised as tags.
  *     <p>
- *     The {@link TagType#isValidPosition(Source, int pos)} method is responsible for this check and has a common default implementation for all tag types
+ *     The {@link TagType#isValidPosition(Source, int pos, int[] fullSequentialParseData)} method is responsible for this check
+ *     and has a common default implementation for all tag types
  *     (although <a href="TagType.html#custom">custom</a> tag types can override it if necessary).
  *     Its behaviour differs depending on whether or not a {@linkplain Source#fullSequentialParse() full sequential parse} is peformed.
- *     See the documentation of the {@link TagType#isValidPosition(Source,int) isValidPosition} method for full details.
+ *     See the documentation of the {@link TagType#isValidPosition(Source,int,int[]) isValidPosition} method for full details.
  *    <li>
  *     A final analysis is performed by the {@link TagType#constructTagAt(Source, int pos)} method of the candidate tag type.
  *     This method returns a valid {@link Tag} object if all conditions of the candidate tag type are met, otherwise it returns
@@ -119,65 +131,24 @@ import java.util.*;
  */
 public abstract class Tag extends Segment implements HTMLElementName {
 	String name=null; // always lower case, can always use == operator to compare with constants in HTMLElementName interface
-	Element element=Element.NOT_CACHED; // cache
-	int allTagsArrayIndex=-1;
 	private Object userData=null;
+	// cached values:
+	Element element=Element.NOT_CACHED;
+	private Tag previousTag=NOT_CACHED;
+	private Tag nextTag=NOT_CACHED;
 
-	/**
-	 * {@linkplain StartTagType#XML_PROCESSING_INSTRUCTION XML processing instruction}
-	 * @deprecated  Use {@link StartTagType#XML_PROCESSING_INSTRUCTION} in combination with <a href="#TagTypeSearch">tag type search</a> methods instead.
-	 */
-	public static final String PROCESSING_INSTRUCTION=StartTagType.XML_PROCESSING_INSTRUCTION.getNamePrefixForTagConstant();
-
-	/**
-	 * {@linkplain StartTagType#XML_DECLARATION XML declaration}
-	 * @deprecated  Use {@link StartTagType#XML_DECLARATION} in combination with <a href="#TagTypeSearch">tag type search</a> methods instead.
-	 */
-	public static final String XML_DECLARATION=StartTagType.XML_DECLARATION.getNamePrefixForTagConstant();
-
-	/**
-	 * {@linkplain StartTagType#DOCTYPE_DECLARATION document type declaration}
-	 * @deprecated  Use {@link StartTagType#DOCTYPE_DECLARATION} in combination with <a href="#TagTypeSearch">tag type search</a> methods instead.
-	 */
-	public static final String DOCTYPE_DECLARATION=StartTagType.DOCTYPE_DECLARATION.getNamePrefixForTagConstant();
-
-	/**
-	 * {@linkplain PHPTagTypes#PHP_STANDARD Standard PHP} tag (<code>&lt;&#63;php &#46;&#46;&#46; &#63;&gt;</code>)
-	 * @deprecated  Use {@link PHPTagTypes#PHP_STANDARD} in combination with <a href="#TagTypeSearch">tag type search</a> methods instead.
-	 */
-	public static final String SERVER_PHP=PHPTagTypes.PHP_STANDARD.getNamePrefixForTagConstant();
-
-	/**
-	 * Common {@linkplain StartTagType#SERVER_COMMON server} tag (<code>&lt;% &#46;&#46;&#46; %&gt;</code>)
-	 * @deprecated  Use {@link StartTagType#SERVER_COMMON} in combination with <a href="#TagTypeSearch">tag type search</a> methods instead.
-	 */
-	public static final String SERVER_COMMON=StartTagType.SERVER_COMMON.getNamePrefixForTagConstant();
-
-	/**
-	 * {@linkplain MasonTagTypes#MASON_NAMED_BLOCK Mason named block} (<code>&lt;%<i>name</i> &#46;&#46;&#46; &gt; &#46;&#46;&#46; &lt;/%<i>name</i>&gt;</code>)
-	 * @deprecated  Use {@link MasonTagTypes#MASON_NAMED_BLOCK} in combination with <a href="#TagTypeSearch">tag type search</a> methods instead.
-	 */
-	public static final String SERVER_MASON_NAMED_BLOCK=MasonTagTypes.MASON_NAMED_BLOCK.getNamePrefixForTagConstant(); // NOTE: this value is the same value as SERVER_COMMON
-
-	/**
-	 * {@linkplain MasonTagTypes#MASON_COMPONENT_CALL Mason component call} (<code>&lt;&amp; &#46;&#46;&#46; &amp;&gt;</code>)
-	 * @deprecated  Use {@link MasonTagTypes#MASON_COMPONENT_CALL} in combination with <a href="#TagTypeSearch">tag type search</a> methods instead.
-	 */
-	public static final String SERVER_MASON_COMPONENT_CALL=MasonTagTypes.MASON_COMPONENT_CALL.getNamePrefixForTagConstant();
-
-	/**
-	 * {@linkplain MasonTagTypes#MASON_COMPONENT_CALLED_WITH_CONTENT Mason component called with content} (<code>&lt;&amp;| &#46;&#46;&#46; &amp;&gt; &#46;&#46;&#46; &lt;/&amp;&gt;</code>)
-	 * @deprecated  Use {@link MasonTagTypes#MASON_COMPONENT_CALLED_WITH_CONTENT} in combination with <a href="#TagTypeSearch">tag type search</a> methods instead.
-	 */
-	public static final String SERVER_MASON_COMPONENT_CALLED_WITH_CONTENT=MasonTagTypes.MASON_COMPONENT_CALLED_WITH_CONTENT.getNamePrefixForTagConstant();
+	static final Tag NOT_CACHED=new StartTag();
 
 	private static final boolean INCLUDE_UNREGISTERED_IN_SEARCH=false; // determines whether unregistered tags are included in searches
 
 	Tag(final Source source, final int begin, final int end, final String name) {
-		super(source, begin, end);
+		super(source,begin,end);
 		this.name=HTMLElements.getConstantElementName(name.toLowerCase());
 	}
 
+	// only used to create Tag.NOT_CACHED
+	Tag() {}
+
 	/**
 	 * Returns the {@linkplain Element element} that is started or ended by this tag.
 	 * <p>
@@ -261,45 +232,39 @@ public abstract class Tag extends Segment implements HTMLElementName {
 	/**
 	 * Returns the next tag in the source document.
 	 * <p>
-	 * If a {@linkplain Source#fullSequentialParse() full sequential parse} has been performed, this method is very efficient.
+	 * This method also returns {@linkplain TagType#isServerTag() server tags}.
 	 * <p>
-	 * If not, it is equivalent to <code>source.</code>{@link Source#findNextTag(int) findNextTag}<code>(</code>{@link #getBegin()}<code>+1)</code>.
+	 * The result of a call to this method is cached.
+	 * Performing a {@linkplain Source#fullSequentialParse() full sequential parse} prepopulates this cache.
+	 * <p>
+	 * If the result is not cached, a call to this method is equivalent to <code>source.</code>{@link Source#findNextTag(int) findNextTag}<code>(</code>{@link #getBegin() getBegin()}<code>+1)</code>.
 	 * <p>
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 *
 	 * @return the next tag in the source document, or <code>null</code> if this is the last tag.
 	 */
 	public Tag findNextTag() {
-		final Tag[] allTagsArray=source.allTagsArray;
-		if (allTagsArray!=null) {
-			final int nextAllTagsArrayIndex=allTagsArrayIndex+1;
-			if (allTagsArray.length==nextAllTagsArrayIndex) return null;
-			return allTagsArray[nextAllTagsArrayIndex];
-		} else {
-			return source.findNextTag(begin+1);
-		}
+		if (nextTag==NOT_CACHED) nextTag=findNextTag(source,begin+1);
+		return nextTag;
 	}
 
 	/**
 	 * Returns the previous tag in the source document.
 	 * <p>
-	 * If a {@linkplain Source#fullSequentialParse() full sequential parse} has been performed, this method is very efficient.
+	 * This method also returns {@linkplain TagType#isServerTag() server tags}.
 	 * <p>
-	 * If not, it is equivalent to <code>source.</code>{@link Source#findPreviousTag(int) findPreviousTag}<code>(</code>{@link #getBegin()}<code>-1)</code>.
+	 * The result of a call to this method is cached.
+	 * Performing a {@linkplain Source#fullSequentialParse() full sequential parse} prepopulates this cache.
+	 * <p>
+	 * If the result is not cached, a call to this method is equivalent to <code>source.</code>{@link Source#findPreviousTag(int) findPreviousTag}<code>(</code>{@link #getBegin() getBegin()}<code>-1)</code>.
 	 * <p>
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 *
 	 * @return the previous tag in the source document, or <code>null</code> if this is the first tag.
 	 */
 	public Tag findPreviousTag() {
-		final Tag[] allTagsArray=source.allTagsArray;
-		if (allTagsArray!=null) {
-			if (allTagsArrayIndex==0) return null;
-			return allTagsArray[allTagsArrayIndex-1];
-		} else {
-			if (begin==0) return null;
-			return source.findPreviousTag(begin-1);
-		}
+		if (previousTag==NOT_CACHED) previousTag=findPreviousTag(source,begin-1);
+		return previousTag;
 	}
 
 	/**
@@ -326,7 +291,7 @@ public abstract class Tag extends Segment implements HTMLElementName {
 	 * specified <code>tagType</code> is either {@link StartTagType#UNREGISTERED} or {@link EndTagType#UNREGISTERED}.
 	 * <p>
 	 * <a href="Tag.html#OpenSearch">Open</a> tag searches and <a href="Tag.html#OtherSearch">other</a> searches always ignore
-	 * unregistered tags, although every discovery of an unregistered tag is {@linkplain Source#setLogWriter(Writer) logged} by the parser.
+	 * unregistered tags, although every discovery of an unregistered tag is {@linkplain Source#getLogger() logged} by the parser.
 	 * <p>
 	 * The logic behind this design is that unregistered tag types are usually the result of a '<code>&lt;</code>' character 
 	 * in the text that was mistakenly left {@linkplain CharacterReference#encode(CharSequence) unencoded}, or a less-than 
@@ -423,39 +388,55 @@ public abstract class Tag extends Segment implements HTMLElementName {
 		return Character.isLetterOrDigit(ch) || ch=='.' || ch=='-' || ch=='_' || ch==':';
 	}
 
-	/**
-	 * Regenerates the HTML text of this tag.
-	 * <p>
-	 * This method has been deprecated as of version 2.2 and replaced with the exactly equivalent {@link #tidy()} method.
-	 *
-	 * @return the regenerated HTML text of this tag.
-	 * @deprecated  Use {@link #tidy()} instead.
-	 */
-	public abstract String regenerateHTML();
-
 	final boolean includeInSearch() {
 		return INCLUDE_UNREGISTERED_IN_SEARCH || !isUnregistered();
 	}
 
-	static final Tag findPreviousOrNextTag(final Source source, final int pos, final boolean previous) {
+	static final Tag findPreviousTag(final Source source, final int pos) {
 		// returns null if pos is out of range.
 		return source.useAllTypesCache
-			? source.cache.findPreviousOrNextTag(pos,previous)
-			: findPreviousOrNextTagUncached(source,pos,previous,ParseText.NO_BREAK);
+			? source.cache.findPreviousTag(pos)
+			: findPreviousTagUncached(source,pos,ParseText.NO_BREAK);
+	}
+
+	static final Tag findNextTag(final Source source, final int pos) {
+		// returns null if pos is out of range.
+		return source.useAllTypesCache
+			? source.cache.findNextTag(pos)
+			: findNextTagUncached(source,pos,ParseText.NO_BREAK);
 	}
 		
-	static final Tag findPreviousOrNextTagUncached(final Source source, final int pos, final boolean previous, final int breakAtPos) {
+	static final Tag findPreviousTagUncached(final Source source, final int pos, final int breakAtPos) {
 		// returns null if pos is out of range.
 		try {
 			final ParseText parseText=source.getParseText();
 			int begin=pos;
 			do {
-				begin=previous?parseText.lastIndexOf('<',begin,breakAtPos):parseText.indexOf('<',begin,breakAtPos); // this assumes that all tags start with '<'
+				begin=parseText.lastIndexOf('<',begin,breakAtPos); // this assumes that all tags start with '<'
 				// parseText.lastIndexOf and indexOf return -1 if pos is out of range.
 				if (begin==-1) return null;
-				final Tag tag=getTagAt(source,begin);
+				final Tag tag=getTagAt(source,begin,false);
 				if (tag!=null && tag.includeInSearch()) return tag;
-			} while (previous ? (begin-=1)>=0 : (begin+=1)<source.end);
+			} while ((begin-=1)>=0);
+		} catch (IndexOutOfBoundsException ex) {
+			// this should never happen during a find previous operation so rethrow it:
+			throw ex;
+		}
+		return null;
+	}
+
+	static final Tag findNextTagUncached(final Source source, final int pos, final int breakAtPos) {
+		// returns null if pos is out of range.
+		try {
+			final ParseText parseText=source.getParseText();
+			int begin=pos;
+			do {
+				begin=parseText.indexOf('<',begin,breakAtPos); // this assumes that all tags start with '<'
+				// parseText.lastIndexOf and indexOf return -1 if pos is out of range.
+				if (begin==-1) return null;
+				final Tag tag=getTagAt(source,begin,false);
+				if (tag!=null && tag.includeInSearch()) return tag;
+			} while ((begin+=1)<source.end);
 		} catch (IndexOutOfBoundsException ex) {
 			// this should only happen when the end of file is reached in the middle of a tag.
 			// we don't have to do anything to handle it as there are no more tags anyway.
@@ -463,26 +444,53 @@ public abstract class Tag extends Segment implements HTMLElementName {
 		return null;
 	}
 
-	static final Tag findPreviousOrNextTag(final Source source, final int pos, final TagType tagType, final boolean previous) {
+	static final Tag findPreviousTag(final Source source, final int pos, final TagType tagType) {
 		// returns null if pos is out of range.
-		if (source.useSpecialTypesCache) return source.cache.findPreviousOrNextTag(pos,tagType,previous);
-		return findPreviousOrNextTagUncached(source,pos,tagType,previous,ParseText.NO_BREAK);
+		if (source.useSpecialTypesCache) return source.cache.findPreviousTag(pos,tagType);
+		return findPreviousTagUncached(source,pos,tagType,ParseText.NO_BREAK);
 	}
 
-	static final Tag findPreviousOrNextTagUncached(final Source source, final int pos, final TagType tagType, final boolean previous, final int breakAtPos) {
+	static final Tag findNextTag(final Source source, final int pos, final TagType tagType) {
 		// returns null if pos is out of range.
-		if (tagType==null) return findPreviousOrNextTagUncached(source,pos,previous,breakAtPos);
+		if (source.useSpecialTypesCache) return source.cache.findNextTag(pos,tagType);
+		return findNextTagUncached(source,pos,tagType,ParseText.NO_BREAK);
+	}
+
+	static final Tag findPreviousTagUncached(final Source source, final int pos, final TagType tagType, final int breakAtPos) {
+		// returns null if pos is out of range.
+		if (tagType==null) return findPreviousTagUncached(source,pos,breakAtPos);
 		final char[] startDelimiterCharArray=tagType.getStartDelimiterCharArray();
 		try {
 			final ParseText parseText=source.getParseText();
 			int begin=pos;
 			do {
-				begin=previous?parseText.lastIndexOf(startDelimiterCharArray,begin,breakAtPos):parseText.indexOf(startDelimiterCharArray,begin,breakAtPos);
+				begin=parseText.lastIndexOf(startDelimiterCharArray,begin,breakAtPos);
 				// parseText.lastIndexOf and indexOf return -1 if pos is out of range.
 				if (begin==-1) return null;
-				final Tag tag=getTagAt(source,begin);
+				final Tag tag=getTagAt(source,begin,false);
 				if (tag!=null && tag.getTagType()==tagType) return tag;
-			} while (previous ? (begin-=1)>=0 : (begin+=1)<source.end);
+			} while ((begin-=1)>=0);
+		} catch (IndexOutOfBoundsException ex) {
+			// this should never happen during a find previous operation so rethrow it:
+			throw ex;
+		}
+		return null;
+	}
+
+	static final Tag findNextTagUncached(final Source source, final int pos, final TagType tagType, final int breakAtPos) {
+		// returns null if pos is out of range.
+		if (tagType==null) return findNextTagUncached(source,pos,breakAtPos);
+		final char[] startDelimiterCharArray=tagType.getStartDelimiterCharArray();
+		try {
+			final ParseText parseText=source.getParseText();
+			int begin=pos;
+			do {
+				begin=parseText.indexOf(startDelimiterCharArray,begin,breakAtPos);
+				// parseText.lastIndexOf and indexOf return -1 if pos is out of range.
+				if (begin==-1) return null;
+				final Tag tag=getTagAt(source,begin,false);
+				if (tag!=null && tag.getTagType()==tagType) return tag;
+			} while ((begin+=1)<source.end);
 		} catch (IndexOutOfBoundsException ex) {
 			// this should only happen when the end of file is reached in the middle of a tag.
 			// we don't have to do anything to handle it as there are no more tags anyway.
@@ -490,22 +498,23 @@ public abstract class Tag extends Segment implements HTMLElementName {
 		return null;
 	}
 
-	static final Tag getTagAt(final Source source, final int pos) {
+	static final Tag getTagAt(final Source source, final int pos, final boolean serverTagOnly) {
 		// returns null if pos is out of range.
 		return source.useAllTypesCache
-			? source.cache.getTagAt(pos)
-			: getTagAtUncached(source,pos);
+			? source.cache.getTagAt(pos,serverTagOnly)
+			: getTagAtUncached(source,pos,serverTagOnly);
 	}
 
-	static final Tag getTagAtUncached(final Source source, final int pos) {
+	static final Tag getTagAtUncached(final Source source, final int pos, final boolean serverTagOnly) {
 		// returns null if pos is out of range.
-		return TagType.getTagAt(source,pos,false);
+		return TagType.getTagAt(source,pos,serverTagOnly,false);
 	}
 
 	static final Tag[] parseAll(final Source source, final boolean assumeNoNestedTags) {
 		int registeredTagCount=0;
 		int registeredStartTagCount=0;
 		final ArrayList list=new ArrayList();
+		source.fullSequentialParseData=new int[1]; // fullSequentialParseData is simply a holder for a single mutable integer. It holds the end position of the last normal tag (ie one that ignores enclosed markup), or MAX_VALUE if we are in a SCRIPT element.
 		if (source.end!=0) {
 			final ParseText parseText=source.getParseText();
 			Tag tag=parseAllFindNextTag(source,parseText,0,assumeNoNestedTags);
@@ -527,7 +536,12 @@ public abstract class Tag extends Segment implements HTMLElementName {
 		source.allTagsArray=allRegisteredTags;
 		source.allTags=Arrays.asList(allRegisteredTags);
 		source.allStartTags=Arrays.asList(allRegisteredStartTags);
-		for (int i=0; i<allRegisteredTags.length; i++) allRegisteredTags[i].allTagsArrayIndex=i;
+		final int lastIndex=allRegisteredTags.length-1;
+		for (int i=0; i<allRegisteredTags.length; i++) {
+			final Tag tag=allRegisteredTags[i];
+			tag.previousTag=i>0 ? allRegisteredTags[i-1] : null;
+			tag.nextTag=i<lastIndex ? allRegisteredTags[i+1] : null;
+		}
 		return allRegisteredTags;
 	}
 
@@ -537,15 +551,15 @@ public abstract class Tag extends Segment implements HTMLElementName {
 			do {
 				begin=parseText.indexOf('<',begin); // this assumes that all tags start with '<'
 				if (begin==-1) return null;
-				final Tag tag=TagType.getTagAt(source,begin,assumeNoNestedTags);
+				final Tag tag=TagType.getTagAt(source,begin,false,assumeNoNestedTags);
 				if (tag!=null) {
 					if (!assumeNoNestedTags) {
 						final TagType tagType=tag.getTagType();
-						if (tag.end>source.endOfLastTagIgnoringEnclosedMarkup
-								&& !tagType.isServerTag()
+						if (tag.end>source.fullSequentialParseData[0]
 								&& tagType!=StartTagType.DOCTYPE_DECLARATION
-								&& tagType!=StartTagType.UNREGISTERED && tagType!=EndTagType.UNREGISTERED)
-							source.endOfLastTagIgnoringEnclosedMarkup=tag.end;
+								&& tagType!=StartTagType.UNREGISTERED && tagType!=EndTagType.UNREGISTERED) {
+							source.fullSequentialParseData[0]=(tagType==StartTagType.NORMAL && tag.name==SCRIPT) ? Integer.MAX_VALUE : tag.end;
+						}
 					}
 					return tag;
 				}
@@ -556,35 +570,4 @@ public abstract class Tag extends Segment implements HTMLElementName {
 		}
 		return null;
 	}
-
-	// delete when deprecated Source.getNextTagIterator method is removed
-	static Iterator getNextTagIterator(final Source source, final int pos) {
-		return new NextTagIterator(source,pos);
-	}
-
-	private static final class NextTagIterator implements Iterator {
-		private Tag nextTag=null;
-
-		public NextTagIterator(final Source source, final int pos) {
-			nextTag=findPreviousOrNextTag(source,pos,false);
-		}
-
-		public boolean hasNext() {
-			return nextTag!=null;
-		}
-
-		public Object next() {
-			final Tag result=nextTag;
-			try {
-				nextTag=findPreviousOrNextTag(result.source,result.begin+1,false);
-			} catch (NullPointerException ex) {
-				throw new NoSuchElementException();
-			}
-			return result;
-		}
-
-		public void remove() {
-			throw new UnsupportedOperationException();
-		}
-	}
 }
