@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.6
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -28,10 +28,13 @@ import java.util.*;
  * This class is the root abstract class common to all tag types, and contains methods to {@linkplain #register() register}
  * and {@linkplain #deregister() deregister} tag types as well as various methods to aid in their implementation.
  * <p>
- * Every tag type is represented by an instance of a class (usually a singleton) that must be a subclass of either 
+ * Every tag type is represented by a singleton instance of a class that must be a subclass of either 
  * {@link StartTagType} or {@link EndTagType}.  These two abstract classes, the only direct descendants of this class,
  * represent the two major classifications under which every tag type exists.
  * <p>
+ * Because all <code>TagType</code> instaces must be singletons, the '<code>==</code>' operator can be used to test for a particular tag type
+ * instead of the <code>equals(Object)</code> method.
+ * <p>
  * The term <i><a name="Predefined">predefined tag type</a></i> refers to any of the tag types defined in this library,
  * including both <a href="#Standard">standard</a> and <a href="#Extended">extended</a> tag types.
  * <p>
@@ -60,8 +63,9 @@ import java.util.*;
  * See the documentation of the <a href="Tag.html#ParsingProcess">tag parsing process</a> for more information
  * on how each tag is identified by the parser.
  * <p>
- * Note that the standard {@linkplain HTMLElementName HTML element names} do not represent different
- * tag <i>types</i>.  All standard HTML tags have a tag type of {@link StartTagType#NORMAL} or {@link EndTagType#NORMAL}.
+ * <a name="Normal"></a>Note that the standard {@linkplain HTMLElementName HTML element names} do not represent different
+ * tag <i>types</i>.  All standard HTML tags have a tag type of {@link StartTagType#NORMAL} or {@link EndTagType#NORMAL},
+ * and are also referred to as <i>normal</i> tags.
  * <p>
  * Apart from the <a href="#Registration">registration related</a> methods, all of the methods in this class and its
  * subclasses relate to the implementation of <a href="#Custom">custom tag types</a> and are not relevant to the majority of users 
@@ -187,6 +191,7 @@ public abstract class TagType {
 	 *   <dd>
 	 *    <table class="bordered" style="margin: 15px" cellspacing="0">
 	 *     <tr><th>Tag Type<th>Start Delimiter
+	 *     <tr><td>{@link MicrosoftTagTypes#DOWNLEVEL_REVEALED_CONDITIONAL_COMMENT}<td><code>&lt;![</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SCRIPT}<td><code>&lt;script</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SHORT}<td><code>&lt;?</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_STANDARD}<td><code>&lt;?php</code>
@@ -240,6 +245,7 @@ public abstract class TagType {
 	 *   <dd>
 	 *    <table class="bordered" style="margin: 15px" cellspacing="0">
 	 *     <tr><th>Tag Type<th>Closing Delimiter
+	 *     <tr><td>{@link MicrosoftTagTypes#DOWNLEVEL_REVEALED_CONDITIONAL_COMMENT}<td><code>]&gt;</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SCRIPT}<td><code>&gt;</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SHORT}<td><code>?&gt;</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_STANDARD}<td><code>?&gt;</code>
@@ -264,13 +270,16 @@ public abstract class TagType {
 	 * Server tags are typically parsed by some process on the web server and substituted with other text or markup before delivery to the
 	 * <a target="_blank" href="http://www.w3.org/TR/html401/conform.html#didx-user_agent">user agent</a>.
 	 * This parser therefore handles them differently to non-server tags in that they can occur at any position in the document
-	 * without regard for the HTML document structure.  As a result they can occur anywhere inside any other tag and vice versa.
-	 * <p>
-	 * To avoid the problem of server tags interfering with the proper parsing of the rest of the document, the
-	 * {@link Segment#ignoreWhenParsing()} method can be called on all server tags found in the document before parsing the non-server tags.
+	 * without regard for the HTML document structure.
+	 * As a result they can occur anywhere inside any other tag, although a non-server tag cannot theoretically occur inside a server tag.
 	 * <p>
 	 * The documentation of the <a href="Tag.html#ParsingProcess">tag parsing process</a> explains in detail 
-	 * how the value of this property affects the recognition of a tag.
+	 * how the value of this property affects the recognition of server tags,
+	 * as well as how the presence of server tags affects the recognition of non-server tags in and around them.
+	 * <p>
+	 * Most XML-style server tags can not be represented as a distinct tag type because they are generally indistinguishable from non-server XML tags.
+	 * See the {@link Segment#ignoreWhenParsing()} method for information about how to prevent such server tags from interfering with the proper parsing
+	 * of the rest of the document.
 	 * <p>
 	 * <dl>
 	 *  <dt>Standard Tag Type Values:</dt>
@@ -295,6 +304,7 @@ public abstract class TagType {
 	 *   <dd>
 	 *    <table class="bordered" style="margin: 15px" cellspacing="0">
 	 *     <tr><th>Tag Type<th>Is Server Tag
+	 *     <tr><td>{@link MicrosoftTagTypes#DOWNLEVEL_REVEALED_CONDITIONAL_COMMENT}<td><code>false</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SCRIPT}<td><code>true</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SHORT}<td><code>true</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_STANDARD}<td><code>true</code>
@@ -347,6 +357,7 @@ public abstract class TagType {
 	 *   <dd>
 	 *    <table class="bordered" style="margin: 15px" cellspacing="0">
 	 *     <tr><th>Tag Type<th>Name Prefix
+	 *     <tr><td>{@link MicrosoftTagTypes#DOWNLEVEL_REVEALED_CONDITIONAL_COMMENT}<td><code>![</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SCRIPT}<td><code>script</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_SHORT}<td><code>?</code>
 	 *     <tr><td>{@link PHPTagTypes#PHP_STANDARD}<td><code>?php</code>
@@ -377,17 +388,26 @@ public abstract class TagType {
 	 * for all <a href="TagType.html#Custom">custom tag types</a>.
 	 * Having this check separated into a different method helps to isolate common code from the code that is unique to each tag type.
 	 * <p>
-	 * In theory, a {@linkplain TagType#isServerTag() server tag} is valid in any position, but a non-server tag is not valid inside another non-server tag.
+	 * In theory, a {@linkplain TagType#isServerTag() server tag} is valid in any position, but a non-server tag is not valid inside any other tag,
+	 * nor inside elements with CDATA content such as {@link HTMLElementName#SCRIPT SCRIPT} and {@link HTMLElementName#STYLE STYLE} elements.
 	 * <p>
 	 * The common implementation of this method always returns <code>true</code> for server tags, but for non-server tags it behaves slightly differently
 	 * depending upon whether or not a {@linkplain Source#fullSequentialParse() full sequential parse} is being peformed.
-	 * If so, it implements the exact theoretical check and rejects a non-server tag if it is inside any other non-server tag.
-	 * If a full sequential parse was not performed (i.e. in <a href="Source.html#ParseOnDemand">parse on demand</a> mode),
-	 * practical constraints do not permit the implementation of the exact theoretical check, and non-server tags are only rejected 
+	 * <p>
+	 * When this method is called during a full sequential parse, the <code>fullSequentialParseData</code> argument contains information
+	 * allowing the exact theoretical check to be performed, rejecting a non-server tag if it is inside any other tag.
+	 * See below for further information about the <code>fullSequentialParseData</code> parameter.
+	 * <p>
+	 * Prior to version 2.6, non-server tags were only rejected if they were located inside other non-server tags.
+	 * The current version rejects non-server tags that are located inside any other tag during a full sequential parse.
+	 * <p>
+	 * When this method is called in <a href="Source.html#ParseOnDemand">parse on demand</a> mode
+	 * (not during a full sequential parse, <code>fullSequentialParseData==null</code>),
+	 * practical constraints prevent the exact theoretical check from being carried out, and non-server tags are only rejected 
 	 * if they are found inside HTML {@linkplain StartTagType#COMMENT comments} or {@linkplain StartTagType#CDATA_SECTION CDATA sections}.
 	 * <p>
 	 * This behaviour is configurable by manipulating the static {@link TagType#getTagTypesIgnoringEnclosedMarkup() TagTypesIgnoringEnclosedMarkup} array
-	 * to determine which tag types can not contain non-server tags.
+	 * to determine which tag types can not contain non-server tags in <a href="Source.html#ParseOnDemand">parse on demand</a> mode.
 	 * The {@linkplain TagType#getTagTypesIgnoringEnclosedMarkup() documentation of this property} contains
 	 * a more detailed analysis of the subject and explains why only the {@linkplain StartTagType#COMMENT comment} and 
 	 * {@linkplain StartTagType#CDATA_SECTION CDATA section} tag types are included by default.
@@ -395,52 +415,126 @@ public abstract class TagType {
 	 * See the documentation of the <a href="Tag.html#ParsingProcess">tag parsing process</a> for more information about how this method fits into the whole tag parsing process.
 	 * <p>
 	 * This method can be overridden in <a href="TagType.html#Custom">custom tag types</a> if the default implementation is unsuitable.
+	 * <p>
+	 * <b>The <code>fullSequentialParseData</code> parameter:</b>
+	 * <p>
+	 * This parameter is used to discard non-server tags that are found inside other tags or inside {@link HTMLElementName#SCRIPT SCRIPT} elements.
+	 * <p>
+	 * In the current version of this library, the <code>fullSequentialParseData</code> argument is either <code>null</code>
+	 * (in <a href="Source.html#ParseOnDemand">parse on demand</a> mode) or an integer array containing only a single entry
+	 * (if a {@linkplain Source#fullSequentialParse() full sequential parse} is being peformed).
+	 * <p>
+	 * The integer contained in the array is the maximum position in the document at which the end of a tag has been found,
+	 * indicating that no non-server tags should be recognised before that position.
+	 * If no tags have yet been encountered, the value of this integer is zero.
+	 * <p>
+	 * If the last tag encountered was the {@linkplain StartTag start tag} of a {@link HTMLElementName#SCRIPT SCRIPT} element,
+	 * the value of this integer is <code>Integer.MAX_VALUE</code>, indicating that no other non-server elements should be recognised until the
+	 * {@linkplain EndTag end tag} of the {@link HTMLElementName#SCRIPT SCRIPT} element is found.
+	 * According to the <a target="_blank" href="http://www.w3.org/TR/html401/types.html#idx-CDATA-1">HTML 4.01 specification section 6.2</a>,
+	 * the first occurrence of the character sequence "<code>&lt;/</code>" terminates the special handling of CDATA within
+	 * {@link HTMLElementName#SCRIPT SCRIPT} and {@link HTMLElementName#STYLE STYLE} elements.
+	 * This library however only terminates the CDATA handling of {@link HTMLElementName#SCRIPT SCRIPT} element content
+	 * when the character sequence "<code>&lt;/script</code>" is detected, in line with the behaviour of the major browsers.
+	 * <p>
+	 * Note that the implicit treatment of {@link HTMLElementName#SCRIPT SCRIPT} element content as CDATA also prevents the recognition of
+	 * {@linkplain StartTagType#COMMENT comments} and explicit {@linkplain StartTagType#CDATA_SECTION CDATA sections} inside script elements. 
+	 * <p>
+	 * Although {@link HTMLElementName#STYLE STYLE} elements should theoretically be treated in the same way as {@link HTMLElementName#SCRIPT SCRIPT} elements,
+	 * the syntax of <a target="_blank" href="http://www.w3.org/Style/CSS/">Cascading Style Sheets</a> (CSS) does not contain any constructs that 
+	 * could be misinterpreted as HTML tags, so there is virtually no need to perform any special checks in this case.
+	 * <p>
+	 * IMPLEMENTATION NOTE: The rationale behind using an integer array to hold this value, rather than a scalar <code>int</code> value,
+	 * is to emulate passing the parameter by reference.
+	 * This value needs to be shared amongst several internal methods during the {@linkplain Source#fullSequentialParse() full sequential parse} process,
+	 * and any one of those methods needs to be able to modify the value and pass it back to the calling method.
+	 * This would normally be implemented by passing the parameter by reference, but because Java does not support this language construct, a container for a
+	 * mutable integer must be passed instead.  
+	 * Because the standard Java library does not provide a class for holding a single mutable integer (the <code>java.lang.Integer</code> class is immutable),
+	 * the easiest container to use, without creating a class especially for this purpose, is an integer array.
+	 * The use of an array does not imply any intention to use more than a single array entry in subsequent versions.
 	 *
 	 * @param source  the {@link Source} document.
 	 * @param pos  the character position in the source document to check.
+	 * @param fullSequentialParseData  an integer array containing data allowing this method to implement a better algorithm when a {@linkplain Source#fullSequentialParse() full sequential parse} is being performed, or <code>null</code> in <a href="Source.html#ParseOnDemand">parse on demand</a> mode.
 	 * @return <code>true</code> if a tag of this type is valid in the specified position of the specified source document, otherwise <code>false</code>.
 	 */
-	protected boolean isValidPosition(final Source source, final int pos) {
+	protected boolean isValidPosition(final Source source, final int pos, final int[] fullSequentialParseData) {
 		if (isServerTag()) return true;
-		if (source.endOfLastTagIgnoringEnclosedMarkup!=-1) {
+		if (fullSequentialParseData!=null) {
 			// use simplified check when doing full sequential parse.  Normally we are only able to check whether a tag is inside specially cached
-			// tag types for efficiency reasons, but during a full sequential parse we can reject a tag if it is inside normal tags as well.
-			return pos>=source.endOfLastTagIgnoringEnclosedMarkup;
+			// tag types for efficiency reasons, but during a full sequential parse we can reject a tag if it is inside any other tag.
+			if (fullSequentialParseData[0]==Integer.MAX_VALUE) { // we are in a SCRIPT element
+				if (this==EndTagType.NORMAL && source.getParseText().containsAt("</script",pos)) {
+					fullSequentialParseData[0]=pos;
+					return true;
+				}
+				return false;
+			}
+			return pos>=fullSequentialParseData[0];
 		}
 		// Use the normal method of checking whether the position is inside a tag of a tag type that ignores enclosed markup:
 		final TagType[] tagTypesIgnoringEnclosedMarkup=getTagTypesIgnoringEnclosedMarkup();
-		for (int i=0; i<tagTypesIgnoringEnclosedMarkup.length; i++)
-			if (tagTypesIgnoringEnclosedMarkup[i].tagEncloses(source,pos)) return false;
+		for (int i=0; i<tagTypesIgnoringEnclosedMarkup.length; i++) {
+			final TagType tagTypeIgnoringEnclosedMarkup=tagTypesIgnoringEnclosedMarkup[i];
+			// If this tag type is a comment, don't bother checking whether it is inside another comment.
+			// See javadocs for getTagTypesIgnoringEnclosedMarkup() for more explanation.
+			// Allowing it might result in multiple comments being recognised with the same end delimiter, but the risk of this occuring in a syntactically invalid document
+			// is outweighed by the benefit of not recursively checking all previous comments in a document, risking stack overflow.
+			if (this==StartTagType.COMMENT && tagTypeIgnoringEnclosedMarkup==StartTagType.COMMENT) continue;
+			if (tagTypeIgnoringEnclosedMarkup.tagEncloses(source,pos)) return false;
+		}
 		return true;
 	}
 
 	/**
-	 * Returns an array of all the tag types inside which the parser ignores all other non-{@linkplain #isServerTag() server} tags
+	 * Returns an array of all the tag types inside which the parser ignores all non-{@linkplain #isServerTag() server} tags
 	 * in <a href="Source.html#ParseOnDemand">parse on demand</a> mode.
 	 * <br />(<a href="TagType.html#ImplementationAssistance">implementation assistance</a> method)
 	 * <p>
 	 * The tag types returned by this property (referred to in the following paragraphs as the "listed types") default to
 	 * {@link StartTagType#COMMENT} and {@link StartTagType#CDATA_SECTION}.
 	 * <p>
-	 * In <a href="Source.html#ParseOnDemand">parse on demand</a> mode,
+	 * This property is used by the default implementation of the {@link #isValidPosition(Source, int pos, int[] fullSequentialParseData) isValidPosition} method
+	 * in <a href="Source.html#ParseOnDemand">parse on demand</a> mode.
+	 * It is not used at all during a {@linkplain Source#fullSequentialParse() full sequential parse}.
+	 * <p>
+	 * In the default implementation of the {@link #isValidPosition(Source, int pos, int[] fullSequentialParseData) isValidPosition} method,
+	 * in <a href="Source.html#ParseOnDemand">parse on demand</a> mode,
 	 * every new non-server tag found by the parser (referred to as a "new tag") undergoes a check to see whether it is enclosed
-	 * by a tag of one of the listed types, including new tags of the listed types themselves.
+	 * by a tag of one of the listed types.
+	 * This includes new tags of the listed types themselves if they are non-server tags.
 	 * The recursive nature of this check means that <i>all</i> tags of the listed types occurring before the new tag must be found 
 	 * by the parser before it can determine whether the new tag should be ignored.
 	 * To mitigate any performance issues arising from this process, the listed types are given special treatment in the tag cache.
 	 * This dramatically decreases the time taken to search on these tag types, so adding a tag type to this array that 
 	 * is easily recognised and occurs infrequently only results in a small degradation in overall performance.
 	 * <p>
-	 * Theoretically, non-server tags appearing inside <i>any</i> other non-server tag should be ignored.
-	 * One situation where a tag can legitimately contain a sequence of characters that resembles a tag,
-	 * which shouldn't be recognised as a tag by the parser, is within an attribute value.
+	 * A special exception to the algorithm described above applies to {@link StartTagType#COMMENT COMMENT} tags.
+	 * The default implementation of the {@link #isValidPosition(Source,int,int[]) isValidPosition} method
+	 * does not check whether a {@link StartTagType#COMMENT COMMENT} tag is inside another {@link StartTagType#COMMENT COMMENT} tag,
+	 * as this should never happen in a syntactically correct document (the characters '<code>--</code>' should not occur inside a comment).
+	 * Skipping this check also avoids the need to recursively check every {@link StartTagType#COMMENT COMMENT} tag back to the start of the document,
+	 * which has the potential to cause a stack overflow in a large document containing lots of comments.
+	 * <p>
+	 * Theoretically, non-server tags appearing inside any other tag should be ignored, which is how the parser behaves during a
+	 * {@linkplain Source#fullSequentialParse() full sequential parse}.
+	 * <p>
+	 * Server tags in particular very often contain other "tags" that should not be recognised as tags by the parser.
+	 * If this behaviour is required in <a href="Source.html#ParseOnDemand">parse on demand</a>, the tag type of each server tag that might be found
+	 * in the source documents can be added to this property using the static {@link #setTagTypesIgnoringEnclosedMarkup(TagType[])} method.
+	 * For example, the following command would prevent non-server tags from being recognised inside {@linkplain PHPTagTypes#PHP_STANDARD standard PHP} tags,
+	 * as well as the default {@linkplain StartTagType#COMMENT comment} and {@linkplain StartTagType#CDATA_SECTION CDATA section} tags:
+	 * <p>
+	 * <blockquote><code>TagType.setTagTypesIgnoringEnclosedMarkup(new TagType[] {PHPTagTypes.PHP_STANDARD, StartTagType.COMMENT, StartTagType.CDATA_SECTION});</code></blockquote>
+	 * <p>
+	 * The only situation where a non-server tag can legitimately contain a sequence of characters that resembles a tag is within an attribute value.
 	 * The <a target="_blank" href="http://www.w3.org/TR/html401/charset.html#h-5.3.2">HTML 4.01 specification section 5.3.2</a>
 	 * specifically allows the presence of '<code>&lt;</code>' and '<code>&gt;</code>' characters within attribute values.
-	 * A common occurrence of this is in <a target="_blank" href="http://www.w3.org/TR/html401/interact/scripts.html#events">event</a>
-	 * attributes such as <code><a target="_blank" href="http://www.w3.org/TR/html401/interact/scripts.html#adef-onclick">onclick</a></code>,
-	 * which contain scripts that often dynamically load new HTML into the document
-	 * (see the file <code><a target="_blank" href="../../../../../../../samples/data/Test.html#TagInsideTag">samples/data/Test.html</a></code> for an example).
-	 * <p>
+	 * A common occurrence of this is in <a target="_blank" href="http://www.w3.org/TR/html401/interact/scripts.html#events">event</a> attributes containing scripts,
+	 * such as the <code><a target="_blank" href="http://www.w3.org/TR/html401/interact/scripts.html#adef-onclick">onclick</a></code> attribute.
+	 * There is no way of preventing such "tags" from being recognised in <a href="Source.html#ParseOnDemand">parse on demand</a> mode, as adding
+	 * {@link StartTagType#NORMAL} to this property as a listed type would be far too inefficient.
 	 * Performing a {@linkplain Source#fullSequentialParse() full sequential parse} of the source document prevents these attribute values from being
 	 * recognised as tags, but can be very expensive if only a few tags in the document need to be parsed.
 	 * The penalty of not parsing every tag in the document is that the exactness of this check is compromised, but in practical terms the difference is inconsequential.
@@ -452,17 +546,15 @@ public abstract class TagType {
 	 * (see the XML 1.0 specification section <a target="_blank" href="http://www.w3.org/TR/REC-xml#CleanAttrVals">3.1</a>),
 	 * so the situation should never arise that a tag is found inside another tag unless one of them is a
 	 * {@linkplain #isServerTag() server tag}.
-	 * <p>
-	 * This method is called from the default implementation of the {@link #isValidPosition(Source, int pos)} method.
 	 *
-	 * @return an array of all the tag types inside which the parser ignores all other non-{@linkplain #isServerTag() server} tags.
+	 * @return an array of all the tag types inside which the parser ignores all non-{@linkplain #isServerTag() server} tags.
 	 */
 	public static final TagType[] getTagTypesIgnoringEnclosedMarkup() {
 		return TagTypesIgnoringEnclosedMarkup.array;
 	}
 
 	/**
-	 * Sets the tag types inside which the parser ignores all other non-{@linkplain #isServerTag() server} tags.
+	 * Sets the tag types inside which the parser ignores all non-{@linkplain #isServerTag() server} tags.
 	 * <br />(<a href="TagType.html#ImplementationAssistance">implementation assistance</a> method)
 	 * <p>
 	 * See {@link #getTagTypesIgnoringEnclosedMarkup()} for the documentation of this property.
@@ -488,7 +580,7 @@ public abstract class TagType {
 	 * Tag types with lower precedence will be considered if this method returns <code>null</code>.
 	 * <p>
 	 * This method is only called after a successful check of the tag's position, i.e.
-	 * {@link #isValidPosition(Source,int) isValidPosition(source,pos)}<code>==true</code>.
+	 * {@link #isValidPosition(Source,int,int[]) isValidPosition(source,pos,fullSequentialParseData)}<code>==true</code>.
 	 * <p>
 	 * The {@link StartTagTypeGenericImplementation} and {@link EndTagTypeGenericImplementation} subclasses provide default
 	 * implementations of this method that allow the use of much simpler <a href="TagType.html#Property">properties</a> and
@@ -507,7 +599,7 @@ public abstract class TagType {
 	 * This is logically equivalent to <code>source.</code>{@link Source#findEnclosingTag(int,TagType) findEnclosingTag(pos,this)}<code>!=null</code>,
 	 * but is safe to use within other implementation methods without the risk of causing an infinite recursion.
 	 * <p>
-	 * This method is called by the {@link TagType} implementation of {@link #isValidPosition(Source, int pos)}.
+	 * This method is called from the default implementation of the {@link #isValidPosition(Source, int pos, int[] fullSequentialParseData)} method.
 	 *
 	 * @param source  the {@link Source} document.
 	 * @param pos  the character position in the source document to check.
@@ -527,28 +619,23 @@ public abstract class TagType {
 		return getDescription();
 	}
 
-	static final Tag getTagAt(final Source source, final int pos, final boolean assumeNoNestedTags) {
+	static final Tag getTagAt(final Source source, final int pos, final boolean serverTagOnly, final boolean assumeNoNestedTags) {
 		final TagTypeRegister.ProspectiveTagTypeIterator prospectiveTagTypeIterator=new TagTypeRegister.ProspectiveTagTypeIterator(source,pos);
 		// prospectiveTagTypeIterator is empty if pos is out of range.
 		while (prospectiveTagTypeIterator.hasNext()) {
 			final TagType tagType=prospectiveTagTypeIterator.getNextTagType();
-			if (assumeNoNestedTags || tagType.isValidPosition(source,pos)) {
-				try {
-					final Tag tag=tagType.constructTagAt(source,pos);
-					if (tag!=null) return tag;
-				} catch (IndexOutOfBoundsException ex) {
-					if (source.isLoggingEnabled()) source.log(source.getRowColumnVector(pos).appendTo(new StringBuffer(200).append("Tag at ")).append(" not recognised as type '").append(tagType.getDescription()).append("' because it has no end delimiter").toString());
-				}
+			if (serverTagOnly && !tagType.isServerTag()) continue;
+			if (!assumeNoNestedTags && !tagType.isValidPosition(source,pos,source.fullSequentialParseData)) continue;
+			try {
+				final Tag tag=tagType.constructTagAt(source,pos);
+				if (tag!=null) return tag;
+			} catch (IndexOutOfBoundsException ex) {
+				if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(pos).appendTo(new StringBuffer(200).append("Tag at ")).append(" not recognised as type '").append(tagType.getDescription()).append("' because it has no end delimiter").toString());
 			}
 		}
 		return null;
 	}
 
-	final String getNamePrefixForTagConstant() {
-		// this method is only used in deprecated constants and will eventually be removed
-		return getNamePrefix();
-	}
-	
 	final char[] getStartDelimiterCharArray() {
 		return startDelimiterCharArray;
 	}
