@@ -1,5 +1,5 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.5
+// Version 2.6
 // Copyright (C) 2007 Martin Jericho
 // http://jerichohtml.sourceforge.net/
 //
@@ -53,7 +53,7 @@ import java.net.*;
  * {@linkplain Element#getContent() content} of each {@link Element} is to be excluded from the output.
  * <p>
  * All tags that are not <a href="TagType.html#Normal">normal</a> tags, such as {@linkplain TagType#isServerTag() server tags},
- * {@linkplain StartTagType#COMMENT comments} etc., are removed from the output without adding whitespace to the output.
+ * {@linkplain StartTagType#COMMENT comments} etc., are removed from the output without adding white space to the output.
  * <p>
  * Note that segments on which the {@link Segment#ignoreWhenParsing()} method has been called are treated as text rather than markup,
  * resulting in their inclusion in the output.
@@ -74,10 +74,12 @@ import java.net.*;
  */
 public class TextExtractor implements CharStreamSource {
 	private final Segment segment;
-	private boolean convertNonBreakingSpaces=true;
+	private boolean convertNonBreakingSpaces=Config.ConvertNonBreakingSpaces;
 	private boolean includeAttributes=false;
 	private boolean excludeNonHTMLElements=false;
 
+	private static Map map=initDefaultAttributeIncludeCheckerMap(); // maps each possibly included attribute name to an AttributeIncludeChecker instance
+
 	/**
 	 * Constructs a new <code>TextExtractor</code> based on the specified {@link Segment}.
 	 * @param segment  the segment from which the text will be extracted.
@@ -106,7 +108,7 @@ public class TextExtractor implements CharStreamSource {
 	/**
 	 * Sets whether non-breaking space ({@link CharacterEntityReference#_nbsp &amp;nbsp;}) character entity references are converted to spaces.
 	 * <p>
-	 * The default value is <code>true</code>.
+	 * The default value is that of the static {@link Config#ConvertNonBreakingSpaces} property at the time the <code>TextExtractor</code> is instantiated.
 	 *
 	 * @param convertNonBreakingSpaces  specifies whether non-breaking space ({@link CharacterEntityReference#_nbsp &amp;nbsp;}) character entity references are converted to spaces.
 	 * @return this <code>TextExtractor</code> instance, allowing multiple property setting methods to be chained in a single statement. 
@@ -129,22 +131,14 @@ public class TextExtractor implements CharStreamSource {
 	}
 
 	/**
-	 * Sets whether the values of 
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-title">title</a>,
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/objects.html#adef-alt">alt</a>,
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#adef-label-OPTION">label</a>,
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/tables.html#adef-summary">summary</a>, and
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-content">content</a>
-	 * attributes of {@linkplain StartTagType#NORMAL normal} tags are to be included in the output.
+	 * Sets whether any attribute values are included in the output.
 	 * <p>
-	 * The value of a <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-content">content</a> attribute is
-	 * only included if a <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-name-META">name</a> attribute is also present,
-	 * as the content attribute of a {@link HTMLElementName#META META} tag only contains human readable text if the name attribute is used as opposed to an
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-http-equiv">http-equiv</a> attribute.
+	 * If the value of this property is <code>true</code>, then each attribute still has to match the conditions implemented in the
+	 * {@link #includeAttribute(StartTag,Attribute)} method in order for its value to be included in the output.
 	 * <p>
 	 * The default value is <code>false</code>.
 	 *
-	 * @param includeAttributes  specifies whether the attribute values are included in the output.
+	 * @param includeAttributes  specifies whether any attribute values are included in the output.
 	 * @return this <code>TextExtractor</code> instance, allowing multiple property setting methods to be chained in a single statement. 
 	 * @see #getIncludeAttributes()
 	 */
@@ -154,22 +148,66 @@ public class TextExtractor implements CharStreamSource {
 	}
 	
 	/**
-	 * Indicates whether the values of 
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-title">title</a>,
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/objects.html#adef-alt">alt</a>,
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#adef-label-OPTION">label</a>, and
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/tables.html#adef-summary">summary</a>, and
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-content">content</a>
-	 * attributes of {@linkplain StartTagType#NORMAL normal} tags are to be included in the output.
+	 * Indicates whether any attribute values are included in the output.
 	 * <p>
 	 * See the {@link #setIncludeAttributes(boolean)} method for a full description of this property.
 	 * 
-	 * @return <code>true</code> if the attribute values are to be included in the output, otherwise <code>false</code>.
+	 * @return <code>true</code> if any attribute values are included in the output, otherwise <code>false</code>.
 	 */
 	public boolean getIncludeAttributes() {
 		return includeAttributes;
 	}
 
+	/**
+	 * Indicates whether the value of the specified {@linkplain Attribute attribute} in the specified {@linkplain StartTag start tag} is included in the output.
+	 * <p>
+	 * This method is ignored if the {@link #setIncludeAttributes(boolean) IncludeAttributes} property is set to <code>false</code>, in which case
+	 * no attribute values are included in the output.
+	 * <p>
+	 * If the {@link #setIncludeAttributes(boolean) IncludeAttributes} property is set to <code>true</code>, every attribute of every
+	 * start tag encountered in the segment is checked using this method to determine whether the value of the attribute should be included in the output.
+	 * <p>
+	 * The default implementation of this method returns <code>true</code> if the {@linkplain Attribute#getName() name} of the specified {@linkplain Attribute attribute}
+	 * is one of
+	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-title">title</a>,
+	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/objects.html#adef-alt">alt</a>,
+	 * <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#adef-label-OPTION">label</a>,
+	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/tables.html#adef-summary">summary</a>,
+	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-content">content</a>*, or
+	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/links.html#adef-href">href</a>,
+	 * but the method can be overridden in a subclass to perform a check of arbitrary complexity on each attribute.
+	 * <p>
+	 * * The value of a <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-content">content</a> attribute is only included if a 
+	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-name-META">name</a> attribute is also present in the specified start tag,
+	 * as the content attribute of a {@link HTMLElementName#META META} tag only contains human readable text if the name attribute is used as opposed to an
+	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-http-equiv">http-equiv</a> attribute.
+	 * <p>
+	 * <dl>
+	 *  <dt>Example:</dt>
+	 *  <dd>
+	 *   To include only the value of <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-title">title</a> and
+	 *   <a target="_blank" href="http://www.w3.org/TR/html401/struct/objects.html#adef-alt">alt</a> attributes:<br /><br />
+	 *   <code>
+	 *    final Set includeAttributeNames=new HashSet(Arrays.asList(new String[] {"title","alt"}));<br />
+	 *    TextExtractor textExtractor=new TextExtractor(segment) {<br />
+	 *    &nbsp; &nbsp; public boolean includeAttribute(StartTag startTag, Attribute attribute) {<br />
+	 *    &nbsp; &nbsp; &nbsp; &nbsp; return includeAttributeNames.contains(attribute.getKey());<br />
+	 *    &nbsp; &nbsp; }<br />
+	 *    };<br />
+	 *    textExtractor.setIncludeAttributes(true);<br />
+	 *    String extractedText=textExtractor.toString();
+	 *   </code>
+	 *  </dd>
+	 * </dl>
+	 * @param startTag  the start tag of the element to check for inclusion.
+	 * @return <true> if the text inside the {@link Element} of the specified start tag should be excluded from the output, otherwise <code>false</code>.
+	 */
+	public boolean includeAttribute(final StartTag startTag, final Attribute attribute) {
+		AttributeIncludeChecker attributeIncludeChecker=(AttributeIncludeChecker)map.get(attribute.getKey());
+		if (attributeIncludeChecker==null) return false;
+		return attributeIncludeChecker.includeAttribute(startTag,attribute);
+	}
+
 	/**
 	 * Sets whether the content of <a href="HTMLElements.html#NonHTMLElement">non-HTML elements</a> is excluded from the output.
 	 * <p>
@@ -228,7 +266,35 @@ public class TextExtractor implements CharStreamSource {
 	public boolean excludeElement(final StartTag startTag) {
 		return false;
 	}
-	
+
+	private static interface AttributeIncludeChecker {
+		boolean includeAttribute(final StartTag startTag, final Attribute attribute);
+	}
+
+	private static AttributeIncludeChecker ALWAYS_INCLUDE=new AttributeIncludeChecker() {
+		public boolean includeAttribute(final StartTag startTag, final Attribute attribute) {
+			return true;
+		}
+	};
+
+	private static AttributeIncludeChecker INCLUDE_IF_NAME_ATTRIBUTE_PRESENT=new AttributeIncludeChecker() {
+		public boolean includeAttribute(final StartTag startTag, final Attribute attribute) {
+			return startTag.getAttributes().get("name")!=null;
+		}
+	};
+
+	private static Map initDefaultAttributeIncludeCheckerMap() {
+		Map map=new HashMap();
+		map.put("title",ALWAYS_INCLUDE); // add title attribute
+		map.put("alt",ALWAYS_INCLUDE); // add alt attribute (APPLET, AREA, IMG and INPUT elements)
+		map.put("label",ALWAYS_INCLUDE); // add label attribute (OPTION and OPTGROUP elements)
+		map.put("summary",ALWAYS_INCLUDE); // add summary attribute (TABLE element)
+		map.put("content",INCLUDE_IF_NAME_ATTRIBUTE_PRESENT); // add content attribute (META element)
+		map.put("href",ALWAYS_INCLUDE); // add href attribute (A, AREA and LINK elements)
+		// don't bother with the prompt attribute from the ININDEX element as the element is deprecated and very rarely used.
+		return map;
+	}
+
 	/**
 	 * This class does the actual work, but is first passed final copies of all the parameters for efficiency.
 	 * Note at present this is not implemented in a memory-efficient manner.
@@ -249,46 +315,42 @@ public class TextExtractor implements CharStreamSource {
 			this.includeAttributes=includeAttributes;
 			this.excludeNonHTMLElements=excludeNonHTMLElements;
 		}
-		
+
 		public String toString() {
 			final StringBuffer sb=new StringBuffer(segment.length());
-			int textBegin=segment.begin;
-			for (final Iterator i=segment.findAllTags().iterator(); i.hasNext();) {
-				final Tag tag=(Tag)i.next();
-				final int textEnd=tag.begin;
-				if (textEnd<textBegin) continue;
-				while (textBegin<textEnd) sb.append(source.charAt(textBegin++));
-				if (tag.getTagType()==StartTagType.NORMAL) {
-					final StartTag startTag=(StartTag)tag;
-					if (tag.name==HTMLElementName.SCRIPT || tag.name==HTMLElementName.STYLE || excludeElement(startTag) || (excludeNonHTMLElements && !HTMLElements.getElementNames().contains(tag.name))) {
-						textBegin=startTag.getElement().end;
+			for (NodeIterator nodeIterator=new NodeIterator(segment); nodeIterator.hasNext();) {
+				Segment nodeSegment=nodeIterator.getNextSegment();
+				if (nodeSegment instanceof Tag) {
+					final Tag tag=(Tag)nodeSegment;
+					if (tag.getTagType().isServerTag()) {
+						// elementContainsMarkup should be made into a TagType property one day.
+						// for the time being assume all server element content is code, although this is not true for some Mason elements.
+						final boolean elementContainsMarkup=false;
+						if (!elementContainsMarkup) {
+							final Element element=tag.getElement();
+							if (element!=null && element.getEnd()>tag.getEnd()) nodeIterator.skipToPos(element.getEnd());
+						}
 						continue;
 					}
-					if (includeAttributes) {
-						final Attributes attributes=startTag.getAttributes();
-						// add title attribute:
-						final Attribute titleAttribute=attributes.get("title");
-						if (titleAttribute!=null) sb.append(' ').append(titleAttribute.getValueSegment()).append(' ');
-						// add alt attribute (APPLET, AREA, IMG and INPUT elements):
-						final Attribute altAttribute=attributes.get("alt");
-						if (altAttribute!=null) sb.append(' ').append(altAttribute.getValueSegment()).append(' ');
-						// add label attribute (OPTION and OPTGROUP elements):
-						final Attribute labelAttribute=attributes.get("label");
-						if (labelAttribute!=null) sb.append(' ').append(labelAttribute.getValueSegment()).append(' ');
-						// add summary attribute (TABLE element):
-						final Attribute summaryAttribute=attributes.get("summary");
-						if (summaryAttribute!=null) sb.append(' ').append(summaryAttribute.getValueSegment()).append(' ');
-						// add content attribute (META element):
-						final Attribute contentAttribute=attributes.get("content");
-						if (contentAttribute!=null && attributes.get("name")!=null) sb.append(' ').append(contentAttribute.getValueSegment()).append(' ');
-						// don't bother with the prompt attribute from the ININDEX element as the element is deprecated and very rarely used.
+					if (tag.getTagType()==StartTagType.NORMAL) {
+						final StartTag startTag=(StartTag)tag;
+						if (tag.name==HTMLElementName.SCRIPT || tag.name==HTMLElementName.STYLE || excludeElement(startTag) || (excludeNonHTMLElements && !HTMLElements.getElementNames().contains(tag.name))) {
+							nodeIterator.skipToPos(startTag.getElement().getEnd());
+							continue;
+						}
+						if (includeAttributes) {
+							for (Iterator attributeIterator=startTag.getAttributes().iterator(); attributeIterator.hasNext();) {
+								Attribute attribute=(Attribute)attributeIterator.next();
+								if (includeAttribute(startTag,attribute)) sb.append(' ').append(attribute.getValueSegment()).append(' ');
+							}
+						}
 					}
+					// Treat both start and end tags not belonging to inline-level elements as whitespace:
+					if (tag.getName()==HTMLElementName.BR || !HTMLElements.getInlineLevelElementNames().contains(tag.getName())) sb.append(' ');
+				} else {
+					Util.appendTo(sb,nodeSegment);
 				}
-				// Treat both start and end tags not belonging to inline-level elements as whitespace:
-				if (tag.getName()==HTMLElementName.BR || !HTMLElements.getInlineLevelElementNames().contains(tag.getName())) sb.append(' ');
-				textBegin=tag.end;
 			}
-			while (textBegin<segment.end) sb.append(source.charAt(textBegin++));
 			final String decodedText=CharacterReference.decodeCollapseWhiteSpace(sb,convertNonBreakingSpaces);
 			return decodedText;
 		}
